#!/bin/sh

print_usage () {
  echo "Usage: $0 [X:|image] path/to/loader option path/to/modules..."
  echo "       where X: is a valid floppy drive on your computer"
  echo "       where image is any file name"
  exit 1
}

# Try to guess the format of the mtoolsrc file
init_mtoolsrc () {
  # Don't change anything if mtoolsrc already exists
  [ -f $MTOOLSRC ] && return
  
  echo "drive u: file=\"$IMG_FNAME\" 1.44M filter" > $MTOOLSRC

  if mdir U: > /dev/null 2>&1 ; then : ; else
    rm -f $MTOOLSRC
    echo "drive u: file=\"$IMG_FNAME\" 1.44M" > $MTOOLSRC
    if mdir U: > /dev/null 2>&1 ; then : ; else
      rm -f $MTOOLSRC
      echo "drive u: file=\"$IMG_FNAME\"" > $MTOOLSRC
      mdir U: > /dev/null 2>&1
    fi
  fi

}

PATH=$PATH:/sbin:/usr/sbin:/usr/local/sbin
export PATH

MTOOLSRC=mtoolsrc
export MTOOLSRC

# Redefined variables
FLOPPY_DRIVE=A:
IMG_FNAME=fd.img

[ "$#" -lt 3 ] && print_usage

dest="$1" ; shift
loader_fname="$1" ; shift
options="$1" ; shift
modules="$*"

# Init destination disk
case x$dest in
  x*:)
    drive=$dest
    IMG_FNAME=$dest
    FLOPPY_DRIVE=$dest
    ;;
  x*) 
    drive=U:
    IMG_FNAME=$dest
    ;;
esac

init_mtoolsrc

# Copy the loader
mmd $drive/system
mcopy -bo $loader_fname $drive/system/`basename $loader_fname`

# Generate the menu.txt file
mmd $drive/modules
rm -f menu.txt
cat <<EOF > menu.txt
timeout 0 
default 0
title  the Kid Operating System (Wolfgang Release)
root   (fd0)
kernel /system/`basename $loader_fname` $options
EOF

# Copy the modules
for f in $modules ; do
    if [ ! -f $f ] ; then
	echo "ERROR: module $f not correctly compiled in."
	exit 1
    fi
    if ! mcopy -bo $f $drive/modules/`basename $f` ; then
	echo "ERROR: module $f could not be transferred to floppy."
	exit 1
    fi
    echo module /modules/`basename $f` >> menu.txt
done

# Transfers the menu.txt file to floppy
mcopy -bo menu.txt $drive/boot/grub/
