#!/bin/sh

print_usage () {
  echo "Usage: $0 [X:|image]"
  echo "       where X: is a valid floppy drive on your computer"
  echo "       where image is any file name"
  exit 1
}

grub_dirs_common="/usr/local/share/grub/i386-freebsd /usr/local/share/grub/i386-pc /usr/share/grub/i386-pc /usr/lib/grub/i386-pc /usr/local/grub /usr/local/src/grub-0.5.94 $HOME/share/grub/i386-pc/"
sbin_grub_path="/usr/local/sbin /usr/sbin /sbin $HOME/sbin"

PATH=$PATH:/sbin:/usr/sbin:/usr/local/sbin
export PATH

MTOOLSRC=/tmp/mtoolsrc.grub
export MTOOLSRC

# Redefined variables
FLOPPY_DRIVE=A:
IMG_FNAME=fd.img

##
## Format disk image
##
init_image () {
  echo "Initialize disk image $IMG_FILE..."
  if [ ! -f $IMG_FNAME ] ; then
    dd if=/dev/zero of=$IMG_FNAME bs=18k count=80 1>/dev/null 2>&1
  fi

  rm -f $MTOOLSRC
  echo "drive u: file=\"$IMG_FNAME\" 1.44M filter" > $MTOOLSRC

  if mformat U: ; then : ; else
    rm -f $MTOOLSRC
    echo "drive u: file=\"$IMG_FNAME\" 1.44M" > $MTOOLSRC
    if mformat U: ; then : ; else
      rm -f $MTOOLSRC
      echo "drive u: file=\"$IMG_FNAME\"" > $MTOOLSRC
      mformat U:
    fi
  fi
}


##
## Format (real) floppy disk
##
init_floppy () {
  echo "Formatting floppy..."
  mformat $FLOPPY_DRIVE || exit 1
}


lookup_grub () {
  # Look for a correct GRUBDIR
  for d in $grub_dirs_common ; do
    if [ -d $d ] ; then
      GRUBDIR=$d
      break
    fi
  done

  # Look for a correct sbin/grub
  for d in $sbin_grub_path ; do
    if [ -x $d/grub ] ; then
      SBIN_GRUB=$d/grub
      break
    fi
  done

  if [ -d "$GRUBDIR" -a -x "$SBIN_GRUB" ] ; then : ; else
    echo "Couldn't find a correct grub installation."
    exit 1
  fi
}

##
## setup_disk [drive]
## => setup disk directory structure / copy files
##
setup_disk () {
  echo "Setup destination disk..."

  mmd $1/boot
  mmd $1/boot/grub

  if [ -d $GRUBDIR/stage1 ] ; then
    mcopy $GRUBDIR/stage1/stage1 $1/boot/grub/
    mcopy $GRUBDIR/stage2/stage2 $1/boot/grub/
  else
    mcopy $GRUBDIR/stage1 $1/boot/grub/
    mcopy $GRUBDIR/stage2 $1/boot/grub/
  fi

  $SBIN_GRUB --batch <<EOT 1>/dev/null 2>/dev/null || exit 1
device (fd0) $IMG_FNAME
install (fd0)/boot/grub/stage1 (fd0) (fd0)/boot/grub/stage2 p (fd0)/boot/grub/menu.txt
quit
EOT
}



#################################################
## Real start
##
[ "$#" -lt 1 ] && print_usage

lookup_grub

dest="$1" ; shift

# Init destination disk
case x$dest in
  x*:)
    drive=$dest
    IMG_FNAME=$dest
    FLOPPY_DRIVE=$dest
    init_floppy
    ;;
  x*) 
    drive=U:
    IMG_FNAME=$dest
    init_image
    ;;
esac

# Create directory structure
setup_disk $drive
