/*
 * Copyright (C) 2000
 * http://kos.enix.org
 *
 * Multiboot header 
 *
 * @(#) $Id: boot.S,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */		

#define ASM	1
#include <boot/multiboot.h>
	
	.globl	start, _start

	.section .text
	
	/* Align 32 bits boundary.  */
	.align	4
	
	/* Multiboot header.  */
multiboot_header:
	/* magic */
	.long	MULTIBOOT_HEADER_MAGIC
	/* flags */
	.long	MULTIBOOT_HEADER_FLAGS
	/* checksum */
	.long	-(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	/* header_addr */
	.long	multiboot_header
	/* load_addr */
	.long	_start
	/* load_end_addr */
	.long	_edata
	/* bss_end_addr */
	.long	_end
	/* entry_addr */
	.long	multiboot_entry

	.text
start:
_start:	
multiboot_entry:
	/* Initialize the stack pointer.  */
	movl	$(stack + STACK_SIZE), %ebp
	movl    %ebp, %esp

	/* Reset EFLAGS.  */
	pushl	$0
	popf

	/* Sauvegarde de l'etat du registre eax, eventuellement passe
	par le multiboot loader */
	movl %eax, %ecx

	/* 1. Chargement de la GDT/IDT */
_load_mdt:
	lidt idtr
	lgdt gdtr               

	/* 2. Initialisation des segments */
	movl $0x10,%eax 
	movw %ax,%ds
	movw %ax,%es
	movw %ax,%fs
	movw %ax,%gs
		
	ljmp $0x8, $reset_cs
reset_cs:

call_main:
	/* Push the pointer to the Multiboot information structure.  */
	pushl	%ebx
	/* Push the magic value.  */
	pushl	%ecx

	/* Now enter the C main function...  */
	call	EXT_C(cmain)

	/* Halt.  */
	pushl	$halt_message

	
loop:	hlt
	jmp	loop

halt_message:
	.asciz	"Halted."

.align 4
gdtr:                           /* Le registre GDTR */
	.word gdtsize
	.long _gdt

.align 8
_gdt:
.quad 0x0000000000000000  /* NULL selector */
.quad 0x00CF9B000000FFFF  /* code, PL 0, size 4 Go */
.quad 0x00CF93000000FFFF  /* data, PL 0, size 4 Go */
.quad 0x00CFFB000000FFFF  /* code, PL 3, size 4 Go */
.quad 0x00CFF3000000FFFF  /* data, PL 3, size 4 Go */
.equ gdtsize, . - _gdt    /* Taille GDT = la ou on est ( . ) - debut gdt
			     (_gdt) */

.align 4
idtr:
	.word   0  /* idt limit=0 */
	.long   0  /* idt base=0L */


/* Our stack area.  */
.comm	stack, STACK_SIZE
