/*
 * Copyright (C) 2000
 * http://kos.enix.org
 *
 * Dump info given by GRUB
 *
 * @(#) $Id: dump_multiboot.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <multiboot.h>
#include <macros.h>
#include <debug/debug.h>
#include <libcrt/stdio.h>

/* Check if bit is set in flags */ 
#define CHECK_FLAG(flags,bit)	((flags) & (1 << (bit)))

void dump_multiboot_info(multiboot_info_t *mbi) 
{
  
  // Show Flags
    DEBUG_PRINT("flags = 0x%x\n", (int) mbi->flags);

    // Memory size info
  if (CHECK_FLAG (mbi->flags, 0))
    DEBUG_PRINT("mem_lower = %dKB, mem_upper = %dKB\n",
		(int)mbi->mem_lower, (int)mbi->mem_upper);

  // Bootdevice info
  if(CHECK_FLAG(mbi->flags, 1))
    DEBUG_PRINT("boot_device = 0x%x\n", (unsigned)mbi->boot_device);
  
  // Command line info
  if(CHECK_FLAG(mbi->flags, 2))
    DEBUG_PRINT("cmdline = %s\n", (char *)mbi->cmdline);
  
  // Modules infos
  if(CHECK_FLAG(mbi->flags, 3)) {
    module_t *mod;
    unsigned int i;

    DEBUG_PRINT("mods_count = %d, mods_addr = 0x%x\n",
		(int) mbi->mods_count, (int) mbi->mods_addr);

    for(i = 0, mod = (module_t *)mbi->mods_addr; 
	i < mbi->mods_count; 
	i++, mod++)
      DEBUG_PRINT(" mod_start = 0x%x, mod_end = 0x%x, string = %s\n", 
		  (unsigned)mod->mod_start, (unsigned)mod->mod_end,
		  (char *)mod->string);
  }
  
  // bits 4 and 5 are mutually exclusive
  if(CHECK_FLAG(mbi->flags, 4) && CHECK_FLAG(mbi->flags, 5)) {
    DEBUG_PRINT("Bits 4 and 5 are set : error\n");
    DEBUG_PRINT("System halted");
    for (; ;);
    
  }
  
  // a.out kernel symbol table 
  if(CHECK_FLAG(mbi->flags, 4)) {
    aout_symbol_table_t *aout_sym = &(mbi->u.aout_sym);
    UNUSED(aout_sym);
    
    DEBUG_PRINT("aout_symbol_table: tabsize = 0x%x, strsize = 0x%x, addr = 0x%x\n",
		(unsigned)aout_sym->tabsize, (unsigned)aout_sym->strsize,
		(unsigned)aout_sym->addr);
  }
  
  // elf kernel symbol table
  if(CHECK_FLAG(mbi->flags, 5)) {
    elf_section_header_table_t *elf_sec = &(mbi->u.elf_sec);
    UNUSED(elf_sec);
    
    DEBUG_PRINT("elf_sec : num = %d, size = 0x%x, addr = 0x%x, shndx = 0x%x\n",
		(int)elf_sec->num, (unsigned)elf_sec->size,
		(unsigned)elf_sec->addr, (int)elf_sec->shndx);
  }
  
  // Memory map given by the BIOS
  if(CHECK_FLAG(mbi->flags, 6)) {
    memory_map_t *mmap;
    
    DEBUG_PRINT("mmap_addr = 0x%x, mmap_length = 0x%x\n",
		(unsigned) mbi->mmap_addr, (unsigned) mbi->mmap_length);
    for(mmap = (memory_map_t *) mbi->mmap_addr;
	(unsigned long) mmap < mbi->mmap_addr+mbi->mmap_length;
	mmap = (memory_map_t *) ((unsigned long) mmap + mmap->size
				 + sizeof(mmap->size)))
      DEBUG_PRINT(" size = 0x%x, base_addr = 0x%x%x, length = 0x%x%x, type = 0x%x\n",
		  (unsigned)mmap->size, (unsigned)mmap->base_addr_high,
		  (unsigned)mmap->base_addr_low, (unsigned)mmap->length_high, 
		  (unsigned)mmap->length_low, (unsigned)mmap->type);
    
  }
  

  
}





