/*
 * Copyright (C) 2000, Julien Munier
 * http://kos.enix.org
 *
 * Dumpmem function
 *
 * @(#) $Id: debug.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <libcrt/stdio.h>
#include <libcrt/string.h>
#include <debug/debug.h>

/*
 * Checksum/dump stuff
 */
static void i2hex(unsigned i, char *dest)
{
#define d2hex(digit) \
  ({ unsigned _res, _d = ((digit) & 0xf); \
     _res = (_d < 0xa)?(_d + '0'):(_d - 0xa + 'a'); \
     _res; })
  dest[7] = d2hex(i);
  dest[6] = d2hex(i>>4);
  dest[5] = d2hex(i>>8);
  dest[4] = d2hex(i>>12);
  dest[3] = d2hex(i>>16);
  dest[2] = d2hex(i>>20);
  dest[1] = d2hex(i>>24);
  dest[0] = d2hex(i>>28);
  dest[8] = '\0';
}

void dump_mem(addr_t addr, size_t len)
{
  size_t i;

  DEBUG_PRINT("Dump(0x%x, +0x%x)", (unsigned)addr, len);
  for (i = 0 ; i < len ; i+=4) {
    char buff[9];
    if (!(i%7))
      DEBUG_PRINT("\n0x%x: ", addr+i);
    i2hex(*((unsigned*) (addr+i)), buff);
    DEBUG_PRINT("%s  ", buff);
  }
  DEBUG_PRINT("\n");
}

void checksum(char *str, addr_t addr, size_t len)
{
  unsigned sum = 0;
  size_t i;

  for (i = 0 ; i < len ; i+=4) {
    unsigned added = *((unsigned*) (addr+i));
    if (i+4 > len) {
        memset((char *) (((addr_t) &added) + len - i), 0x0, i+4-len);
    }
    sum += added;
  }
  if (str)
    DEBUG_PRINT("Checksum(%s) [0x%x,0x%x] (0x%x) = 0x%x\n",
		str, addr, addr+len, len, sum);
  else
    DEBUG_PRINT("Checksum [0x%x,0x%x] (0x%x) = 0x%x\n",
		addr, addr+len, len, sum);
}

void halt()
{
  { for( ; ; ) asm("hlt"); }
}
