/*
 * Copyright (C) 2000, Julien Munier
 * http://kos.enix.org
 *
 * Header for debugging functions
 *
 * @(#) $Id: debug.h,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */


#ifndef __LD_DEBUG_H__
#define __LD_DEBUG_H__

/*
  In order to debug the code 2 macros are available :
  DEBUG_PRINT(fmt, arg...) : print messages to the choosen console
  __dbg_msg(fmt, arg...) : print __FILE__ __LINE__ and messages to
  the choosen console
 */

#include <libcrt/stdio.h>
#include <debug/bochs.h>
#include <debug/serial.h>
#include <types.h>

void checksum(char *str, addr_t addr, size_t len);
void dump_mem(addr_t addr, size_t len);

#ifdef DEBUG_ON_CONSOLE
# define _macro_dbg_console_printl(x...)        printl( x)
# define _macro_dbg_console_file_printl(d,x...) printl( x)
#else
# define _macro_dbg_console_printl(x...)
# define _macro_dbg_console_file_printl(x...)
#endif

#ifdef DEBUG_ON_BOCHS
# define _macro_dbg_bochs_printl      bochs_printl
# define __dbg_file_open              bochs_file_open
# define _macro_dbg_bochs_file_printl bochs_file_printl
# define __dbg_file_close             bochs_file_close
#else
# define _macro_dbg_bochs_printl(x...)
# define __dbg_file_open(x...)  1
# define _macro_dbg_bochs_file_printl(x...)
# define __dbg_file_close(x...)
#endif

#ifdef DEBUG_ON_SERIAL
# define _macro_dbg_serial_printl(x...)        serial_printl(x)
# define _macro_dbg_serial_file_printl(d,x...) serial_printl(x)
#else
# define _macro_dbg_serial_printl(x...)
# define _macro_dbg_serial_file_printl(x...)
#endif

#define __dbg_printl(x...) \
  ({ _macro_dbg_console_printl( x); \
     _macro_dbg_bochs_printl( x); \
     _macro_dbg_serial_printl( x); })

#define __dbg_file_printl(x...) \
  ({ _macro_dbg_console_file_printl( x); \
     _macro_dbg_bochs_file_printl( x); \
     _macro_dbg_serial_file_printl( x); })


#define DEBUG_PRINT __dbg_printl

/*
 * DEBUG_PRINT with file/function/line indication.
 */
#define DEBUG_PRINT_VERBOSE(fmt, args...) ({ \
  DEBUG_PRINT("[%s@%s:%d] ", __PRETTY_FUNCTION__, __FILE__, __LINE__); \
  DEBUG_PRINT(fmt, ## args); \
})


#ifndef DEBUG_ON_CONSOLE
# define SEVERE_PRINT(fmt,args...) ({  \
  DEBUG_PRINT(fmt, ## args); \
  printl(fmt, ## args);         \
})
#else
# define SEVERE_PRINT DEBUG_PRINT
#endif

#define SEVERE_PRINT_VERBOSE(fmt, args...) ({ \
  SEVERE_PRINT("[%s@%s:%d] ", __PRETTY_FUNCTION__, __FILE__, __LINE__); \
  SEVERE_PRINT(fmt, ## args); \
})

void halt(void);
#define HALT() halt()

#define FAILED() ({ \
  SEVERE_PRINT("\n********************************************\n"); \
  SEVERE_PRINT_VERBOSE("**** System Halted (No explanation) ****\n"); \
  HALT(); \
})

#define FAILED_VERBOSE(fmt,args...) ({ \
  SEVERE_PRINT("\n********************************************\n"); \
  SEVERE_PRINT_VERBOSE("**** System Halted **** :\n"); \
  SEVERE_PRINT(fmt, ## args); \
  SEVERE_PRINT("\n"); \
  HALT(); \
})

#endif
