/*
 * Copyright (C) 2000, David Decotigny
 * http://kos.enix.org
 *
 * (ultra simple) debug messages sent to a serial port
 *
 * @(#) $Id: serial.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <libcrt/stdio.h>
#include "serial.h"

/* Store the port number of a serial unit.  */
static unsigned short serial_port;

static unsigned short  _serial_iobases [] = { 0x3f8, 0x2f8, 0x3e8, 0x2e8 } ;

#define inb(port)						\
({								\
  unsigned char _v;						\
  __asm__ volatile (						\
	"inb %w1,%0"						\
	:"=a" (_v)						\
	:"Nd" (port)						\
	);							\
  _v;								\
})


#define outb(value, port)					\
  __asm__ volatile (	       					\
	"outb %b0,%w1"						\
	::"a" (value),"Nd" (port)				\
	)							\

#define serial_inb inb
#define serial_outb(port,val) outb(val,port)

int
serial_isready (void)
{
  unsigned char status;

  status = serial_inb (serial_port + UART_LSR);
  return (status & UART_DATA_READY) ? : -1;
}


char
serial_getchar (void)
{
  /* Wait until data is ready.  */
  while ((serial_inb (serial_port + UART_LSR) & UART_DATA_READY) == 0)
    ;

  /* Read and return the data.  */
  return serial_inb (serial_port + UART_RX);
}


/* 0 on success */
int
serial_putchar (char c)
{
  /* Perhaps a timeout is necessary.  */
  int timeout = 10000;

  /* Wait until the transmitter holding register is empty.  */
  while ((serial_inb (serial_port + UART_LSR) & UART_EMPTY_TRANSMITTER) == 0)
    if (--timeout == 0)
      /* There is something wrong. But what can I do?  */
      return -1;

  serial_outb (serial_port + UART_TX, c);
  return 0;
}

void serial_printl(char *format, ...) 
{
  char **arg = (char **) &format;
  int c;
  char buf[20];
  
  arg++;
  

  while ((c = *format++) != 0)
    {
      if (c != '%')
	serial_putchar (c);
      else
	{
	  char *p;
	  
	  c = *format++;
	  switch (c)
	    {
	    case 'd':
	    case 'u':
	    case 'x':
	      itoa (buf, c, *((int *) arg++));
	      p = buf;
	      goto string;
	      break;

	    case 's':
	      p = *arg++;
	      if (! p)
		p = "(null)";

	    string:
	      while (*p)
		serial_putchar (*p++);
	      break;

	    default:
	      serial_putchar (*((int *) arg++));
	      break;
	    }
	}
    }
}

/* 0 On success */
int serial_init (unsigned short unit, unsigned int speed,
		 int word_len, int parity, int stop_bit_len)
{
  unsigned short div = 0;
  unsigned char status = 0;

  if (unit >= 4)
    return -1;
  serial_port = _serial_iobases[unit];
  
  /* Turn off the interrupt.  */
  serial_outb (serial_port + UART_IER, 0);

  /* Set DLAB.  */
  serial_outb (serial_port + UART_LCR, UART_DLAB);
  
  /* Set the baud rate.  */
  if (speed > SERIAL_BAUDRATE_MAX)
    return -1;

  div = SERIAL_BAUDRATE_MAX / speed;
  
  serial_outb (serial_port + UART_DLL, div & 0xFF);
  serial_outb (serial_port + UART_DLH, div >> 8);
  
  /* Set the line status.  */
  status |= parity | word_len | stop_bit_len;
  serial_outb (serial_port + UART_LCR, status);

  /* Enable the FIFO.  */
  serial_outb (serial_port + UART_FCR, UART_ENABLE_FIFO);

  /* Turn on DTR, RTS, and OUT2.  */
  serial_outb (serial_port + UART_MCR, UART_ENABLE_MODEM);

  /* Drain the input buffer.  */
  while (serial_isready () != -1)
    (void) serial_getchar ();
  
  return 0;
}
