#ifndef _KOS_LD_ELF32_PRIV_H
#define _KOS_LD_ELF32_PRIV_H

#include <types.h>
#include <libld/ld_bfd.h>

#include <elf32/elf32.h>

/*
 * The contents of the *COM* symbols are placed at the end of the .bss
 * section.
 */

/* Data associated with each ELF32 module */
struct elf_mod_info_s
{
  struct ld_module_struct *ldmod;
  
  Elf32_Ehdr *header;
  Elf32_Shdr *section; /* Array */
  Elf32_Half bss_id;   /* Index of the .bss section. Needed to
                          allocate the *COM* symbols. */
  Elf32_Half exp_section_text_id; /* Index of the exp_section.text
                                     section. Needed to compute the
                                     length of exported symbols
                                     marshalls */
  Elf32_Half exp_section_data_id; /* Index of the exp_section.text
                                     section. Needed to compute the
                                     length of exported symbols
                                     marshalls */

  /* size of the .init heap space allocated for this module */
  size_t   init_heap_size;
};

char*  elf32_get_section_name(struct elf_mod_info_s* elfmod,
			      Elf32_Shdr *section);

/** special kos sh_flag: elf32_check mark the sections to be allocated
    in the .init heap with this flag. */
#define SHF_KOS_ALLOC_IN_INIT_HEAP 0x8
/** special kos sh_flag: elf32_check mark the symtab sections really
    used (ie symtab for alloc sections not placed in the .init heap)
    with this flag */
#define SHF_KOS_USED_SYMTAB        0x20

/** This particular symtab must *ALWAYS* be considered used by kos */
#define COMPILER_SYMTAB ".symtab"

/** list of section names that are to be allocated in the .init heap */
#define INIT_HEAP_SECTIONS_LIST \
  INIT_TEXT_SECTION_NAME, \
  INIT_DATA_SECTION_NAME, \
  CTOR_SECTION_NAME

/* ELF32 ld operations */
extern struct ld_bfd_operations_struct elf32_ops;
int elf32_check_magic (struct ld_module_struct *module);
int elf32_init (ld_module_t *module);
int elf32_alloc (ld_module_t *ldmod);
int elf32_build (ld_module_t *ldmod);
int elf32_relocate_local(ld_module_t *module);
int elf32_update_tables (ld_module_t *ldmod);
int elf32_relocate_import(ld_module_t *module);

#endif /* KOS_LD_ELF32_PRIV_H */
