
#include <macros.h>

#include "_elf32.h"

int elf32_check_magic (struct ld_module_struct *module)
{
  Elf32_Ehdr *current_Ehdr = (Elf32_Ehdr *)module->raw_image_start;

  /* checking for a ELF module   */
  if ( current_Ehdr->e_ident[EI_MAG0] != 0x7F || 
       current_Ehdr->e_ident[EI_MAG1] != 'E'  || 
       current_Ehdr->e_ident[EI_MAG2] != 'L'  || 
       current_Ehdr->e_ident[EI_MAG3] != 'F' ) 
    return FALSE;
  
  /* checking that the file is a ELF 32 bits */
  if(current_Ehdr->e_ident[EI_CLASS] != ELFCLASS32)
    {
      WARNING("Invalid ELF class for module %s\n",
	      module->name);
      return FALSE;
    }

  /* checking machine */
  if(current_Ehdr->e_machine != EM_386)
    {
      WARNING("Invalid ELF architecture type for module %s\n",
	      module->name);
      return FALSE;
    }

  /* checking that byte_order is OK */
  if(current_Ehdr->e_ident[EI_DATA] != ELFDATA2LSB)
    {
      WARNING("Invalid ELF byte order for module %s\n",
	      module->name);
      return FALSE;
    }

  /* checking ELF version */
  if(current_Ehdr->e_ident[EI_VERSION] != EV_CURRENT)
    {
      WARNING("Invalid ELF version for module %s\n",
	      module->name);
      return FALSE;
    }

  /* checking that the file is a relocatable file */
  if(current_Ehdr->e_type != ET_REL)
    {
      WARNING("ELF module %s is not relocatable\n",
	      module->name);
      return FALSE;
    }

  /* Make sure header data structure is correct */
  if (current_Ehdr->e_ehsize != sizeof(Elf32_Ehdr))
    {
      WARNING("ELF module %s has wrong header data structure size\n",
	      module->name);
      return FALSE;
    }

  /* Make sure section data structures are correct */
  if (current_Ehdr->e_shentsize != sizeof(Elf32_Shdr))
    {
      WARNING("ELF module %s has wrong section data structure size\n",
	      module->name);
      return FALSE;
    }

  /* Make sure the section table exists */
  if (current_Ehdr->e_shoff == 0)
    {
      WARNING("ELF module %s has no section header table\n",
	      module->name);
      return FALSE;
    }

  /* Make sure header string section index is correct */
  if (current_Ehdr->e_shstrndx >= current_Ehdr->e_shnum)
    {
      WARNING("ELF module %s has wrong header string section index\n",
	      module->name);
      return FALSE;
    }

  /* Don't support program headers yet */
  if ((current_Ehdr->e_phoff != 0) || (current_Ehdr->e_phnum != 0))
    {
      WARNING("ELF module %s: the loader does not support program headers yet \n",
	      module->name);
      return FALSE;
    }

  return TRUE; 
}
