
#include <macros.h>
#include <assert.h>
#include <libcrt/string.h>

#include "_elf32.h"

int elf32_update_tables (ld_module_t *ldmod)
{
  struct kernel_module *km = ldmod->kernel_module;
  struct elf_mod_info_s *elfmod = (struct elf_mod_info_s*)ldmod->operation_data;
  Elf32_Half section_index;

  for (section_index = 0 ;
       section_index < elfmod->header->e_shnum ;
       section_index++)
    {
      Elf32_Shdr *section = elfmod->section + section_index;

      if (! strcmp(CTOR_SECTION_NAME,
		   elf32_get_section_name(elfmod, section)))
	{
	  km->u.reloc_executable.init_routines
	    = (init_cleanup_routine_t*)section->sh_addr;
	  km->u.reloc_executable.nb_init_routines
	    = section->sh_size / sizeof(init_cleanup_routine_t);

	  /* If there is garbage afterwards, we suspect that someone
             did not put real init_cleanup_routine_t data.... */
	  if (section->sh_size % sizeof(init_cleanup_routine_t))
	    FAILED_VERBOSE("Incorrect reserved section %s in module %s\n",
			   CTOR_SECTION_NAME,
			   elfmod->ldmod->name);
	}

      else if (! strcmp(DTOR_SECTION_NAME,
			elf32_get_section_name(elfmod, section)))
	{
	  km->u.reloc_executable.cleanup_routines
	    = (init_cleanup_routine_t*)section->sh_addr;
	  km->u.reloc_executable.nb_cleanup_routines
	    = section->sh_size / sizeof(init_cleanup_routine_t);

	  /* If there is garbage afterwards, we suspect that someone
             did not put real init_cleanup_routine_t data.... */
	  if (section->sh_size % sizeof(init_cleanup_routine_t))
	    FAILED_VERBOSE("Incorrect reserved section %s in module %s\n",
			   DTOR_SECTION_NAME,
			   elfmod->ldmod->name);
	}

      else if (! strcmp(EXP_CC_FUNCTAB_SECTION_NAME,
			elf32_get_section_name(elfmod, section)))
	{
	  km->u.reloc_executable.exp_cc_funcs
	    = (exported_symbol_t*)section->sh_addr;
	  km->u.reloc_executable.nb_exp_cc_funcs
	    = section->sh_size / sizeof(exported_symbol_t);

	  /* If there is garbage afterwards, we suspect that someone
             did not put real init_cleanup_routine_t data.... */
	  if (section->sh_size % sizeof(exported_symbol_t))
	    FAILED_VERBOSE("Incorrect reserved section %s in module %s\n",
			   EXP_CC_FUNCTAB_SECTION_NAME,
			   elfmod->ldmod->name);
	}

      else if (! strcmp(EXP_CC_VARTAB_SECTION_NAME,
			elf32_get_section_name(elfmod, section)))
	{
	  km->u.reloc_executable.exp_cc_vars
	    = (exported_symbol_t*)section->sh_addr;
	  km->u.reloc_executable.nb_exp_cc_vars
	    = section->sh_size / sizeof(exported_symbol_t);

	  /* If there is garbage afterwards, we suspect that someone
             did not put real init_cleanup_routine_t data.... */
	  if (section->sh_size % sizeof(exported_symbol_t))
	    FAILED_VERBOSE("Incorrect reserved section %s in module %s\n",
			   EXP_CC_VARTAB_SECTION_NAME,
			   elfmod->ldmod->name);
	}
    }

  DEBUG_PRINT("Nb exp cc funcs = %d\n",
	      km->u.reloc_executable.nb_exp_cc_funcs);
  DEBUG_PRINT("Nb exp cc vars = %d\n",
	      km->u.reloc_executable.nb_exp_cc_vars);
  DEBUG_PRINT("Nb exp section funcs = %d\n",
	      km->u.reloc_executable.nb_exp_section_funcs);
  DEBUG_PRINT("Nb exp section vars = %d\n",
	      km->u.reloc_executable.nb_exp_section_vars);

  ld_register_exported_functions(ldmod,
				 km->u.reloc_executable.exp_cc_funcs,
				 km->u.reloc_executable.nb_exp_cc_funcs);
  ld_register_exported_functions(ldmod,
				 km->u.reloc_executable.exp_section_funcs,
				 km->u.reloc_executable.nb_exp_section_funcs);
  ld_register_exported_variables(ldmod,
				 km->u.reloc_executable.exp_cc_vars,
				 km->u.reloc_executable.nb_exp_cc_vars);
  ld_register_exported_variables(ldmod,
				 km->u.reloc_executable.exp_section_vars,
				 km->u.reloc_executable.nb_exp_section_vars);

  return 0;
}
