#ifndef __KOS_LD_BFD_H__
#define __KOS_LD_BFD_H__

/*
 * KOS Binary file formats handling routines
 */

#include <types.h>
#include <mod.h>


#define MOD_NAME_MAX_LEN 64

struct ld_module_struct; /* See below */

struct ld_bfd_operation_struct {
  char *name;
  int (*operation)(struct ld_module_struct *module);
};


typedef struct ld_bfd_operations_struct {

  char *name;

  /* Returns TRUE if the magic of module can be processed by this
     operations_struct */
  int (*check_magic)(struct ld_module_struct *module);

  int num_ops; /* Number of operations in the operation array */
  struct ld_bfd_operation_struct *op;

} ld_bfd_operations_t;


typedef struct ld_module_struct {
  char name[MOD_NAME_MAX_LEN];
  addr_t raw_image_start; /* start of the not-yet cooked module */
  size_t raw_image_size;

  /* Set up after the load_module phase */
  struct kernel_module *kernel_module;

  /* Set up after the check_header phase */
  struct ld_bfd_operations_struct *operation;

  /* Private data for the operations associated with module */
  void *operation_data; /* For ELF32, this will be the elf_mod_t */

  /* modules are arranged in a linked list */
  struct ld_module_struct *next;
} ld_module_t;


extern kernel_parameter_t *kernel_params;
extern struct ld_module_struct *ld_modules_head, *ld_modules_tail;


/**
 * Add the module at address addr to the linked list of modules,
 * after "after_which_module" if it is not NULL, or at the end of the
 * ll otherwise. Does also associate it to its corresponding
 * operation. If the module can't be hold by any operation, returns
 * NULL. Returns the resulted 'module struct' on success.
 */
struct ld_module_struct * ld_register_module (char *name, addr_t addr,
					      size_t size,
					      ld_module_t *after_which_module);

/** Get current heap size to allocate / already allocated. This is
    also the offset in the heap of the next ld_inc_init_heap_size(). */
size_t ld_get_init_heap_size(void);
/** Ask for some space in the init heap. @return new size. @note Must
    NOT call it once ld_build_init_heap() has been called. */
size_t ld_inc_init_heap_size(size_t sz);
/** NO more ld_inc_init_heap_size() is possible once this has been
    called. */
vaddr_t ld_alloc_in_init_heap(size_t sz);

/**
 * Register each of the exported functions of module owner in the hash
 * table.
 * Succeeds, or else stops the system.
 */
void ld_register_exported_functions(struct ld_module_struct *owner,
				    exported_symbol_t *exp_funcs,
				    uint nb_funcs);

/**
 * Register each of the exported variables of module owner in the hash
 * table.
 * Succeeds, or else stops the system.
 */
void ld_register_exported_variables(struct ld_module_struct *owner,
				    exported_symbol_t *exp_vars,
				    uint nb_vars);

/**
 * @return 1 when found. 0 otherwise.
 */
int ld_lookup_expsym(const char *str,
		     /*out*/int *is_function,
		     /*out*/exported_symbol_t **sym,
		     /*out*/struct ld_module_struct **ldmod);

/* For loader.c only */
void ld_init_expsym_hashtable(void);
void ld_stats_hashtable(void);
int ld_build (void);

#endif /* __KOS_LD_BFD_H__ */
