
#include <assert.h>
#include <mm/mm.h>

#include "ld_bfd.h"

/* .init heap management */

static size_t init_heap_used = 0;

/** Get current heap size to allocate / already allocated. This is
    also the offset in the heap of the next ld_inc_init_heap_size(). */
size_t ld_get_init_heap_size(void)
{
  return kernel_params->init_heap_size;
}

/** Ask for some space in the init heap. @return new size. @note Must
    NOT call it once ld_build_init_heap() has been called. */
size_t ld_inc_init_heap_size(size_t sz)
{
  /* Don't extend heap if it is already allocated !!! */
  if (kernel_params->init_heap_vaddr != 0)
    FAILED_VERBOSE("An ld_op tries to extend init heap though it has already been allocated !\n");

  kernel_params->init_heap_size += sz;
  return kernel_params->init_heap_size;
}

/** NO more ld_inc_init_heap_size() is possible once this has been
    called. */
vaddr_t ld_alloc_in_init_heap(size_t sz)
{
  vaddr_t ret;

  if (sz == 0)
    return 0;

  ASSERT_FATAL(init_heap_used + sz <= kernel_params->init_heap_size);
  
  /* If heap is not already allocated, allocate it */
  if (kernel_params->init_heap_vaddr == 0)
    {
      ASSERT_FATAL(kernel_params->init_heap_size > 0);
      kernel_params->init_heap_vaddr
	= (vaddr_t) vmm_page_alloc(kernel_params->init_heap_size,
				   & kernel_params->init_heap_paddr);
      ASSERT_FATAL(kernel_params->init_heap_vaddr != 0);
    }

  /* Allocate space on the heap */
  ret = kernel_params->init_heap_vaddr + init_heap_used;
  init_heap_used += sz;
  
  return ret;
}
