/*
 * Copyright (C) 2000, Julien Munier, Thomas Petazzoni
 * http://kos.enix.org
 *
 * Paging initialization functions
 *
 * @(#) $Id: mm.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <macros.h>
#include <assert.h>
#include <libld/ld_bfd.h>
#include <types.h>
#include <debug/debug.h>
#include <libcrt/stdio.h>
#include <modules/kos/system.h>

#include "_mm.h"

unsigned long total_mem_size, loader_allocatable_mem_size;

void mm_init(struct multiboot_info *mbi)
{
  total_mem_size = mbi->mem_upper * 1024 + 0x100000;
  printl(" Total mem size: %dM aka %dB", 
	 (unsigned)(total_mem_size >> 20), 
	 (unsigned)total_mem_size);
  
  // identity mapping on all physical memory, with a limit of 256 Mb
  // calculting how much memory must be identically mapped.
  PAUSE("determine loader_alloc_mem_size");
  if(( total_mem_size / (1024*1024) ) > 256 )
    loader_allocatable_mem_size = 256 * 1024 * 1024 ;
  else
    loader_allocatable_mem_size = total_mem_size;

  
  // Initialisation of the BOOTMEM
  PAUSE("pmm_init");
  pmm_init(mbi, loader_allocatable_mem_size);
  PAUSE("paging_init");
  vmm_init(loader_allocatable_mem_size);
  
  // Allocate some memory for the kernel paramater structure
  // this has to be after bootmem_init, but before the end of memory_init
  // so here is the only place ...
  DEBUG_PRINT_VERBOSE("valloc kp");
  kernel_params =
    (kernel_parameter_t *) vmm_page_alloc(sizeof(kernel_parameter_t), NULL);
  ASSERT_FATAL(kernel_params != NULL);
  kernel_params->modules_head = kernel_params->modules_tail = NULL;
  kernel_params->init_heap_size  = 0;
  kernel_params->init_heap_vaddr = 0;
  kernel_params->init_heap_paddr = 0;

  // fill in kernel parameters fields related to memory
  kernel_params->total_mem_size = total_mem_size;
  kernel_params->loader_allocatable_mem_size = loader_allocatable_mem_size;
}
