/*
 * Copyright (C) 2000, Christophe Avoinne
 * http://kos.enix.org
 *
 *
 * @(#) $Id: pmm_alloc.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <debug/debug.h>
#include <libcrt/string.h>
#include <macros.h>

#include "_mm.h"

addr_t palloc_min_phys_addr = 0x0;

void *pmm_page_alloc (unsigned int size)
{
  struct free *free;
  struct free *next;
  struct free *prev = 0;

  if (!size)
    return 0;

  // Align size on a 4k boundary, and compute a number of struct free
  // blocks (>>3) used throughout this code using free+free->size
  size = ALIGN_SUP(size, PAGE_SIZE);

  // Computes the size of this cell as a number of struct free (8
  // Bytes) elements
  size = size >>3;

  // Look for THE LAST free element in the free_list, which HAS to be
  // convenient for us (otherwise return 0). If we find adjacent free
  // spaces during this walk, we merge them together.
  free = pmm_head;
  while (free+free->skip < pmm_tail)
    {
      // Merging if the current free slot is adjacent to the next
      if (free->size == free->skip)
        {
          next       = free+free->size;
          free->size+= next->size;
          free->skip+= next->skip;
          continue;
        }
      // Remember both the previous & the current free slots
      prev = free;
      free+= free->skip;
    }

  // Here, we are at the last free slot, having merged all the other
  // former adjacent free slots
  
  // The last free slot (aka current) fits exactly our needs => take
  // it and eventually update the head of the free slots if it happens
  // to be this free slot
  if (free->size == size) 
    {
      if (!prev)
        pmm_head = free+free->skip;
      else
	prev->skip += free->skip;
      palloc_min_phys_addr = (addr_t)free;
      return (void *)free;
    }
  // This last block is larger than what we need => split it
  else if (free->size > size)
    {        
      // split the current block
      free->size-= size;
      next = free+free->size;
      palloc_min_phys_addr = (addr_t)next;
      return (void *)next;               
    }
  // Or else : the last block was not large enough => allocation aborted
  return (void *)0;   
}


addr_t pmm_page_alloc_min_phys_addr()
{
  return palloc_min_phys_addr;
}


void *pmm_malloc (unsigned int size)
{
  struct free *free;
  struct free *next;
  struct free *prev = 0;

  if (!size)
    return 0;

  // Each allocated cell contains 1/ the struct free (8 Bytes), 2/ the
  // data. Align the total size to a 8 byte boundary in order for it
  // to be a multiple of struct free elements
  size = 8 + ALIGN_SUP(size, 8) ;

  // Computes the size of this cell as a number of struct free (8
  // Bytes) elements
  size = size >>3;

  // Look for THE FIRST convenient free slot for us.  If we find
  // adjacent free spaces during this walk, we merge them together.
  free = pmm_head;
  while (free < pmm_tail) 
    {
      // if the block size is insufficient, we try to join the current
      // block with other adjacent block when needed
      if (free->size < size)
        {
          if (free->size == free->skip)
            {
	      next       = free+free->size;
              free->size+= next->size;
              free->skip+= next->skip;
              continue;
            }
	  // Remember both the previous & the current free slots
          prev = free;
          free+= free->skip;
	  continue;
        }

      // Ok: Here we have a suitable free slot
        
      // If the block is exactly what we need, take it : update the
      // free slots' list
      if (free->size == size) 
        {
          if (!prev)
            pmm_head = free+free->skip;
	  else
	    prev->skip += free->skip;

	  // Mark as used
	  free->skip = 0;

	  // Usable space is after the struct free => free+1
          return (void *)((int *)free+1);
        }
      
      // Or else the slot is larger than what we need => split it
      next = free+size;
      next->size = free->size-size;
      next->skip = free->skip-size;
      free->size = size;

      // Mark as used
      free->skip = 0;
            
      if (!prev)
        pmm_head = next;
      else
        prev->skip = next-prev;

      return (void *)((int *)free+1);               
    }
        
  // no block was found
  return (void *)0;
}

void *pmm_realloc (void *block,unsigned int size)
{
  char *old;
  char *new;

  if (!block)
    return (void *)0;
    
  old = (char *)block;
  
  if ((new = (char *)pmm_malloc(size)))
    memcpy(new,old,size);

  pmm_free(block);

  return (void *)new;
}
