/*
 * Copyright (C) 2000, Christophe Avoinne
 * http://kos.enix.org
 *
 * Mark free function for the heap memory manager
 *
 * @(#) $Id: pmm_free.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <loader/macros.h>
#include <assert.h>

#include "_mm.h"

void pmm_mark_free(void *block,unsigned int size)
{
  struct free *free = (struct free *)block;

  if (size <= 0)
    return;

  free->size = size = ALIGN_SUP(size, 8) >>3;
  if (pmm_tail < pmm_head)
    {
      pmm_head =
      pmm_base = free;
      pmm_tail = free+size;
      free->skip     = size;
    }
  else
    {
      pmm_free((void *)((int *)block+1));
    }
  
      
}

void pmm_free(void *block)
{
  struct free *free;
  struct free *next;
  struct free *prev;

  if (!block)
    return;

  free = (struct free *)((int *)block-1);

  /* Thomas Petazzoni 31/01/02 removed line because bootmem_free is
     used to mark some blocks as free even if they were not allocated
     with bootmem_alloc. This is done during the initialization
     process. maybe some check needs to be done, but that's what I
     think ! */
  // Blocks marked used set the skip field to NULL
  // ASSERT_FATAL(free->skip == 0);

  // If we are the brand new head of the free slots' list, we become
  // the brand new head of the free nslots' list :)
  if (free < pmm_head)
    {
      pmm_head = free;

      if (free < pmm_base)
	pmm_base = free;

      free->skip = pmm_head - free;

      ASSERT_FATAL(pmm_tail >= free); // Or uncomment above

      return;
    }

  // Ok: If we are here, we have to insert somewhere between 2 free
  // slots

  next = prev = pmm_head;  
  while (next < pmm_tail)
    {
      prev = next;
      next+= next->skip;

      // We come later in the list
      if (next < free)
        continue;

      // Ok: We got a 'next' that comes after us for the first time =>
      // that's where we must insert

      prev->skip = free - prev; // I will be 'prev's next
      free->skip = next - free; // My next will be 'next'
      
      return;
    }

  // Couldn't find any element in the list that could come after us =>
  // We come after the tail
 
  pmm_tail = free + free->size;
  prev->skip  = free - prev;
  free->skip  = free->size;
}
