/*
 * Copyright (C) 2000, Christophe Avoinne
 * http://kos.enix.org
 *
 * Initialization of the heap memory manager
 *
 * @(#) $Id: pmm_init.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <debug/debug.h>
#include <boot/multiboot.h>
#include <libcrt/stdio.h>
#include <libcrt/string.h>
#include <macros.h>

#include "_mm.h"

struct free *pmm_head = (struct free *)-1;
struct free *pmm_base = (struct free *)-1;
struct free *pmm_tail;

extern char __b_loader[], __e_loader[];


/*
 * Shrink the [min, max] interval (local variables in _init() so that :
 * - [hole_min, hole_max] Intersects [min, max] and min doesn't belong
 *   to this intersection => new interval [min, max] <- [min, __min]
 * - If they intersect, but min is in the intersection => next iteration
 * - Otherwise do nothing
 */
#define skip(hole_min,hole_max) \
  ({ \
    unsigned int \
      __min = (hole_min), \
      __max = (hole_max); \
    if ((max > __min) && (min < __max)) \
      { \
        if (min < __min) \
          max = __min; \
        else \
          { \
            min = __max; \
            goto retry; \
	  } \
      } \
  })

unsigned int lowermem,  uppermem;
  
void pmm_init (multiboot_info_t *mbi,
	       unsigned long loader_upper_mem)
{
  unsigned int
    min,max; 

  unsigned int
    __b_cmdline =
      (mbi->flags & MULTIBOOT_CMDLINE ? mbi->cmdline : 0),
    __e_cmdline =
    (__b_cmdline ? __b_cmdline+strlen((char*)__b_cmdline)+1 : 0);

  lowermem = mbi->mem_lower*1024;
  uppermem = ALIGN_INF(loader_upper_mem,PAGE_SIZE);
  

  /* For each iteration, min is set, and we want to find the closest
   * max so that the interval doesn't intersect to any other interval */
  min = 0x00001000;
  do
    {
      max = 0xffffffff;

      // skip some memory areas

      // first skip the 1st Mb
      skip(lowermem,0x00100000);

      // then skip all non physical memory
      skip(uppermem,0xffffffff);

      // skip the space occupied by the loader code
      skip((unsigned int)__b_loader,(unsigned int)__e_loader);

      // skip the space occupied by the command line
      skip(__b_cmdline,__e_cmdline);
      
      // skip the space occupied by the modules and their comand lines
      if ((mbi->flags & MULTIBOOT_MODS) && mbi->mods_count)
        {
          module_t *m = (module_t *)mbi->mods_addr;
	  unsigned int i;
	  
          skip(mbi->mods_addr,mbi->mods_addr+mbi->mods_count*sizeof(*m));
	  
          for (i = 0; i < mbi->mods_count; i++)
	    {
               if (m[i].string != 0)
                 {
                   char *s = (char *)m[i].string;
                   unsigned int len = strlen(s);
                   skip(m[i].string,m[i].string+len+1);
                 }
               skip(m[i].mod_start,m[i].mod_end);
             }
        }
      
      min = ALIGN_SUP(min,8);

      pmm_mark_free( (void *) min,max-min);
      
      min = max;
      
    retry:
    }
  while (min < 0xffffffff);

}

