/*
 * Copyright (C) 2000, Julien Munier, Thomas Petazzoni
 * http://kos.enix.org
 *
 * Paging initialization & management functions
 *
 * @(#) $Id: vmm.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */
#include <macros.h>
#include <assert.h>
#include <types.h>

#include <debug/debug.h>
#include <libcrt/stdio.h>
#include <libcrt/string.h>
#include <mod.h>
#include <modules/kos/system.h>

#include "_mm.h"

/*
 * virt AND phys MUST be page-aligned !!!
 */ 
void map_virtual_to_physical(addr_t virt, addr_t phys, int flags)
{
  addr_t *pd, pde;
  addr_t *pt;

  pd = (addr_t*) CURRENT_PD_VADDR;
  //  DEBUG_PRINT_VERBOSE("Pd vaddr=0x%x, pd[511]=0x%x", pd,pd[511]);
  
  pde = pd[va_to_pde(virt)];
  //  DEBUG_PRINT_VERBOSE("Mirroring seems to be Ok\n");

  if (! pde) /* Then allocate a new PT */ {
    pde = (addr_t) pmm_page_alloc(PAGE_SIZE);
    CONCEPTION_ASSERT(pde);
    memset((void*)pde, 0x0, PAGE_SIZE);

    pde |= PTE_WRT | PTE_VAL;

    /* Update real PDE */
    pd[va_to_pde(virt)] = pde;
  }

  /* Get PT (masking of pde's flags) */
  pt = (addr_t *) (pde & (~(PAGE_SIZE-1)));

  /* Effective mapping of the page */
  pt[va_to_pte(virt)]  = phys;
  pt[va_to_pte(virt)] |= PTE_WRT | PTE_VAL | flags;
}

static addr_t current_vaddr_ptr = CORE_KERNEL_VIRTUAL_ADDR;

/*
 * Map a block of contiguous memory located at at_phys in physical
 * memory into virtual memory
 */
void * vmm_page_alloc_at_phys_addr(size_t size, addr_t at_phys, int flags)
{
  void *top_page;
  unsigned int i;

  if (size <= 0)
    return 0;

  top_page = (void *) current_vaddr_ptr;
  for (i = 0 ; i < ALIGN_SUP(size, PAGE_SIZE) ; i += PAGE_SIZE) {
    map_virtual_to_physical(current_vaddr_ptr, at_phys, flags);
    current_vaddr_ptr += PAGE_SIZE;
    at_phys           += PAGE_SIZE;
  }

  return top_page;
}


void * vmm_page_alloc(size_t size, addr_t *returned_paddr)
{
  void *top_page;
  unsigned int i;

  if (size <= 0)
    return 0;

  top_page = (void *) current_vaddr_ptr;
  for (i = 0 ; i < ALIGN_SUP(size, PAGE_SIZE) ; i += PAGE_SIZE) {
    addr_t phys = (addr_t)pmm_page_alloc(PAGE_SIZE);
    ASSERT_FATAL(phys != 0);
    memset((char*)phys, 0x0, PAGE_SIZE);
    if (returned_paddr)
      *returned_paddr = phys;
    DEBUG_PRINT_VERBOSE(">>> Mapping phys=0x%x at virt=0x%x...\n",
			phys, current_vaddr_ptr);
    map_virtual_to_physical(current_vaddr_ptr, phys, 0);
    //    DEBUG_PRINT_VERBOSE("Mapping done\n");
    current_vaddr_ptr += PAGE_SIZE;
  }

  return top_page;
}

addr_t vmm_get_top_virt_addr()
{
  return current_vaddr_ptr;
}
