
#include <assert.h>
#include <libcrt/string.h>

#include "_mm.h"

int vmm_init(size_t im_size)
{
  addr_t *pd; /* A PD is a pointer to an array of 1024 PT */
  unsigned int i;

  DEBUG_PRINT_VERBOSE("Initializing Paging (%dM)\n", im_size >> 20);

  // allocating space for the PD
  pd = (addr_t *)pmm_page_alloc(PAGE_SIZE);
  ASSERT_FATAL(pd != NULL);
  
  // reset the PD to zero
  memset((char*)pd, 0, PAGE_SIZE);

  DEBUG_PRINT_VERBOSE("Building identity mapping up to : %d Mb\n",
		      (unsigned) im_size/(1024*1024));

  /* Set-up Identity-mapping */
  for (i = 0 ;
       i < ALIGN_SUP(im_size, 1024*PAGE_SIZE) / (1024*PAGE_SIZE) ;
       i++)
  {
    addr_t *pt;
    unsigned int j;

    DEBUG_PRINT_VERBOSE("Alloc pd[%d]\n", i);

    /* allocation PTs for this identity mapping. */
    pd[i] = (addr_t)pmm_page_alloc(PAGE_SIZE);
    ASSERT_FATAL(pd[i]);

    pt = (addr_t*)pd[i];
    memset((char*)pt, 0, PAGE_SIZE);
  
    /* build those PTs */

    for ( j = 0 ; j < 1024 ; j++ ) {
      
      if ((i*1024 + j)*PAGE_SIZE > im_size)
	break;

      // Entry at phys addr 0 = Not Present => Page fault !
      if ((i == 0) && (j == 0))
	continue;

      pt[j] = ((i*1024 + j)*PAGE_SIZE) | PTE_VAL | PTE_WRT;
    }
    //    dump_table(pt);
  
    /* Set up pde flags */

    // Number of PT allocated = loader_allocatable_mem_size / 4 MB
    // We have to update the PD from the beginning up to the entry #
    // loader_allocatable_mem_size/4MB
    pd[i] |= (PTE_VAL | PTE_WRT);
  }

  // enable mirroring
  pd[CURRENT_PT_AREA_INDEX]  = (addr_t)pd;
  pd[CURRENT_PT_AREA_INDEX] |= (PTE_VAL | PTE_WRT);

  //printk("Switching to pagined mode ...");
  DEBUG_PRINT_VERBOSE("Switching to pagined mode");
  __asm__ __volatile__ (
       "movl %0,%%cr3\n\t"
       "movl %%cr0,%%eax\n\t"
       "orl $0x80000000, %%eax\n\t"
       "movl %%eax,%%cr0\n\t"
       "jmp 1f\n\t"
       "1:\n\t"
       "movl $2f, %%eax\n\t"
       "jmp *%%eax\n\t"
       "2:\n\t" ::"r"(pd):"memory","eax");

  flush_tlb();

  __asm__ __volatile__("movl %%cr3, %0\n" : "=r" (i) : );
  DEBUG_PRINT_VERBOSE("CR3 : 0x%x\n", (unsigned)i);

  __asm__ __volatile__("movl %%cr0, %0\n" : "=r" (i) : );
  DEBUG_PRINT_VERBOSE("CR0 : 0x%x\n", (unsigned)i);

  return 0;
}
