
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include <libld/ld_bfd.h>
#include <mm/_mm.h> // For PAGE_SIZE macro
#include <modules/kos/system.h>

#include "_utils.h"


/* ************************************************************
 * Print Init order
 */
int print_init_order(unsigned int base_level, unsigned int nb_levels)
{
  int i;
  unsigned int cur_level  = 0;
  unsigned int next_level = 1;
  struct kernel_module *km = NULL;

  if (kernel_params->modules_head == NULL)
    return -1;

  /* Init code modules */
  while (1) {
    init_cleanup_routine_t *ctor;

    if (km == NULL)
      {
	km = kernel_params->modules_head;
	cur_level  = next_level;
	next_level = base_level + nb_levels + 10;

	if (cur_level >= (base_level + nb_levels))
	  break;
      }
    else
      {
	km = km->next;
	if (km == NULL)
	  continue;
      }

    if(km == NULL)
      {
	abort();
      }

    if (km->magic != MAGIC_RELOC_EXECUTABLE)
      continue;

    /* In Current module, try to figure out a init level equal to
       cur_level */
    for (i = km->u.reloc_executable.nb_init_routines, ctor = km->u.reloc_executable.init_routines ;
	 i ;
	 i--, ctor ++)
      {
	if (ctor->level == cur_level)
	  {
	    printf("  Level %d: module %s\n",
		   cur_level - base_level,
		   km->data);

	    ctor->level = 0; /* Reset level */
	  }
	/* Or else remember the least next level */
	else if ((ctor->level > 0) && (ctor->level < next_level))
	  next_level = ctor->level;
      }

    /* Try next module */
  }

  return 0;
}
