
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include <libld/ld_bfd.h>
#include <mm/_mm.h> // For PAGE_SIZE macro
#include <modules/kos/system.h>

#include "_utils.h"


/* ************************************************************
 * bochs friendly functions
 */
static void dump_module_syms(FILE* f, kernel_module_t* km, size_t offset)
{
  addr_t addr;
  
  for (addr = (addr_t)km->u.reloc_executable.def_syms ;
       addr < (addr_t)km->u.reloc_executable.contents ;
       addr += sizeof(defined_symbol_t))
    {
      defined_symbol_t *ds = (defined_symbol_t*)addr;
      fprintf(f, "%x %s:%s\n",
	      offset + (ds->addr - (addr_t)km),
	      km->data, ds->name);
    }
}

int gen_bochs_symfile(char* fname) {
  struct ld_module_struct* ldmod;
  size_t offset = CORE_KERNEL_VIRTUAL_ADDR;
  FILE* f;

  f = fopen(fname, "w");
  if (!f)
    {
      perror("open bochs symbol file");
      return -1;
    }

  /* Enough room for the kernel_params */
  offset += sizeof(struct kernel_parameter);

  /* Simulate one more page, which is used in the "real" kos for the
     command line parameters */
  offset += PAGE_SIZE;

  /* Lookup the module if given */
  for (ldmod = ld_modules_head ; ldmod ; ldmod = ldmod->next)
    {
      
      /* Adjust 4k page boundary (take the .ar modules into account) */
      offset = ALIGN_SUP(offset, PAGE_SIZE);

      if (ldmod->kernel_module
	  && (ldmod->kernel_module->magic == MAGIC_RELOC_EXECUTABLE))
	dump_module_syms(f, ldmod->kernel_module, offset);

      /* Prepare for next module (take the .ar modules into account) */
      if (ldmod->kernel_module)
	offset += ldmod->kernel_module->size;
    }

  fclose(f);
  return 0;
}
