
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <libld/ld_bfd.h>
#include <mm/_mm.h> // For PAGE_SIZE macro
#include <modules/kos/system.h>

#include "_utils.h"

int bochs_printl(const char *format, ...)
{
  va_list ap;
  int res;

  if (! view_bochs_output)
    return 0;
  
  va_start (ap, format);
  res = vfprintf(stderr, format, ap);
  va_end(ap);

  return res;
}

int serial_printl(const char *format, ...)
{
  va_list ap;
  int res;

  if (! view_serial_output)
    return 0;
  
  va_start (ap, format);
  res = vfprintf(stderr, format, ap);
  va_end(ap);

  return res;
}

int printl (const char *format, ...)
{
  va_list ap;
  int res;

  if (! view_console_output)
    return 0;
  
  va_start (ap, format);
  res = vfprintf(stderr, format, ap);
  va_end(ap);

  return res;
}

void halt()
{
  exit(-1);
}

void * vmm_page_alloc(size_t size, addr_t *allocated_phys_addr)
{
  int dev_zero_fd;
  void* res;

  /* Use mmap in order not to bother with the 4k page boundary
     alignments */

  dev_zero_fd = open("/dev/zero", O_RDWR);
  if (dev_zero_fd < 0)
    {
      perror("Open /dev/zero");
      exit(-1);
    }

  res = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE,
	     dev_zero_fd, 0);
  if (res == NULL)
    {
      perror("mem_valloc");
      exit(-1);
    }
  close(dev_zero_fd);

  /* Make sure address is page-aligned */
  if (((addr_t)res) & (PAGE_SIZE-1))
    {
      fprintf(stderr, "ERROR: mmaped page not page-aligned on this system\n");
      exit(-1);
    }

  /* Update the result with a fake physical address */
  if (allocated_phys_addr)
    *allocated_phys_addr = (addr_t)res;

  return res;
}

void *pmm_malloc(unsigned int size)
{
  return malloc(size);
}

