/*
 * Backtracing
 */
#include <kos/assert.h>

#include <debug/_debug.h>
#include <arch/task/task.h>

#define called_ip priv0
#define frame     priv1

static enum {UNKNOWN, _NOT_SET, _SET} fomit_frame_pointer = UNKNOWN;

__init_text int _init_backtracing_stuff(void)
{
  char *first_opcode = (char*)bt_next;

  if (*first_opcode != 0x55) /* pushl %ebp */ {
    fomit_frame_pointer = _SET;
    WARNING("Backtracing disabled (due to gcc -fomit-frame-pointer)\n");
  } else {
    fomit_frame_pointer = _NOT_SET;
    DEBUG_PRINT("Backtracing enabled.\n");
  }

  return 0;
}


int bt_init(struct bt_data* d, struct cpu_state_s *cpu_state)
{  
  ASSERT_RETURN_VAL(d, -1);

  if (cpu_state) {
    d->called_ip = cpu_state->eip;
    d->frame     = cpu_state->ebp;
  } else {
    d->called_ip = 0;
    d->frame     = 0;
  }
  
  d->next_instruction = 0;
  d->params_array     = 0;

  return 0;
}


bool_t bt_next(struct bt_data *d)
{
  if (fomit_frame_pointer != _NOT_SET)
    return FALSE;


  if (!d->called_ip) {
    d->called_ip = (vaddr_t)__builtin_return_address(0);
    d->frame     = (vaddr_t)__builtin_frame_address(1);
  }

  if (!d->frame)
    return FALSE;

  d->next_instruction = d->called_ip;
  d->params_array     = d->frame + 8;

  d->called_ip = *((vaddr_t*) (d->frame + 4));
  d->frame     = *((vaddr_t*) d->frame);

  return TRUE;
}


bool_t bt_is_enabled()
{
  return (fomit_frame_pointer == _NOT_SET);
}
