/*
 * Copyright (C) 2000, Thomas PETAZZONI 
 * http://kos.enix.org
 *
 * Debugging functions for kernel
 *
 * @(#) $Id: debug.h,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */


#ifndef __KOS_DEBUG_H__
#define __KOS_DEBUG_H__

#include <kos/macros.h>
#include <kos/types.h>

#include <debug/bochs.h>
#include <debug/serial.h>
#include <lib/stdio.h>
#include <arch/debug/debug.h>


/* Memory dump -- Calls __dbg_printk */
EXPORT void __dbg_dumpmem(char *src, unsigned long size);

/* Symbol lookup */
EXPORT void addr2sym(addr_t addr, char *buffer, size_t len);
EXPORT int dump_symbol_map(void);
EXPORT int __dbg_lookup_sym(addr_t addr); /* calls __dbg_printk */

EXPORT int __dbg_printk(char *fmt, ...);
EXPORT int __dbg_enable_output(int method);
EXPORT int __dbg_disable_output(int method);

/* backtrace data structure */
struct bt_data {
  vaddr_t next_instruction;  /* the opcode right after the 'call function'  */
  vaddr_t params_array; /* Params passed to the function before being called */

  /* For private use by the bt stuff */
  vaddr_t priv0, priv1;

};

/* Backtrace */
struct cpu_state_s;
struct thread;
EXPORT int bt_init(struct bt_data *d, struct cpu_state_s* cpu_state /* Or NULL */);
EXPORT bool_t bt_next(struct bt_data *d);
EXPORT bool_t bt_is_enabled(void); /* FALSE when gcc -fomit-frame-pointer */
EXPORT int __dbg_backtrace(struct thread* thread /*Or get_current_thread()*/,
			   struct cpu_state_s* /*Or NULL*/); /* Calls __dbg_printk */

/* Returns the size of the decoded instruction, or -1 on error */
EXPORT int disasm(vaddr_t vaddr, char *text);
EXPORT int __dbg_disasm(vaddr_t vaddr, int nb_instr);

#define _B_RED      "\033[31m"
#define _B_GREEN    "\033[32m"
#define _B_KKDOI    "\033[33m" 
#define _B_BLUE     "\033[34m"
#define _B_VIOLET   "\033[35m"
#define _B_CYAN     "\033[36m"
#define _B_NSFR     "\033[42m"
#define _B_NSFVE    "\033[43m"
#define _B_NSFKKDOI "\033[44m"
#define _B_NSFB     "\033[45m"
#define _B_NSFVI    "\033[46m"
#define _B_NSFC     "\033[47m"
#define _B_BF       "\033[1m"
#define _B_NORM     "\033[m"

#define DEBUG_SERIAL0 0
#define DEBUG_SERIAL1 1
#define DEBUG_CONSOLE 2
#define DEBUG_BOCHS   3

#define DEBUG_PRINT __dbg_printk

/*
 * DEBUG_PRINT with file/function/line indication.
 */
#define DEBUG_PRINT_VERBOSE(fmt, args...) ({ \
  DEBUG_PRINT("[%s@%s:%d] ", __PRETTY_FUNCTION__, __FILE__, __LINE__); \
  DEBUG_PRINT(fmt, ## args); \
})


# define SEVERE_PRINT(fmt,args...) ({  \
  __dbg_printk(fmt, ## args); \
  printk(fmt, ## args);       \
})

#define SEVERE_PRINT_VERBOSE(fmt, args...) ({ \
  SEVERE_PRINT("[%s@%s:%d] ", __PRETTY_FUNCTION__, __FILE__ , __LINE__); \
  SEVERE_PRINT(fmt, ## args); \
})

void __dbg_halt_handler(void);
#ifdef DONT_CALL_UNSAFE_HALT_HANDLER
#define __halt_handler()
#else
#define __halt_handler __dbg_halt_handler
#endif
#define HALT() ({ asm volatile("cli"); \
                  __halt_handler(); \
                  for( ; ; ) asm volatile("hlt"); })

#define FAILED() ({ \
  SEVERE_PRINT_VERBOSE("**** System Halted (No explanation) ****\n"); \
  HALT(); \
})

#define FAILED_VERBOSE(fmt,args...) ({ \
  SEVERE_PRINT_VERBOSE("**** System Halted **** :\n"); \
  SEVERE_PRINT(fmt, ## args); \
  SEVERE_PRINT("\n"); \
  HALT(); \
})


#endif /* __KOS_DEBUG_H__ */



