/*
 * Copyright (C) 2001, David Decotigny
 * http://kos.enix.org
 *
 * Debug functions for modules symbols
 *
 * @(#) $Id: syms.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <kos/macros.h>
#include <kos/assert.h>
#include <kos/types.h>

#include <loader/mod.h>
#include <lib/string.h>
#include <lib/stdio.h>

#include <debug/_debug.h>

void addr2sym(addr_t addr, char *buffer, size_t len)
{
  struct kernel_module  *cur_mod;

  if ((!buffer) || (len <= 0))
    return;

  if (! kernel_params) {
    snprintf(buffer, len, "<unknown (0x%x)>", addr);
    return;
  }

  for (cur_mod = kernel_params->modules_head ;
       cur_mod ;
       cur_mod = cur_mod->next) {
    struct defined_symbol *sym, *best_sym = NULL;
    int min_offset;

    addr_t mod_start_addr;
    addr_t mod_end_addr = ((addr_t)cur_mod) + cur_mod->size;

    if (addr > mod_end_addr) /* addr after module */
      continue;

    switch (cur_mod->magic) {
    case MAGIC_RELOC_EXECUTABLE:
      mod_start_addr = cur_mod->u.reloc_executable.contents;
      break;
    default:
      /* Does not contain any symbol table */
      continue;
    }

    if (addr < mod_start_addr) /* addr before module */
      continue;

    /* addr contained by module */

    /* No symbol table available */
    if (!  (cur_mod->u.reloc_executable.def_syms)
	|| ((addr_t)cur_mod->u.reloc_executable.def_syms
	      >= (addr_t)cur_mod->u.reloc_executable.contents)) {
      snprintf(buffer, len, "%s(+0x%x):<unknown>(0x%x)",
	       cur_mod->data, mod_start_addr - addr, addr);
      return;
    }

    /* YES : symbol table is available */
    min_offset = mod_end_addr - mod_start_addr + 1;
    for (sym = (struct defined_symbol*)cur_mod->u.reloc_executable.def_syms ;
	 ((addr_t)sym) < (addr_t)cur_mod->u.reloc_executable.contents ;
	 sym++)
      {
	if (sym->addr > addr)
	  continue;
	
	if (sym->addr + min_offset > addr) {
	  /* found better */
	  best_sym   = sym;
	  min_offset = addr - sym->addr;
	}
      } /* end for */
    
    CONCEPTION_ASSERT(best_sym);
    snprintf(buffer, len, "%s:%s+0x%x", cur_mod->data, best_sym->name,
	     min_offset);
    return;
  }

  snprintf(buffer, len, "<not in a module (0x%x)>", addr);
}


int dump_symbol_map(void) {
  struct kernel_module  *cur_mod;

  if (! kernel_params) {
    return -1;
  }

  for (cur_mod = kernel_params->modules_head ;
       cur_mod ;
       cur_mod = cur_mod->next) {
    addr_t addr;
    
    if (!cur_mod
	|| (cur_mod->magic != MAGIC_RELOC_EXECUTABLE))
      continue;
    
    for (addr = (addr_t)cur_mod->u.reloc_executable.def_syms ;
	 addr < (addr_t)cur_mod->u.reloc_executable.contents ;
	 addr += sizeof(defined_symbol_t))
      {
	defined_symbol_t *ds = (defined_symbol_t*)addr;
	__dbg_printk("%x %s:%s\n",
		     ds->addr, cur_mod->data, ds->name);
      }
  }

  return 0;
}
