
#include <kos/types.h>
#include <arch/mm/mm.h>

#include "diskos/asm.h"

int printf( const char *format, ... ) __attribute__ ((format (printf, 1, 2)));

/* Returns the size of the decoded instruction, or -1 on error */
int disasm(addr_t vaddr, char *text)
{
  struct i386_context ctx;
  struct i386_decode instr;

  memset (&ctx, 0, sizeof (ctx));
  memset (&instr, 0, sizeof (instr));

  ctx.mode = CODE32;
  ctx.base = 0;
  ctx.segment = SEG_CODE_KERNEL_ID;
  ctx.offset  = vaddr;

  if (!i386_decode (&ctx, &instr) || !i386_decode_text (&ctx, &instr, text,
							SX_ATT))
    return -1;

  return instr.length;
}


static void test_disasm()
{
  char txt[256];
  int len;
  addr_t vaddr;
  int offset = 0; 

  while (offset < 100) 
    {
      vaddr = (addr_t) test_disasm + offset;

      len = disasm(vaddr, txt);
      if (len <= 0) {
	printf("!!! Disasm of 0%x returned %d !!!\n", vaddr, len);
      }

      printf("Disassemble instruction at 0x%x (size %d) : %s.\n",
	     vaddr, len, txt);

      offset += len;
    }

}


int main() {
  test_disasm();

  return 0;
}
