/* _fat.h - Copyright (c) 2002 the Kos Project
   fat private definitions

   the Kid Operating System (http://kos.enix.org)
   @(#) $Id: _fat.h,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $ */

#ifndef ___FAT_H__
#define ___FAT_H__

#include <babel/tower/tower.h>
#include <fs/filesystem.h>
#include <fs/fat/fat.h>
#include <kos/types.h>

typedef struct fat_methods_table {
  struct fs_methods_table parent;
} fat_methods_table_t;


typedef struct fat16_bpb 
{
  /* common part of fat16/fat12/fat32 */
  k_ui8_t  bs_jmpboot[3];
  k_ui8_t  bs_oemname[8];
  k_ui16_t bpb_bytspersec;
  k_ui8_t  bpb_secperclus;
  k_ui16_t bpb_rsvdseccnt;
  k_ui8_t  bpb_numfats;
  k_ui16_t bpb_rootentcnt;
  k_ui16_t bpb_totsec16;
  k_ui8_t  bpb_media;
  k_ui16_t bpb_fatsz16;
  k_ui16_t bpb_secpertrk;
  k_ui16_t bpb_numheads;
  k_ui32_t bpb_hiddsec;
  k_ui32_t bpb_totsec32;
  /* specific part for fat 12/16 */
  k_ui8_t  bs_drvnum;
  k_ui8_t  bs_reserved;
  k_ui8_t  bs_bootsig;
  k_ui32_t bs_volid;
  k_ui8_t  bs_vollab[11];
  k_ui8_t  bs_filsystype[8];
} __attribute__((packed)) fat16_bpb_t;


typedef struct fat32_bpb
{
  /* common part of fat16/fat12/fat32 */
  k_ui8_t  bs_jmpboot[3];
  char     bs_oemname[8];
  k_ui16_t bpb_bytspersec;
  k_ui8_t  bpb_secperclus;
  k_ui16_t bpb_rsvdseccnt;
  k_ui8_t  bpb_numfats;
  k_ui16_t bpb_rootentcnt;
  k_ui16_t bpb_totsec16;
  k_ui8_t  bpb_media;
  k_ui16_t bpb_fatsz16;
  k_ui16_t bpb_secpertrk;
  k_ui16_t bpb_numheads;
  k_ui32_t bpb_hiddsec;
  k_ui32_t bpb_totsec32;
  /* specific part for fat 32 */
  k_ui32_t bpb_fatsz32;
  k_ui16_t bpb_extflags;
  k_ui16_t bpb_fsver;
  k_ui32_t bpb_rootclus;
  k_ui16_t bpb_fsinfo;
  k_ui16_t bpb_bkbootsec;
  k_ui8_t  bpb_reserved[12];
  k_ui8_t  bs_drvnum;
  k_ui8_t  bs_reserved1;
  k_ui8_t  bs_bootsig;
  k_ui32_t bs_volid;
  k_ui8_t  bs_vollab[11];
  k_ui8_t  bs_filsystype[8];
} __attribute__((packed)) fat32_bpb_t;

typedef struct direntry
{
  k_ui8_t  filename[8];
  k_ui8_t  extension[3];
  k_ui8_t  attribute;
#define ATTR_READ_ONLY  0x1
#define ATTR_HIDDEN     0x2
#define ATTR_SYSTEM     0x4
#define ATTR_VOLUME_ID  0x8
#define ATTR_DIRECTORY 0x10
#define ATTR_ARCHIVE   0x20
#define ATTR_LONG_NAME (ATTR_READ_ONLY | ATTR_HIDDEN | ATTR_SYSTEM | ATTR_VOLUME_ID)
  k_ui8_t  ntres;
  k_ui8_t  crttimetenth;
  k_ui16_t crttime;
  k_ui16_t crtdate;
  k_ui16_t lastaccessdate;
  k_ui16_t firstclus_hi;
  k_ui16_t lastwritetime;
  k_ui16_t lastwritedate;
  k_ui16_t firstclus_lo;
  k_ui32_t filesize;
} __attribute__((packed)) direntry_t;

/* Structure describing a FAT partition */
typedef struct fat_translator
{
  struct translator parent;
  
  k_ui16_t bytes_per_sector;    /* number of bytes per sector */
  k_ui8_t  sector_per_cluster;  /* number of sector per cluster */
  k_ui16_t root_dir_start;      /* starting cluster of root directory */
  k_ui16_t root_dir_size;       /* size in cluster of root directory */
  k_ui16_t data_start;          /* starting cluster of data */
  k_ui16_t fat_start;           /* FATs start = number of reserved sectors */
  k_ui16_t fat_size;            /* size (in cluster) of one FAT */
  k_ui8_t  fat_nb;              /* number of FATs */
  k_ui32_t total_sector_nb;     /* total number of sectors */
  enum { FAT12, FAT16, FAT32 } type;

  struct shadow_resource *part;

} fat_translator_t;

int fat_constructor (struct translator *trans, void *rc);
int fat_check_capability( struct shadow_resource *part);
int fat_read(struct shadow_resource *sr, char *buf, offset_t offset, size_t len);
int fat_init_sr(struct shadow_resource *sr);

int fat_getsize(struct resource *res);

int __init_fat(void);

#endif /* ___FAT_H__ */


