#ifndef ___IDE_H__
#define ___IDE_H__

#include <ide/ide.h>

/* Registers */
#define ATA_DATA                        0x00    /* (R/W) Data register */

#define ATA_ERROR                       0x01    /* (R) error register */

#define ATA_PRECOMP                     0x01    /* (W) precompensation */

#define ATA_SECTOR_COUNT                0x02    /* (W) sector count register */

#define ATA_SECTOR_NUMBER               0x03    /* (W) sector number register */

#define ATA_CYL_LSB                     0x04    /* (W) cylinder# LSB */

#define ATA_CYL_MSB                     0x05    /* (W) cylinder# MSB */

#define ATA_DRIVE                       0x06    /* (W) drive/head/lba */
#define         ATA_D_IBM               0xa0    /* bits that must be set */
#define         ATA_D_LBA               0x40    /* use LBA ? */
#define         ATA_D_MASTER            0x00    /* select master */
#define         ATA_D_SLAVE             0x10    /* select slave */


#define ATA_STATUS                      0x07    /* (R) status register */
#define         ATA_S_ERROR             0x01    /* error */
#define         ATA_S_INDEX             0x02    /* index */
#define         ATA_S_CORR              0x04    /* data corrected */
#define         ATA_S_DRQ               0x08    /* data request */
#define         ATA_S_DSC               0x10    /* drive Seek Completed */
#define         ATA_S_DWF               0x20    /* drive write fault */
#define         ATA_S_DRDY              0x40    /* drive ready */
#define         ATA_S_BSY               0x80    /* busy */

#define ATA_CMD                         0x07    /* (W) command register */
#define         ATA_C_ATA_IDENTIFY      0xec    /* get ATA params */
#define         ATA_C_ATAPI_IDENTIFY    0xa1    /* get ATAPI params*/
#define         ATA_C_READ              0x20    /* read command */
#define         ATA_C_WRITE             0x30    /* write command */
#define         ATA_C_READ_MULTI        0xc4    /* read multi command */
#define         ATA_C_WRITE_MULTI       0xc5    /* write multi command */
#define         ATA_C_SET_MULTI         0xc6    /* set multi size command */
#define         ATA_C_PACKET_CMD        0xa0    /* set multi size command */

#define ATA_ALTPORT                     0x206   /* (R) alternate
                                                   Status register */

#define ATA_DEVICE_CONTROL              0x206   /* (W) device control
                                                   register */
#define         ATA_A_nIEN              0x02    /* disable interrupts */
#define         ATA_A_RESET             0x04    /* RESET controller */
#define         ATA_A_4BIT              0x08    /* 4 head bits */



/* Magic numbers used to detect ATAPI devices */
#define ATAPI_MAGIC_LSB                 0x14
#define ATAPI_MAGIC_MSB                 0xeb

/* This structure describe the informations returned by the Identify
   Device command (imposed by the ATA standard) */
typedef struct identify_device_info
{
  k_ui16_t general_config_info;      /* 0 */
  k_ui16_t nb_logical_cylinders;     /* 1 */
  k_ui16_t reserved1;                /* 2 */
  k_ui16_t nb_logical_heads;         /* 3 */
  k_ui16_t unformatted_bytes_track;  /* 4 */
  k_ui16_t unformatted_bytes_sector; /* 5 */
  k_ui16_t nb_logical_sectors;       /* 6 */
  k_ui16_t vendor1[3];               /* 7-9 */
  k_ui8_t  serial_number[20];        /* 10-19 */
  k_ui16_t buffer_type;              /* 20 */
  k_ui16_t buffer_size;              /* 21 */
  k_ui16_t ecc_bytes;                /* 22 */
  k_ui8_t  firmware_revision[8];     /* 23-26 */
  k_ui8_t  model_number[40];         /* 27-46 */
  k_ui8_t  max_multisect;            /* 47 */
  k_ui8_t  vendor2;
  k_ui16_t dword_io;                 /* 48 */
  k_ui8_t  vendor3;                  /* 49 */
  k_ui8_t  capabilities;
  k_ui16_t reserved2;                /* 50 */
  k_ui8_t  vendor4;                  /* 51 */
  k_ui8_t  pio_trans_mode;
  k_ui8_t  vendor5;                  /* 52 */
  k_ui8_t  dma_trans_mode;
  k_ui16_t fields_valid;             /* 53 */
  k_ui16_t cur_logical_cylinders;    /* 54 */
  k_ui16_t cur_logical_heads;        /* 55 */
  k_ui16_t cur_logical_sectors;      /* 56 */
  k_ui16_t capacity1;                /* 57 */
  k_ui16_t capacity0;                /* 58 */
  k_ui8_t  multsect;                 /* 59 */
  k_ui8_t  multsect_valid;
  k_ui32_t lba_capacity;             /* 60-61 */
  k_ui16_t dma_1word;                /* 62 */
  k_ui16_t dma_multiword;            /* 63 */
  k_ui16_t pio_modes;                /* 64 */
  k_ui16_t min_mword_dma;            /* 65 */
  k_ui16_t recommended_mword_dma;    /* 66 */
  k_ui16_t min_pio_cycle_time;       /* 67 */
  k_ui16_t min_pio_cycle_time_iordy; /* 68 */
  k_ui16_t reserved3[11];            /* 69-79 */
  k_ui16_t major_version;            /* 80 */
  k_ui16_t minor_version;            /* 81 */
  k_ui16_t command_sets1;            /* 82 */
  k_ui16_t command_sets2;            /* 83 dixit lk : b14 (smart enabled) */
  k_ui16_t reserved4[4];             /* 84-87 */
  k_ui16_t dma_ultra;                /* 88 dixit lk */
  k_ui16_t reserved5[37];            /* 89-125 */
  k_ui16_t last_lun;                 /* 126 */
  k_ui16_t reserved6;                /* 127 */
  k_ui16_t security;                 /* 128 */
  k_ui16_t reserved7[127];
} identify_device_info_t __attribute__((packed));



/* ide (_ide.c) */
int _ide_add_op_polled_mode(struct shadow_resource *sr, int type, 
			    k_ui32_t sector_num, k_ui16_t *buffer, size_t len);
int _ide_get_drv_infos(struct ide_translator *);
int __init_ide(void);

/* ide (_ide_babel.c) */
int __ide_add_hd(char *str, struct ide_dev *);
int __init_ide_babel(void);

/* partition (_part.c) */
int   _part_get_partition_table(struct shadow_resource *sr);
char *_part_partition_type(int type);


#endif /* ___IDE_H__ */



