#include <babel/tower/tower.h>
#include <kos/types.h>
#include <kos/macros.h>
#include <kos/assert.h>
#include <ide/_ide.h>
#include <part/part.h>


static int ide_constructor(struct translator *trans, void *rc);
static int ide_destructor(struct translator *trans, void *rc);
static int ide_raw_write(struct shadow_resource *sr, char *buf,
			 offset_t offset, size_t len);
static int ide_raw_read(struct shadow_resource  *sr, char *buf,
			offset_t offset, size_t len);


struct kernel_methods_table ide_kmt = {
  .read=        ide_raw_read,
  .write=       ide_raw_write,
  .init_sr=     NULL,
  .init_res=    NULL,
  .cleanup_sr=  NULL,
  .cleanup_res= NULL
};

interface_t ide_interface = {
  .name=        "ide", 
  .family=      "disk",
  .description= "Disk Interface",
  .authors=     "Thomas Petazzoni",
  .depends=     NULL,

  .version= { 0, 0, 1 },       

  .prev=       NULL, 
  .next=       NULL,        
  .trans_list= NULL,              

  .max_trans_instance= 8,                 
  .trans_size=         sizeof(struct ide_translator),
  .res_size=           0,
  .sr_size=            0,
  .trans_cnt=          0,
  .trans_greatest_id=  0,

  .constructor=  ide_constructor,
  .destructor=   ide_destructor,

  .kernel_methods= &ide_kmt,
  
  .methods_table= NULL,
  .shared_data=   NULL
};


static int ide_constructor(struct translator *trans, void *rc)
{
  ((struct ide_translator *)trans)->ctrl   = ((struct ide_dev *) rc)->controller;
  ((struct ide_translator *)trans)->device = ((struct ide_dev *) rc)->device;

  _ide_get_drv_infos((struct ide_translator *)trans);

  _part_request_for_registering(trans, ((struct ide_translator *)trans)->blk_size);

  return 0;
}

static int ide_destructor(struct translator *trans, void *rc)
{
  UNUSED(trans);
  UNUSED(rc);
  return 0;
}

static int ide_raw_read(struct shadow_resource  *sr, char *buf,
			offset_t offset, size_t len)
{
#define ide_trans ((struct ide_translator *) sr->translator)

  return _ide_add_op_polled_mode(sr, READ, offset, (k_ui16_t *) buf, 
				 ALIGN_SUP(len, ide_trans->blk_size));
}

static int ide_raw_write(struct shadow_resource *sr, char *buf,
			 offset_t offset, size_t len)
{
#define ide_trans ((struct ide_translator *) sr->translator)

  return _ide_add_op_polled_mode(sr, WRITE, offset, (k_ui16_t *) buf, 
				 ALIGN_SUP(len, ide_trans->blk_size));

  return 0;
}

int __ide_add_hd(char *name, struct ide_dev *rc)
{
  struct translator *trans;
  
  trans = bbl_add_translator(&ide_interface, name,(void *)rc);
  if (!trans)
    {
      printk("Error while registering harddrive\n");
      return -1;
    }

  return 0;
}

int __init_ide_babel(void)
{
  if (bbl_add_interface(&ide_interface) != 0)
    return -1;
 
  return 0;
}


