/*
 * Copyright (C) 2000, Thomas Petazzoni
 * http://kos.enix.org
 *
 * Partition table reader
 *
 * @(#) $Id: _part.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <debug/debug.h>
#include <kmem/kmem.h>
#include <liblist/liblist.h>
#include <lib/string.h>
#include <ide/_ide.h>


typedef struct partition_entry 
{
  k_ui8_t active;
  k_ui8_t start_dl;
  k_ui16_t start_cylinder;
  k_ui8_t type;
  k_ui8_t end_dl;
  k_ui16_t end_cylinder;
  k_ui32_t lba;
  k_ui32_t size;
} partition_entry_t;

/* Given a harddisk, this function reads the partition table, and
   create the partition list */

int _part_get_partition_table (harddisk_t *harddisk)
{
  int i;
  partition_entry_t *p;
  partition_t *new;
  int extstart = 0;  
  int extsup = 0;
  int partnum = 1;
  k_ui8_t buffer[512];
  
  if(ide_add_op_polled_mode(harddisk, READ, 0, (k_ui16_t *) buffer) < 0)
    {
      __dbg_printk("Error while reading bootsector\n");
      return -1;
    }

  p = (partition_entry_t *) (buffer+446);
  harddisk->partition_list = NULL;

  /* Manage primary partitions */
  for (i = 0; i < 4; i++)
    {
      if (p[i].size == 0)
	continue;

      new = kmalloc(sizeof(partition_t));
      if(new == NULL)
	{
	  __dbg_printk("Couldn't allocate memory\n");
	  return -1;
	}

      if(p[i].type != 0x5)
	{
	  new->is_active = (p[i].active == 0) ? 0 : 1;
	  new->fs_type   = p[i].type;
	  new->start     = p[i].lba;
	  new->size      = p[i].size;
	  new->number    = partnum++;
	  list_add_tail(harddisk->partition_list, new);
	}
      else
	extstart = p[i].lba;
    }

  /* Manage extended partitions */
  while(extstart != 0)
    {
      if(partnum > 6)
	break;
      /*
      __dbg_printk("Reading MBR for extended partition : %d\n",
	     extstart+extsup);
      */

      memset(buffer, 0, 512);
      if(ide_add_op_polled_mode(harddisk, READ, extstart+extsup, 
				(k_ui16_t *) buffer) < 0)
	{
	  __dbg_printk("Error while reading sector %d\n", extstart+extsup);
	  return -1;
	}
      
      p = (partition_entry_t *) (buffer + 446);

      new = kmalloc(sizeof(partition_t));
      if(new == NULL)
	{
	  __dbg_printk("Couldn't allocate memory\n");
	  return -1;
	}

      new->is_active = (p[0].active == 0) ? 0 : 1;
      new->fs_type   = p[0].type;
      new->start     = extstart + extsup + p[0].lba;
      new->size      = p[0].size;
      new->number    = partnum++;

      list_add_tail(harddisk->partition_list, new);

      extsup = p[1].lba;
      if(extsup == 0)
	break;
    }

  return 0;
}

char *_part_partition_type(int type)
{
  switch(type)
    {
    case 0xe:
    case 0x6:
      return "FAT16";
    case 0xb:
    case 0xc:
      return "FAT32";
    case 0x82:
      return "Linux Swap";
    case 0x83:
      return "Linux";
    default:
      return "Unknow";
    }
}
