/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Exception Management functions.
 * Part of the idt.o module
 *
 * @(#) $Id: _exception.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <kos/macros.h>
#include <kos/assert.h>
#include <kos/asm.h>
#include <idt/_exception.h>
#include <idt/_idt.h>

static cpu_state_t *exception_without_error_code(int number,
						   cpu_state_t *cpu_state);
static cpu_state_t *exception_with_error_code(int number,
						cpu_state_t *cpu_state);
static cpu_state_t *intel_reserved(int number, cpu_state_t *cpu_state);

exception_handler_t exception_handler[NUM_EXCEPTION] =
{
  exception_without_error_code, /*  0 */
  exception_without_error_code, /*  1 */
  exception_without_error_code, /*  2 */
  exception_without_error_code, /*  3 */
  exception_without_error_code, /*  4 */
  exception_without_error_code, /*  5 */
  exception_without_error_code, /*  6 */
  exception_without_error_code, /*  7 */
  exception_with_error_code,    /*  8 */
  exception_without_error_code, /*  9 */
  exception_with_error_code,    /* 10 */
  exception_without_error_code, /* 11 */
  exception_without_error_code, /* 12 */
  exception_with_error_code,    /* 13 */
  exception_without_error_code, /* 14 */
  intel_reserved,               /* 15 */
  exception_without_error_code, /* 16 */
  exception_with_error_code,    /* 17 */
  exception_without_error_code, /* 18 */
  intel_reserved, /* 19 */
  intel_reserved, /* 20 */
  intel_reserved, /* 21 */
  intel_reserved, /* 22 */
  intel_reserved, /* 23 */
  intel_reserved, /* 24 */
  intel_reserved, /* 25 */
  intel_reserved, /* 26 */
  intel_reserved, /* 27 */
  intel_reserved, /* 28 */
  intel_reserved, /* 29 */
  intel_reserved, /* 30 */
  intel_reserved, /* 31 */
};


exception_handler_t register_exception_handler(short num_ex, 
					       exception_handler_t new_ex)
{
  exception_handler_t old_ex;

  if ((num_ex < 0) || (num_ex >= NUM_EXCEPTION))
    return NULL;

  old_ex = exception_handler[num_ex];
  exception_handler[num_ex] = new_ex;

  return old_ex;
}


addr_t get_exception_handler_address(int num_ex)
{
  addr_t ex_addr;
  
  CONCEPTION_ASSERT((num_ex >= 0) && (num_ex < NUM_EXCEPTION));

  ex_addr = _idt[num_ex+EXCEPTION_BASE].offset_low;
  ex_addr |= (_idt[num_ex+EXCEPTION_BASE].offset_high << 16);
  
  return ex_addr;
}



static cpu_state_t *exception_without_error_code(int number, 
						   cpu_state_t *cpu_state)
{
  __dbg_printk("Uncaught Exception Number %d\n", number);
  dump_cpu_state(cpu_state);
  __dbg_lookup_sym(cpu_state->eip);

  HALT();
  return cpu_state;
}

static cpu_state_t *exception_with_error_code(int number, 
						cpu_state_t *cpu_state)
{
  char buffer[256];
  addr_t *stack;
  int i;
  
  __dbg_printk("Uncaught Exception Number %d with error code %x (0x%x)\n", 
  		 number, cpu_state->error_code, (unsigned) cpu_state);
  dump_cpu_state(cpu_state);
  __dbg_lookup_sym(cpu_state->eip);
  disasm(cpu_state->eip, buffer);
  __dbg_printk("Instruction : %s\n", buffer);
  for(stack = (addr_t *) (cpu_state), i = 0;
      i < 20;
      i+=1) {
    if(i==15)
      __dbg_lookup_sym((*(stack+i)));
    
    __dbg_printk("Stack-%d (0x%x) 0x%x\n", 
		 i,
		 (unsigned)(stack+i), (unsigned)(*(stack+i)));
  }

  HALT();
  return cpu_state;
}


static cpu_state_t * intel_reserved(int number, cpu_state_t *cpu_state)
{
  __dbg_printk("Intel Reserved Interrupt number %d\n", number);
  return cpu_state;
}


