#include <kos/asm.h>
#include <idt/_i8259.h>

void __i8259_setup(void)
{
  // Send ICW1
  outb(0x11, PIC_MASTER);
  outb(0x11, PIC_SLAVE);

  // Send ICW2
  // 0x20 is first IRQ for master
  // 0x28 is first IRQ for slave
  outb(0x20, PIC_MASTER+1);
  outb(0x28, PIC_SLAVE+1);
  
  // Send ICW3
  outb(0x4, PIC_MASTER+1);
  outb(0x2, PIC_SLAVE+1);
  
  // Send ICW4
  outb(0x1, PIC_MASTER+1);
  outb(0x1, PIC_SLAVE+1);
  
  // Send OCW1 
  // Closing all IRQs : waiting for a correct handler The only IRQ
  // enabled is the cascade (that's why we use 0xFB for the master)
  outb(0xFB, PIC_MASTER+1);
  outb(0xFF, PIC_SLAVE+1);
}

void i8259_enable_irq(short numirq)
{
    if(numirq < 8)
        /*  irq on master PIC */
        outb((inb(0x21) & ~(1 << numirq)), 0x21);
    else
        /*  irq on slave PIC */
      outb((inb(0xa1) & ~(1 << (numirq-8))), 0xa1);
}

void i8259_disable_irq(short numirq)
{
    if(numirq < 8)
        /*  irq on master PIC */
        outb((inb(0x21) | (1 << numirq)), 0x21);
    else
       /*  irq on slave PIC */
      outb((inb(0xa1) | (1 << (numirq-8))), 0xa1);
}



