/*
 * Copyright (C) 2000, Thomas PETAZZONI, David DECOTIGNY, Coffeeman
 * http://kos.enix.org
 *
 * == >Interrupt management, assembly part. <==
 *
 * @(#) $Id: _idt.S,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#define ASM_SOURCE 1
	
#include "_idt.h"
		
#define CODE_SEL 0x8
#define DOUBLE_FAULT_TSS_SEL (0x6 << 3)
				
.file "_idt.S"

.text
.extern irq_handler
.extern exception_handler
.extern syscall_handler
.extern reschedule_after_interrupt
/* FIXME : ONE PER CPU */
.extern hw_isr_nested_level
.extern spawn_dsr
				
	/* These pre-handlers are for exception without error code */
.irp id,                              \
	 DIVIDE_ERROR, 	              \
	 DEBUG,                       \
	 NMI_INTERRUPT,               \
	 BREAKPOINT,                  \
	 OVERFLOW,                    \
	 BOUND_RANGE_EXCEDEED,        \
	 INVALID_OPCODE,              \
	 DEVICE_NOT_AVAILABLE,        \
	 COPROCESSOR_SEGMENT_OVERRUN, \
	 INTEL_RESERVED_1,            \
	 FLOATING_POINT_ERROR,        \
	 MACHINE_CHECK,               \
	 INTEL_RESERVED_2,            \
	 INTEL_RESERVED_3,            \
	 INTEL_RESERVED_4,            \
	 INTEL_RESERVED_5,            \
	 INTEL_RESERVED_6,            \
	 INTEL_RESERVED_7,            \
	 INTEL_RESERVED_8,            \
	 INTEL_RESERVED_9,            \
	 INTEL_RESERVED_10,           \
	 INTEL_RESERVED_11,           \
	 INTEL_RESERVED_12,           \
	 INTEL_RESERVED_13,           \
	 INTEL_RESERVED_14

	.p2align 2, 0x90
	exception_handler_\id:
	.type exception_handler_\id,@function
	
		/* Fake error code */
		pushl $0
		/* Backup the context */
		pushl %ebp
		movl %esp, %ebp

		pushl %edi
		pushl %esi
		pushl %edx
		pushl %ecx
		pushl %ebx
		pushl %eax
		subl  $2,%esp
		pushw %ss
		pushw %ds
		pushw %es
		pushw %fs
		pushw %gs
	
		/* Call the handler with interrupt number and 0 as 
		 * arguments 
		 */
		pushl %esp
		pushl $\id
		leal  exception_handler,%edi
		call  *\id*4(%edi)
		addl  $8, %esp

		movl  %eax, %esp
	
		/* Restore the context */
		popw  %gs
		popw  %fs
		popw  %es
	        popw  %ds
		popw  %ss
		addl  $2,%esp
		popl  %eax
		popl  %ebx
		popl  %ecx
		popl  %edx
		popl  %esi
		popl  %edi

		popl  %ebp
		/* Remove fake error code */
		addl $4, %esp
		iret
.endr


.irp id,		\
	DOUBLE_FAULT

	.p2align 2, 0x90

	exception_handler_\id:
	.type exception_handler_\id,@function
	
		/* ret eflags */
		/* ret cs */
		/* ret eip */
		/* Error code */
	
		/* Do not backup the context, because
		   the current stack is specific to the
		   double fault handler */
			
		/* Call the handler with interrupt number as 
		 * arguments. cpu_context has no meaning (set to 0) 
		 */
		pushl $0
		pushl $\id
		leal  exception_handler,%edi
		call  *\id*4(%edi)
		addl  $8, %esp


		/* Error code isn't compatible with iretd */
		addl $4, %esp

		iret
		jmp exception_handler_\id

.endr	
	
	/* These pre-handlers are for exception with error code */
.irp id,                              \
	INVALID_TSS,                  \
	SEGMENT_NOT_PRESENT,          \
	STACK_SEGMENT_FAULT,          \
	GENERAL_PROTECTION,           \
	PAGE_FAULT,                   \
	ALIGNEMENT_CHECK
	
	.p2align 2, 0x90
	exception_handler_\id:
	.type exception_handler_\id,@function
	
		/* ret eflags */
		/* ret cs */
		/* ret eip */
		/* Error code */
	
		/* Backup the context */
		pushl %ebp        
		movl %esp, %ebp

		pushl %edi        
		pushl %esi        
		pushl %edx        
		pushl %ecx        
		pushl %ebx        
		pushl %eax
		subl  $2,%esp
		pushw %ss
		pushw %ds
		pushw %es
		pushw %fs
		pushw %gs

	
		/* Call the handler with interrupt number and error_code as 
		 * arguments 
		 */
		pushl %esp
		pushl $\id
		leal  exception_handler,%edi
		call  *\id*4(%edi)
		addl  $8, %esp

		movl %eax, %esp
	
		/* Restore the context */
		popw  %gs
		popw  %fs
		popw  %es
	        popw  %ds
		popw  %ss
		addl  $2,%esp
		popl  %eax
		popl  %ebx
		popl  %ecx
		popl  %edx
		popl  %esi
		popl  %edi
		popl  %ebp

		/* Error code isn't compatible with iretd */
		addl $4, %esp

		iret
.endr
	/* These pre-handlers are for IRQ (Master PIC) */
.irp id,                              \
	 IRQ_TIMER,                   \
	 IRQ_KEYBOARD,                \
	 IRQ_SLAVE_PIC,               \
	 IRQ_COM2,                    \
	 IRQ_COM1,                    \
	 IRQ_LPT2,                    \
	 IRQ_FLOPPY,                  \
	 IRQ_LPT1,                    \

	.p2align 2, 0x90

	irq_handler_\id:
	.type irq_handler_\id,@function

		incl hw_isr_nested_level
	
		/* Fake error code */
		pushl $0
		/* Backup the context */
		pushl %ebp
		movl %esp, %ebp

	
		pushl %edi
		pushl %esi
		pushl %edx
		pushl %ecx
		pushl %ebx
		pushl %eax
		subl  $2,%esp
		pushw %ss
		pushw %ds
		pushw %es
		pushw %fs
		pushw %gs
	
		/* Call the handler with interrupt number and 0 as 
		 * arguments 
		 */
		pushl %esp
		pushl $\id
		leal   irq_handler,%edi
		call  *\id*4(%edi)
		addl  $8, %esp

		/* Send EOI to PIC */
		movb  $0x20, %al
		outb  %al, $0x20

		/* Spawn DSR only for most external IRQ */
		cmpl $1, hw_isr_nested_level
		jne 1f
		call spawn_dsr

		pushl %esp
		call reschedule_after_interrupt
		addl $4, %esp
		movl %eax, %esp
1:
	
		/* Restore the context */
		popw  %gs
		popw  %fs
		popw  %es
	        popw  %ds
		popw  %ss
		addl  $2,%esp
		popl  %eax
		popl  %ebx
		popl  %ecx
		popl  %edx
		popl  %esi
		popl  %edi
		popl  %ebp

		/* Remove fake error code */
		addl  $4, %esp

		decl hw_isr_nested_level
		iret

.endr

	/* These pre-handlers are for IRQ (Slave PIC) */
.irp id,                              \
	 IRQ_8_NOT_DEFINED,           \
	 IRQ_RESERVED_1,              \
	 IRQ_RESERVED_2,              \
	 IRQ_RESERVED_3,              \
	 IRQ_RESERVED_4,              \
	 IRQ_COPROCESSOR,             \
	 IRQ_HARDDISK,                \
	 IRQ_RESERVED_5

	.p2align 2, 0x90

	irq_handler_\id:
	.type irq_handler_\id,@function

		incl hw_isr_nested_level
	
		/* Fake error code */
		pushl $0
		/* Backup the context */
		pushl %ebp
		movl %esp, %ebp

	
		pushl %edi
		pushl %esi
		pushl %edx
		pushl %ecx
		pushl %ebx
		pushl %eax
		subl  $2,%esp
		pushw %ss
		pushw %ds
		pushw %es
		pushw %fs
		pushw %gs
	
		/* Call the handler with interrupt number and 0 as 
		 * arguments 
		 */
		pushl %esp
		pushl $\id
		leal   irq_handler,%edi
		call  *\id*4(%edi)
		addl  $8, %esp

		/* Send EOI to PIC */
		movb  $0x20, %al
		outb  %al, $0xa0
		outb  %al, $0x20

		/* Spawn DSR only for most external IRQ */
		cmpl $1, hw_isr_nested_level
		jne 1f
		call spawn_dsr

		pushl %esp
		call reschedule_after_interrupt
		addl $4, %esp
		movl %eax, %esp
1:
	
		/* Restore the context */
		popw  %gs
		popw  %fs
		popw  %es
	        popw  %ds
		popw  %ss
		addl  $2,%esp
		popl  %eax
		popl  %ebx
		popl  %ecx
		popl  %edx
		popl  %esi
		popl  %edi
		popl  %ebp

		/* Remove fake error code */
		addl  $4, %esp

		decl hw_isr_nested_level
		iret

.endr
	
	/* These pre-handlers are for softwares interrupts */
.irp id,                              \
	 KOS_SYSCALL
	
	.p2align 2, 0x90
	syscall_handler_\id:
	.type syscall_handler_\id,@function

		/* Fake error code */
		pushl $0
		/* Backup the context */
		pushl %ebp
		movl %esp, %ebp

		pushl %edi
		pushl %esi
		pushl %edx
		pushl %ecx
		pushl %ebx
		pushl %eax
		subl  $2,%esp
		pushw %ss
		pushw %ds
		pushw %es
		pushw %fs
		pushw %gs

	
		/* Call the handler with interrupt number and 0 as 
		 * arguments 
		 */
		pushl %esp
		pushl $\id
		leal  syscall_handler,%edi
		call  *\id*4(%edi)
		addl  $8, %esp

		/* Restore the context */
		pushl %esp
		call reschedule_after_interrupt
		addl $4, %esp
		movl %eax, %esp

		popw  %gs
		popw  %fs
		popw  %es
	        popw  %ds
		popw  %ss
		addl  $2,%esp
		popl  %eax
		popl  %ebx
		popl  %ecx
		popl  %edx
		popl  %esi
		popl  %edi
	
		popl  %ebp
		/* Remove fake error code */
		addl $4, %esp
		iret
.endr
	
/* ASSUMES cli ! */
.globl _init_idt
_init_idt:
	.type _init_idt, @function
	pushl %ebp
	movl %esp,%ebp

	/* Backuping registers, because GCC (with -O2) doesn't know
	which registers we're using, and which registers it can use
	for its optimization */
	push %edi
	push %eax
	push %edx
	
	/* cli is undoubtedly already done */
	
	movl $_idt, %edi
	addl $8*EXCEPTION_BASE, %edi
	
	.irp id, 0,                             \
	 1,  2,  3,  4,  5,  6,  7,  8,  9, 10, \
	11, 12, 13, 14, 15, 16, 17, 18, 19, 20, \
	21, 22, 23, 24, 25, 26, 27, 28, 29, 30, \
	31
		
		movl $exception_handler_\id, %edx

		movl $(CODE_SEL << 16), %eax

		movw %dx, %ax

		movw $0x8E00, %dx

		movl %eax, (%edi)
		movl %edx, 4(%edi)

		addl $8, %edi
	.endr

	.irp id, DOUBLE_FAULT
		movl $(DOUBLE_FAULT_TSS_SEL << 16), %eax

		movl $0x8500, %edx

		movl $_idt+8*(EXCEPTION_BASE+DOUBLE_FAULT), %edi
		
		movl %eax, (%edi)
		movl %edx, 4(%edi)
	.endr

	movl $_idt, %edi
	addl $8*IRQ_BASE, %edi
	
	.irp id, 0,                             \
	 1,  2,  3,  4,  5,  6,  7,  8,  9, 10, \
	11, 12, 13, 14, 15
		
		movl $irq_handler_\id, %edx

		movl $(CODE_SEL << 16), %eax

		movw %dx, %ax

		movw $0x8E00, %dx

		movl %eax, (%edi)
		movl %edx, 4(%edi)

		addl $8, %edi
	.endr

	/* Make SYSCALL availaible to CPL3 */

	movl $_idt, %edi
	addl $8*SYSCALL_BASE, %edi

	.irp id, KOS_SYSCALL
		movl $syscall_handler_\id, %edx

		movl $(CODE_SEL << 16), %eax

		movw %dx, %ax

		movw $0xEE00, %dx

		movl %eax, (%edi)
		movl %edx, 4(%edi)

		addl $8, %edi
	.endr
	

	lidt idtr	

	pop %edx
	pop %eax
	pop %edi
		
	leave
	ret	

	
.globl exception_8_alias
.type    exception_8_alias,@function
	.set exception_8_alias,exception_handler_8

			
.data
	.p2align 3, 0x90
.globl _idt
_idt:
		.rept IDT_ENTRIES
		.quad 0
		.endr
idt_fin:

	.p2align 3, 0x90	
idtr:
		.word (idt_fin) - (_idt) - 1
		.long (_idt)
