/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * IRQ Management functions.
 * Part of the idt.o module
 *
 * @(#) $Id: _irq.c,v 1.1.1.1 2003/04/27 12:49:48 thomas Exp $
 */

#include <kos/macros.h>
#include <kos/assert.h>
#include <idt/_irq.h>
#include <idt/_idt.h>

int hw_isr_nested_level;
static void uncaught_irq(int number, cpu_state_t *cpu_state);

irq_handler_t irq_handler[NUM_IRQ] = { uncaught_irq, 
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq,
				       uncaught_irq };


irq_handler_t register_irq_handler(short numint, 
				   irq_handler_t new_isr)
{
  irq_handler_t old_isr;

  if ((numint < 0) || (numint >= NUM_IRQ))
    return NULL;

  old_isr = irq_handler[numint];
  irq_handler[numint] = new_isr;

  i8259_enable_irq(numint);

  return old_isr;
}

int get_hw_isr_nested_level()
{
  return hw_isr_nested_level;
}

addr_t get_irq_handler_address(int irq_number)
{
  addr_t isr_addr;
  
  CONCEPTION_ASSERT((irq_number >= 0) && (irq_number < NUM_IRQ));

  isr_addr = _idt[irq_number+IRQ_BASE].offset_low;
  isr_addr |= (_idt[irq_number+IRQ_BASE].offset_high << 16);
  
  return isr_addr;
}

// These are the default handlers.

static void uncaught_irq(int number, cpu_state_t *cpu_state)
{
  UNUSED(cpu_state);
  __dbg_printk("Uncaught IRQ number %d\n", number);
}

void init_irq_management() 
{
  hw_isr_nested_level = 0;

  __dbg_printk("Initializing PIC...\n");
  __i8259_setup();
}


