/*
 * Copyright (C) 2000, 
 * http://kos.enix.org
 *
 * Bootstrap for KOS modules
 *
 * @(#) $Id: _bootstrap.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */


#include <kos/assert.h>
#include <debug/debug.h>
#include <arch/mm/mm.h>
#include <kos/asm.h>
#include <lib/string.h>

#include "_init.h"

#include <arch/internals.h> /* => pour le remove_identity_mapping. */
/* TBI : On a un probleme : les PTs de l'identity mapping ne sont pas
  liberes, parce qu'on ne peut pas faire put_physical_page tant que le
  GPFM n'est pas initialise. Or il est initialise apres le
  bootstrap. => bouger le code remove_identity_mapping dans arch/mm et
  l'appeler apres init_gpfm en utilisant put_physical_page. */


int kernel_start(kernel_parameter_t *);

static void remove_identity_mapping(kernel_parameter_t *kp)
{
  addr_t *pd;
  addr_t pt;
  unsigned int i;
  
  pd = (addr_t *) CURRENT_PD_VADDR;
  
  for (pt = CURRENT_PT_AREA_START, i = 0 ; 
       i < (ALIGN_SUP(kp->loader_allocatable_mem_size, 1024*PAGE_SIZE) 
	    / (1024*PAGE_SIZE)); 
       i++, pt+=PAGE_SIZE)
  {
    memset((char *)pt, 0, PAGE_SIZE);
    pd[i] = 0;
  }
  
#define flush_tlb() \
  do { \
	unsigned long tmpreg; \
       __asm__ __volatile__("movl %%cr3,%0\n\tmovl %0,%%cr3" :"=r" (tmpreg) : :"memory"); \
  } while (0)

  flush_tlb();
}

int kernel_init(kernel_parameter_t *kp)
{
  remove_identity_mapping(kp);

  if(init_modules(kp, INIT_BASE_LEVEL, INIT_NB_LEVELS) < 0)
    FAILED_VERBOSE("Error while initializing modules\n");

  {
    int i;
    for (i = 0; i < kp->nb_modules_parameters_entries; i++)
      __dbg_printk("Module %s has param %s\n",
		   kp->module_parameters[i].modname,
		   kp->module_parameters[i].params);
  }

  __dbg_printk("Calling kernel start (0x%x)...\n", (unsigned) kernel_start);
  kernel_start(kp);

  FAILED_VERBOSE("Never print this\n");
  return -1;
}

DECLARE_KERNEL_BOOTSTRAP(kernel_init);
