/*
 * Copyright (C) 2000, 
 * http://kos.enix.org
 *
 * Init Modules for KOS modules
 *
 * @(#) $Id: _init_modules.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */
#include <kos/assert.h>
#include <debug/debug.h>

#include "_init.h"

int init_modules(kernel_parameter_t *kernel_params,
		 unsigned int base_level, unsigned int nb_levels)
{
  unsigned int cur_level  = 0;
  unsigned int next_level = 1;
  struct kernel_module *km = NULL;

  if (kernel_params->modules_head == NULL)
    return -1;

  /* Init code modules */
  while (1) {
    int i;
    init_cleanup_routine_t *ctor;

    if (km == NULL)
      {
	km = kernel_params->modules_head;
	cur_level  = next_level;
	next_level = base_level + nb_levels + 10;

	if (cur_level >= (base_level + nb_levels))
	  break;
      }
    else
      {
	km = km->next;
	if (km == NULL)
	  continue;
      }

    ASSERT_FATAL(km != NULL);
    if (km->magic != MAGIC_RELOC_EXECUTABLE)
      continue;

    /* In Current module, try to figure out a init level equal to
       cur_level */
    for (i = km->u.reloc_executable.nb_init_routines, ctor = km->u.reloc_executable.init_routines ;
	 i ;
	 i--, ctor ++)
      {
	if (ctor->level == cur_level)
	  {
	    int r;
	    __dbg_printk("[Init modules] init_module() for module %s (@0%x), level %d...\n", 
			 km->data,
			 ctor->routine,
			 cur_level - base_level);

	    r = ((init_routine_t) ctor->routine)(kernel_params);
	    if(r < 0)
	      FAILED_VERBOSE("[Init Modules] Init_module for module %s (@0x%x),\n"
			     "  level %d returned %d (aka 0x%x)\n", 
			     km->data,
			     (unsigned) km,
			     cur_level - base_level,
			     (unsigned)r,
			     (unsigned)r);
	    
	    ctor->level = 0; /* Reset level */
	  }
	/* Or else remember the least next level */
	else if ((ctor->level > 0) && (ctor->level < next_level))
	  next_level = ctor->level;
      }

    /* Try next module */
  }

  return 0;
}
