
#include <debug/debug.h>
#include <loader/mod.h>
#include <arch/mm/mm.h>
#include <vmm/vmm.h>

#include "_init.h"

static int kernel_del_init_heap(kernel_parameter_t *kp)
{
  vaddr_t vaddr;
  
  printk("(Free %dkB init space...", kp->init_heap_size >> 10);
  for (vaddr = kp->init_heap_vaddr;
       vaddr < (kp->init_heap_vaddr + PAGE_ALIGN_SUP(kp->init_heap_size));
       vaddr += PAGE_SIZE)
    {
      unmap_virtual_page(NULL, vaddr);
    }
  printk("Ok)\n");
  
  return 0;
}


int kernel_post_init(kernel_parameter_t *kp)
{
  struct kernel_module *mod;

  if(init_modules(kp, POST_INIT_BASE_LEVEL, POST_INIT_NB_LEVELS) < 0)
    FAILED_VERBOSE("Error while post-initializing modules\n");

  kernel_del_init_heap(kp);

  /* Clear the address of the tables of init routines */
  for (mod = kp->modules_head ; mod ; mod = mod->next)
    {
      if (mod->magic == MAGIC_RELOC_EXECUTABLE)
	{
	  mod->u.reloc_executable.init_routines    = NULL;
	  mod->u.reloc_executable.nb_init_routines = 0;
	}
    }

  return 0;
}





