#ifndef __DEVFS_H__
#define __DEVFS_H__

#include <karm/karm.h>
#include <kos/types.h>

typedef unsigned dev_t;

/** Device definition */
struct devfs_device {
  char *group;
  char *dev;

  struct devfs_device *prev, *next;

  unsigned    nb_views;
  struct view *views; 
};

/** Registers a device in the device filesystem. 
 *
 *  @param group     The directory inside the device filesystem where the
 *                   device will be registered (for example : 'disk',
 *                   'part', 'sound', etc..).
 *
 *  @param dev_name  The name of the device in the device filesystem.
 *
 *  @param nb_views  The number of interfaces supported by the device.
 *
 *  @param view      The implementation of the supported interfaces.
 *
 *  @param device_id Where to return the ID of the device in the 
 *                   device filesystem.
 *
 *  @return 0 on success, an error code on failure
 */
result_t devfs_register(const char *group, const char *dev_name, 
			unsigned nb_views, struct view *view,
			dev_t *device_id);

/** Unregisters a device from the device filesystem
 *
 *  @param device_id The ID of the device to unregister. This ID is
 *                   the one that is returned (in the device_id
 *                   parameter) by the devfs_register() function.
 *
 *  @return 0 on success, an error code on failure
 */
result_t devfs_unregister(dev_t device_id);

#endif /* __DEVFS_H__ */
