#include <karm/fs_support.h>
#include <liblist/liblist.h>

#include <lib/string.h>
#include <kmem/kmem.h>

/** supported filesystem entry */
struct fs_supported {
  char *name;
  unsigned mounts;
  struct filesystem *fs;

  struct fs_supported *prev, *next;
};

static struct fs_supported *fs_sup_list;

/** return the filesystem structure for fs_name */
struct filesystem *lookup_supported_fs(const char *fs_name)
{
  struct fs_supported *tmp;
  unsigned i;

  list_foreach(fs_sup_list, tmp, i)
    {
      if (strcmp(fs_name, tmp->name) == 0)
	return tmp->fs;
    }
  
  return NULL;
}


/** register a supported filesystem */
int register_supported_fs(const char *fs_name, struct filesystem *fs)
{
  struct fs_supported *tmp;
  unsigned i;

  list_foreach(fs_sup_list, tmp, i)
    {
      if (strcmp(fs_name, tmp->name) == 0)
	return -1;
    }
  
  tmp = kmalloc(sizeof(struct fs_supported));
  ASSERT(tmp != NULL);
  
  tmp->fs = kmalloc(sizeof(struct filesystem));
  ASSERT(tmp != NULL);

  tmp->name        = strdup(fs_name);
  tmp->fs->attach  = fs->attach;
  tmp->fs->detach  = fs->detach;
  tmp->mounts      = 0;

  list_add_head(fs_sup_list, tmp);
  return 0;
}

/** unregister a filesystem and assert it isn't currently mounted */
int unregister_supported_fs(const char *fs_name)
{
  struct fs_supported *tmp;
  unsigned i;

  ASSERT(fs_name != NULL);

  list_foreach(fs_sup_list, tmp, i)
    {
      if (strcmp(fs_name, tmp->name) == 0)
	break;
    }

  if ((tmp == NULL) || (tmp->mounts != 0))
    return -1;
  
  list_delete(fs_sup_list, tmp);
  kfree(tmp->fs);
  kfree(tmp);

  return 0;
}
