#ifndef __FS_SUPPORT_H__
#define __FS_SUPPORT_H__

#include <karm/ures.h>
#include <karm/kres.h>
#include <kos/types.h>

/** This structure describe a filesystem driver, and it allows to
    mount and umount filesystem. */
struct filesystem {
  /** Attach is a function pointer to the function of the filesystem
      driver that allows to mount the device. 
      
      @param device  The device to mount. It assumes that the
                     device is opened with the BLOCK view.

      @param root    This is the location at which the filesystem driver
                     has to put a pointer to the kernel ressource of
                     the root point of the mounted filesystem.

      @param fs_info This is the location where the filesystem driver
                     can return a pointer to private data, that will
                     be used to unmount the filesystem. It may
                     contains information for the filesystem, such as
                     number of files, location of the root directory,
                     etc..

      @param conf    Some configuration informations for the filesystem

      @return 0 on success, an error code on failure
  */
  result_t (*attach)(struct ures *device, struct kres **root, 
		     void **fs_info, void *conf);

  /** Detach is a function pointer to the function of the filesystem
      driver that allows to unmount a filesystem.

      @param fs_info This is the pointer that was returned by the
                     attach() function of the filesystem driver, and
                     that allows the filesystem driver to identify the
                     filesystem.

      @return 0 on success, an error code on failure
  */
  result_t (*detach)(void *fs_info);
};

struct filesystem *lookup_supported_fs(const char *fs_name);
int register_supported_fs(const char *fs_name, struct filesystem *fs);
int unregister_supported_fs(const char *fs_name);


#endif  /* __FS_SUPPORT_H__ */
