#ifndef __KRES_H__
#define __KRES_H__

#include <karm/view.h>
#include <karm/ures.h>

/** A kernel ressource is the abstract representation of a object in
    the system (a socket, a file, an hard driver, a sound card,
    etc..). For one object is the system, there is only one kernel
    ressource : it is a global data structure.

    A global ressource is mainly interfaces implementations (the
    interface supported by this specific ressource), and private data
    (that make sense for the driver).
*/
typedef struct kres {

  /** The array of view. A view is simply an array of function
      pointers which is fact the implementation of an interface */
  struct view *views;
  
  /** The number of views in the previous array */
  unsigned nb_views;

  /** This pointer is used by the driver that manages this ressource
      to associate each ressource to some private data. For example,
      for a 'file' it might be length of the file, the access rights,
      etc... It contains information that are global to all open() of
      this kernel ressource. */
  void *ext;

  /** The list of ures that correspond to this ressource */
  struct ures *ures_list;
  
  /** Chained-list pointers, maybe useless */
  struct kres *prev, *next;
} kres_t;

/** 
 * Add an user ressource to the list of user ressource of a kernel
 * ressource.
 *
 * @param kres The kernel ressource to which we want to add an user
 *             ressource.
 *
 * @param ures The user ressource we want to add.
 *
 * @return 0 on success, an error code on failure
 */
result_t kres_add_ures(struct kres *kres, struct ures *ures);

/**
 * Delete an user ressource from the list of user ressource of a
 * kernel ressource.
 *
 * @param ures The user ressource to remove.
 *
 * @return 0 on sucess, an error code on failure
 */
result_t kres_remove_ures(struct ures *ures);

#endif /* __KRES_H__ */
