#include <karm/kres.h>

#include <kmem/kmem.h>
#include <liblist/liblist.h>

#include <kos/assert.h>
#include <kos/macros.h>

/** Open an user ressource, corresponding to the given kernel
 *  ressource with the given interface.
 *
 *  @param kres      The kernel ressource for which we want to generate a
 *                   user ressource.
 *
 *  @param interface The ID of the interface with which we want to
 *                   open the kernel ressource.
 *
 *  @param ures      Where to return the pointer of the new user ressource
 *
 *  @result 0 on success, an error code on failure
*/
result_t ures_open(struct kres *kres, unsigned interface, struct ures **ures)
{
  struct ures *new_ures;
  struct view *view = NULL;
  unsigned int i;

  __dbg_printk("ures_open: kres=%p, interface=%d, (out) ures=%p\n",
	       kres, interface, ures);

  __dbg_printk("Going through all views\n");

  for (i = 0; i < kres->nb_views; i++)
    {
      __dbg_printk(" View %d\n", i);
      if(kres->views[i].iid == interface)
	{
	  view = & kres->views[i];
	  break;
	}
    }

  if(view == NULL)
    {
      return -EBADIFACE;
    }

  new_ures = kmalloc(sizeof(struct ures));

  if(new_ures == NULL)
    {
      return -ENOMEM;
    }

  new_ures->kres = kres;
  new_ures->view = view;

  /* TODO : call a function of a special mandatory interface to
     initialize the ures ext field. */
  new_ures->ext  = NULL;

  new_ures->prev = new_ures->next = NULL;

  *ures = new_ures;
  
  return 0;
}

/** Close a user ressource
 *
 *  @param ures The user ressource to close
 *
 *  @return 0 on success, an error code on failure
 */
result_t ures_close(struct ures *ures)
{
  UNUSED(ures);
  return -ENOSYS;
}

