/*
 * Copyright (C) 2001, Thomas Petazzoni (thomas.petazzoni@meridon.com)
 * http://kos.enix.org
 *
 * Kitc : ksignal part
 *
 * @(#) $Id: _ksignal.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <kitc/_ksignal.h>
#include <task/task.h>
#include <kos/spinlock.h>
#include <kos/assert.h>
#include <kos/types.h>
#include <debug/debug.h>
#include <scheduler/scheduler.h>

int ksignal_init(ksig_t *ksig)
{
  ASSERT_RETURN_VAL(ksig != NULL, -1);
  ksig->waiting_thread = NULL;

  return 0;
}

int ksignal_wait(ksig_t *ksig)
{
  k_ui32_t flags;

  ASSERT_RETURN_VAL(ksig->waiting_thread == NULL, -1);

  write_spin_lock(kernel_thread_list_lock, flags);
  ksig->waiting_thread = get_current_thread();
  ksig->waiting_thread->state = WAITING;
  write_spin_unlock_remote(kernel_thread_list_lock, flags);

  reschedule_run_next();

  write_spin_unlock_local(kernel_thread_list_lock, flags);
  
  return 0;
}

int ksignal_wakeup(ksig_t *ksig)
{
  k_ui32_t flags;

  ASSERT_RETURN_VAL(ksig != NULL, -1);
  ASSERT_RETURN_VAL(ksig->waiting_thread != NULL, -1);

  write_spin_lock(kernel_thread_list_lock, flags);
  ksig->waiting_thread->state = INTERRUPTED;
  ksig->waiting_thread = NULL;
  write_spin_unlock(kernel_thread_list_lock, flags);

  return 0;
}

int ksignal_destroy(ksig_t *ksig)
{
  ASSERT_RETURN_VAL(ksig != NULL, -1);
  ASSERT_RETURN_VAL(ksig->waiting_thread == NULL, -1);
  
  return 0;
}

