#include <kitc/_kwaitqueue.h>
#include <kos/types.h>
#include <kos/spinlock.h>
#include <kos/assert.h>
#include <liblist/liblist.h>

spinlock_t global_waitqueue_lock;

int kwaitqueue_init(struct kwaitqueue *kwq)
{
  k_ui32_t flags;

  CONCEPTION_ASSERT(kwq != NULL);

  write_spin_lock(global_waitqueue_lock, flags);

  kwq->thread_list = NULL;
  spinlock_init(kwq->lock);
  
  write_spin_unlock(global_waitqueue_lock, flags);

  return 0;
}

int kwaitqueue_add(struct kwaitqueue *kwq)
{
  struct thread *cthread;
  k_ui32_t flags;

  CONCEPTION_ASSERT(kwq != NULL);

  write_spin_lock(kwq->lock, flags);

  cthread = get_current_thread();
  CONCEPTION_ASSERT(cthread != NULL);
  
  list_add_tail_named(kwq->thread_list,
		      cthread,
		      u.waiting_waitqueue_thread.prev,
		      u.waiting_waitqueue_thread.next);
  cthread->state = WAITING_WAIT_QUEUE;
  write_spin_unlock(kwq->lock, flags);
  
  reschedule_run_next();

  return 0;
}

int kwaitqueue_wake_up(struct kwaitqueue *kwq)
{
  k_ui32_t flags;
  struct thread *next_thread;

  if(kwq == NULL)
    return 0;

  write_spin_lock(kwq->lock, flags);
  next_thread = kwq->thread_list;

  if(next_thread == NULL)
    {
      write_spin_unlock(kwq->lock, flags);
      return 0;
    }

  list_delete_named(kwq->thread_list,
		    next_thread,
		    u.waiting_waitqueue_thread.prev,
		    u.waiting_waitqueue_thread.next);

  next_thread->u.waiting_waitqueue_thread.next = NULL;
  next_thread->u.waiting_waitqueue_thread.prev = NULL;
  next_thread->u.waiting_waitqueue_thread.kwq  = NULL;
  next_thread->state = INTERRUPTED;
  write_spin_unlock(kwq->lock, flags);
  
  return 0;
}

__init_text int _kwaitqueue_init(void)
{
  spinlock_init(global_waitqueue_lock);
  return 0;
}
