#include <kmem/_kmem.h>
#include <lib/stdlib.h>
#include <kos/types.h>
#include <kos/assert.h>
#include <kos/system.h>

#define _CR "\r"

static int rnd_int1(int max) {
  int ret = random() % max;
  CONCEPTION_ASSERT((ret >= 0) && (ret < max));
  return ret;
}

static int rnd_int2(int min, int max) {
  int ret = min + (random() % (max - min + 1));
  CONCEPTION_ASSERT((ret >= min) && (ret <= max));
  return ret;
}

static void rnd_array(int *array, int nb_items)
{
  int i;

  for (i = 0 ; i < nb_items ; i++)
    array[i] = -1;

  for (i = 0 ; i < nb_items ; i++) {
    int index;
    /* On va essayer de caser i dans une case, tiree aleatoirement, de
       array */
    index = rnd_int1(nb_items);
    if (array[index] >= 0) {
      /* Mince, c'est deja pris */
      /* On choisit une direction, et on prend le premier emplacement
         libre */
      if (random() % 2) {
	while (array[index] >= 0) {
	  index++;
	  if (index >= nb_items)
	    index = 0;
	}
      } else {
	while (array[index] >= 0) {
	  index--;
	  if (index < 0)
	    index = nb_items - 1;
	}
      }
    }
    CONCEPTION_ASSERT((index >= 0) && (index < nb_items));
    CONCEPTION_ASSERT((i >= 0) && (i < nb_items));
    array[index] = i;
  }

}

// --------------------- The tests


static int test_campaign(int free_len, alloc_table_t *alloc_table)
{
  int i, free_array[NB_ALLOC_MAX];

  CONCEPTION_ASSERT((free_len >=0) && (free_len <= NB_ALLOC_MAX));

#ifdef KMEM_HARD_CORE_TEST_VERBOSE
  __dbg_printk("# alloc/free campaign for %d elements..\n", free_len);

  __dbg_printk(_CR "Gen free array...             ");
#endif

  rnd_array(free_array, NB_ALLOC_MAX);

#ifdef KMEM_HARD_CORE_TEST_VERBOSE
  __dbg_printk(_CR "kmalloc test...               ");
#endif

  for (i = 0 ; i < NB_ALLOC_MAX ; i++) {
    if (alloc_table[i].base_addr)
      continue; /* Already allocated */
    alloc_table[i].size      = rnd_int2(4,2048);
    alloc_table[i].base_addr = (addr_t)kmalloc(alloc_table[i].size);
    if (! alloc_table[i].base_addr)
      __dbg_printk("Error on alloc %d (iterartion %d)\n", alloc_table[i].size, i);
    else
      *((int*)alloc_table[i].base_addr) = i;
  }

#ifdef KMEM_HARD_CORE_TEST_VERBOSE
  __dbg_printk(_CR "Check alloc...               ");
#endif

  for (i = 0 ; i < NB_ALLOC_MAX ; i++) {
    if (! alloc_table[i].base_addr) {
      // __dbg_printk("(alloc[%d] has not been allocated)\n", i);
      continue;
    }
    if (*((int*)alloc_table[i].base_addr) != i) {
      __dbg_printk("alloc[%d]=0x%x - Wrong magic (is: %d, should be: %d)\n",
	     i, alloc_table[i].base_addr,
	     *((int*)alloc_table[i].base_addr), i);
      return -1;
    }
  }

#ifdef KMEM_HARD_CORE_TEST_VERBOSE
  __dbg_printk(_CR "Free...                 ");
#endif

  for (i = 0 ; i < free_len ; i++) {
    int index = free_array[i];
    if (! alloc_table[index].base_addr)
      continue;
    if (*((int*)alloc_table[index].base_addr) != index) {
      __dbg_printk("alloc[%d]=0x%x - Wrong magic\n", index,
	     alloc_table[index].base_addr);
      return -1;
    }
    kfree_size_known((void*)alloc_table[index].base_addr,
		     alloc_table[index].size);
    alloc_table[index].size      = -1;
    alloc_table[index].base_addr = 0;
  }

#ifdef KMEM_HARD_CORE_TEST_VERBOSE
  __dbg_printk(_CR "Check alloc(2)...           ");
#endif

  for (i = 0 ; i < NB_ALLOC_MAX ; i++) {
    if (! alloc_table[i].base_addr) {
      //      __dbg_printk("(alloc[%d] has not been allocated)\n", i);
      continue;
    }
    if (*((int*)alloc_table[i].base_addr) != i) {
#ifdef KMEM_HARD_CORE_TEST_VERBOSE
      __dbg_printk("alloc[%d]=0x%x - Wrong magic (is: %d, should be: %d)\n",
	     i, alloc_table[i].base_addr,
	     *((int*)alloc_table[i].base_addr), i);
#endif

      return -1;
    }
  }

  return 0;
}


int kmem_test(alloc_table_t *alloc_table) 
{
  int i;

  __dbg_printk("Init alloc_table...\n");
  for (i = 0 ; i < NB_ALLOC_MAX ; i++) {
    alloc_table[i].size      = -1;
    alloc_table[i].base_addr = 0;
  }

  for (i = 0 ; i<2000; i++) {
    __dbg_printk(_CR "----- Test campaign %d/100000 ----\n", i+1);

    if(i % 10)
      usleep(rnd_int1(5)*(1000000/HZ));

    if (test_campaign(rnd_int1(NB_ALLOC_MAX+1), alloc_table)) {
      __dbg_printk("\nError\n");
      return -1;
    }
  }

#ifdef KMEM_HARD_CORE_TEST_VERBOSE
  __dbg_printk(_CR "---------------------------\n# Final Free...\n");
#endif

  for (i = 0 ; i < NB_ALLOC_MAX ; i++) {
    if (! alloc_table[i].base_addr)
      continue;
    if (*((int*)alloc_table[i].base_addr) != i) {
#ifdef KMEM_HARD_CORE_TEST_VERBOSE
      __dbg_printk("alloc[%d]=0x%x - Wrong magic\n", i,
	     alloc_table[i].base_addr);
#endif
      continue;
    }
    kfree((void*)alloc_table[i].base_addr);
    alloc_table[i].size      = -1;
    alloc_table[i].base_addr = 0;
  }


  __dbg_printk("The end.\n");
  return 0;
}




