#ifndef ___KSLAB_H__
#define ___KSLAB_H__

#include <kos/spinlock.h>
#include <kos/types.h>
#include <kmem/kmem.h>
#include <arch/mm/mm.h>
#include <liblist/liblist.h>

#define BYTES_PER_WORD 4

/* Internal flags for kslab_cache_t */
#define SLAB_POS 0
#define ON_SLAB  (1<<SLAB_POS)
#define OFF_SLAB (0<<SLAB_POS)
#define SLAB_IS_SWAPPABLE_POS 1
#define SLAB_IS_SWAPPABLE  (1<<SLAB_IS_SWAPPABLE_POS)
#define SLAB_NOT_SWAPPABLE (0<<SLAB_IS_SWAPPABLE_POS)

#define L1_CACHE_SIZE 16

#define MAX_PAGES_PER_SLAB 8
#define MIN_ELTS_PER_SLAB 4

#define KSLAB_CACHE_NAME_LENGTH 16

typedef struct kslab_cache
{
  char name[KSLAB_CACHE_NAME_LENGTH];
  size_t size;
  size_t original_size;
  
  struct kslab_slab *full;
  struct kslab_slab *semi_full;
  struct kslab_slab *free;

  k_ui32_t nb_elts_per_slab;
  k_ui32_t nb_pages_per_slab;
  spinlock_t lock;
  k_ui32_t colour;
  k_ui32_t flags;

  int nb_available;
  int grow_threshold;
  bool_t growth_in_progress;/* Replace this with a semaphore */
  
  struct kslab_cache *next;
  struct kslab_cache *prev;
  
}kslab_cache_t;


  
typedef struct kslab_slab
{
  k_ui32_t nb_free;
  struct kslab_free_blk *free;
  
  struct kslab_cache *cache;
  struct kslab_slab *next;
  struct kslab_slab *prev;

}kslab_slab_t;

typedef struct kslab_free_blk
{
  struct kslab_free_blk *next;
} kslab_free_blk_t;


/* Global variables */
extern kslab_cache_t *kslab_cache_list;
extern kslab_cache_t cache_of_kslab_cache;
extern kslab_cache_t *cache_of_kslab_slab;

int __kslab_init(void);
int __kslab_cache_grow(kslab_cache_t *cache, addr_t new_space);
#define __kslab_slab_move_head(from,to) \
  ({ typeof(from) __ret = list_pop_head(from); \
     list_add_head((to),__ret); })
#define __kslab_slab_move_element(elt,from,to) \
  ({ list_delete((from),(elt)); \
     list_add_head((to),(elt)); })
int __kslab_cache_free_by_slab(kslab_slab_t *slab, void *block);
kslab_slab_t *__kslab_get_slab(void *block);

#endif

