/*
 * Copyright (C) 2000, Thomas Petazzoni
 * http://kos.enix.org
 *
 * Free space from a cache. 
 *
 * @(#) $Id: _kslab_cache_free.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <kmem/_kslab.h>
#include <pmm/pmm.h>

int __kslab_cache_free_by_slab(kslab_slab_t *slab, void *block)
{
  kslab_free_blk_t *freed_block = (kslab_free_blk_t *) block;
  k_ui32_t flags;

  write_spin_lock(slab->cache->lock, flags);
  freed_block->next = slab->free;
  slab->free = freed_block;

  /* Slab was completely full, move it to semi_full list */
  if(slab->nb_free == 0)
    __kslab_slab_move_element(slab, slab->cache->full,
			      slab->cache->semi_full);

  slab->nb_free++;
  slab->cache->nb_available++;
  
  /* If number of elements is egal to the total number of elements per
     slab, the slab is free. It is then moved from the semi_full list
     to the free list. We are sure that the page is in the semi_full
     list because slab have at least MIN_ELTS_PER_SLAB elements, which
     != 1 */
  if(slab->nb_free == slab->cache->nb_elts_per_slab)
    {
      __kslab_slab_move_element (slab, slab->cache->semi_full,
				 slab->cache->free);
    }

  write_spin_unlock(slab->cache->lock, flags);
  
  return 0;
}

/* Read gpfm in order to find the slab corresponding to the given
   block */
kslab_slab_t *__kslab_get_slab(void *block)
{
  gpfme_t *gpfme;
  k_ui32_t flags;
  kslab_slab_t *slab;

  gpfme = get_gpfme_at_virt_addr((addr_t)block, & flags);
  if(!gpfme)
    {
      gpfme_unlock(gpfme, & flags);
      return NULL;
    }

  slab = gpfme->slab;
  gpfme_unlock(gpfme, & flags);

  return slab;
}


int kslab_cache_free(kslab_cache_t *cache,
		     void *block)
{
  
  kslab_slab_t *slab;

  /* Get the slab containing the block */
  slab = __kslab_get_slab(block);

  if(!slab)
    return -1;
  
  /* Make sure the cache corresponding to the found slab match the
     given cache */
  CONCEPTION_ASSERT(cache == slab->cache);
  
  /* Free the block */
  return __kslab_cache_free_by_slab(slab, block);
}



