/*
 * Copyright (C) 2000, Thomas Petazzoni
 * http://kos.enix.org
 *
 * Add a page to the given cache
 *
 * @(#) $Id: _kslab_cache_grow.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <kmem/_kslab.h>
#include <pmm/pmm.h>

int __kslab_cache_grow(kslab_cache_t *cache, addr_t new_space)
{
  addr_t ptr;
  kslab_slab_t *new_slab;
  addr_t virt_addr;
  kslab_free_blk_t *free_blk;

  if(!new_space)
    return -1;
  
  /* Create the linked list of free blocks in the new slab */
  for(ptr = new_space; 
      ptr < ((cache->nb_elts_per_slab *  cache->size) + new_space); 
      ptr += (cache->size))
    {
      free_blk = (kslab_free_blk_t *) ptr;
      free_blk->next = (kslab_free_blk_t *) (ptr + cache->size);
    }

  /* Make sure that the next pointers point to NULL for the last
     element of the slab */
  free_blk = (kslab_free_blk_t *) (new_space + ((cache->nb_elts_per_slab-1) 
					       * cache->size));
  free_blk->next = NULL;
  
  /* If the kslab_slab_t have to be on the slab, its address is at the
     end of this slab */
  if(cache->flags & ON_SLAB)
      new_slab = (kslab_slab_t *) 
	(new_space + cache->nb_pages_per_slab * PAGE_SIZE
	- sizeof(kslab_slab_t));

  /* Else the kslab_slab_t have to be out of the slab, allocate space
     from the cache of kslab_slab */
  else
    {
      new_slab = kslab_cache_alloc(cache_of_kslab_slab);
      
      if(! new_slab)
	return -1;
    }
  
  /* Update 'slab' pointers of all gpfme_t concerned by this cache grow */
  for(virt_addr = (addr_t) new_space;
      virt_addr < (addr_t) new_space + cache->nb_pages_per_slab*PAGE_SIZE;
      virt_addr += PAGE_SIZE)
    {
      k_ui32_t gpfm_lock_flags;
      gpfme_t *gpfme = get_gpfme_at_virt_addr(virt_addr, & gpfm_lock_flags);
      CONCEPTION_ASSERT(gpfme != NULL);

      gpfme->slab = new_slab;
      gpfme_unlock(gpfme, & gpfm_lock_flags);
    }

  /* No spinlock needed because the slab is known to be available only
     through the last operation, which is assumed to be atomic */
  new_slab->next = NULL;
  new_slab->prev = NULL;
  new_slab->free = (kslab_free_blk_t *) new_space;
  new_slab->nb_free = cache->nb_elts_per_slab;
  new_slab->cache = cache;
  list_add_head(cache->free, new_slab);
  cache->nb_available += cache->nb_elts_per_slab;
  
  return 0;
}

