/*
 * Copyright (C) 2000, Thomas Petazzoni
 * http://kos.enix.org
 *
 * Initialization for the slab allocator
 * 
 * @(#) $Id: _kslab_init.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <kmem/_kslab.h>
#include <debug/debug.h>
#include <kos/macros.h>
#include <lib/string.h>
#include <liblist/liblist.h>

kslab_cache_t *kslab_cache_list;
kslab_cache_t cache_of_kslab_cache;
kslab_cache_t *cache_of_kslab_slab;

int __kslab_init(void)
{
  cache_of_kslab_cache.size = ALIGN_SUP(sizeof(kslab_cache_t), BYTES_PER_WORD);

  strncpy(cache_of_kslab_cache.name ,"Kslab caches",KSLAB_CACHE_NAME_LENGTH);
  cache_of_kslab_cache.full           = NULL;
  cache_of_kslab_cache.semi_full      = NULL;
  cache_of_kslab_cache.free           = NULL;
  cache_of_kslab_cache.next           = NULL;
  cache_of_kslab_cache.prev           = NULL;
  cache_of_kslab_cache.flags          = ON_SLAB;
  cache_of_kslab_cache.grow_threshold = 0;
  cache_of_kslab_cache.nb_available   = 0;
  cache_of_kslab_cache.growth_in_progress = FALSE;
  spinlock_init(cache_of_kslab_cache.lock);
  cache_of_kslab_cache.nb_pages_per_slab = 1;
  cache_of_kslab_cache.nb_elts_per_slab = 
    (PAGE_SIZE - sizeof(kslab_slab_t)) / sizeof(kslab_cache_t);

  kslab_cache_list = NULL;
  list_add_tail(kslab_cache_list, &cache_of_kslab_cache);

  cache_of_kslab_slab = kslab_cache_create("kslab_slab cache",
					   sizeof(kslab_slab_t),
					   0, 0, 0);
  
  if(! cache_of_kslab_slab)
    return -1;
  
  if((cache_of_kslab_slab->flags & (1<<SLAB_POS)) == 0)
    FAILED_VERBOSE("Could not correctly create cache of kslab\n");

  return 0;
}



