#ifndef __KVMEM_H__
#define __KVMEM_H__

#include <loader/mod.h>
#include <kos/types.h>
#include <kos/spinlock.h>
#include <kmem/kmem.h>

typedef k_ui32_t bitmap_t;

typedef struct chained_page_range chained_page_range_t;

struct chained_page_range {
  addr_t                    start;
  k_ui32_t                  nb_pages;
  struct chained_page_range *next;
  struct chained_page_range *prev;
};

#define MINIMAL_SURVIVAL_RANGE_NB 5

#define RANGES_PER_PAGE 255
typedef struct page_of_range page_of_range_t;
struct page_of_range {
  
  struct chained_page_range ranges[RANGES_PER_PAGE];

  k_ui32_t nb_ranges;

  struct page_of_range *prev;
  struct page_of_range *next;
};

extern chained_page_range_t *free_page_range_list;
extern chained_page_range_t *used_page_range_list;
extern page_of_range_t *page_of_range_list;
extern k_ui32_t total_range_nb;


#define RANGE_END(range) ((range)->start + ((range)->nb_pages * PAGE_SIZE))

#define IS_RANGE_FREE(range)  ((range)->start == 0x0)
#define SET_RANGE_FREE(range) ((range)->start  = 0x0)

void __kvmem_unmap_range(chained_page_range_t *range);
int __kvmem_map_range(addr_t virt_base_addr, int nb_pages,
		      bool_t is_swappable);
chained_page_range_t *__kvmem_allocate_new_unused_range(void);
int __kvmem_add_range_to_used_list(chained_page_range_t *range);
void __kvmem_reclaim_free_range(chained_page_range_t *range);
int __kvmem_remove_range_from_free_list(chained_page_range_t *range);
void __kvmem_show_all_ranges(void);
int __kvfree_unsafe(addr_t range_start, bool_t needs_unmapping);
int __kvmem_reclaim_page_of_ranges(page_of_range_t *range_page);
int __kvmem_add_new_page_of_ranges(void);
chained_page_range_t *__kvmem_find_suitable_range_in_free_list(unsigned int nb_pages);
int __kvmem_init(kernel_parameter_t *kp);

struct chained_page_range * __kvmem_get_used_page_range_list(void);


#endif
