#include <kmem/_kslab.h>
#include <kmem/_kmem.h>
#include <loader/mod.h>
#include <kmem/_kvmem.h>
#include <task/task.h>

#define NB_TEST_THREAD 20
//#define KMEM_TEST

alloc_table_t table[NB_TEST_THREAD][NB_ALLOC_MAX];

__init_text static int init_module_level1(kernel_parameter_t *kp)
{
  printk("(Init kmem: ");

  printk("[kvmem");
  if(__kvmem_init(kp) < 0)
    return -1;
  printk("] ");
  
  printk("[kslab");
  if(__kslab_init() < 0)
    return -1;
  printk("] ");

  printk("[kmem");
  if(kmem_init() < 0)
    return -1;
  printk("] ");

  printk("Ok)\n");

  return 0;
}

#ifdef KMEM_TEST
static void kmem_test_thread(void *args)
{
  kmem_test((alloc_table_t *) args);
}
#endif

__init_text static int post_init_module_level3(kernel_parameter_t *kp)
{
  UNUSED(kp);

#ifdef KMEM_TEST
  {
    int i;
    for (i = 0; i < NB_TEST_THREAD; i++)
      create_kernel_thread(NULL, kmem_test_thread, (void *) table[i]);
  }
#endif


  return 0;
}

DECLARE_INIT_SYMBOL(init_module_level1, INIT_LEVEL1);
DECLARE_INIT_SYMBOL(post_init_module_level3, POST_INIT_LEVEL3);

EXPORT_FUNCTION(kmalloc);
EXPORT_FUNCTION(kfree);
EXPORT_FUNCTION(kvalloc);
EXPORT_FUNCTION(kvfree);
EXPORT_FUNCTION(kslab_cache_create);
EXPORT_FUNCTION(kslab_cache_destroy);
EXPORT_FUNCTION(kslab_cache_alloc);
EXPORT_FUNCTION(kslab_cache_free);

EXPORT_FUNCTION(__kvmem_get_used_page_range_list);

SPINLOCK(kernel_kvalloc_lock);
EXPORT_SPINLOCK(kernel_kvalloc_lock);
SPINLOCK(kernel_kslab_lock);
EXPORT_SPINLOCK(kernel_kslab_lock);
