#ifndef __KMEM_H__
#define __KMEM_H__

#include <kos/types.h>

struct kslab_cache;

#define KSLAB_FLAG_BASE        20
#define KSLAB_FLAG_MASK        (~((1<<KSLAB_FLAG_BASE)-1))
#define KSLAB_FLAG_CACHE_ALIGN (1<<(KSLAB_FLAG_BASE+0))
#define KSLAB_FLAG_IS_SWAPPABLE     (1<<(KSLAB_FLAG_BASE+1))

/* Flags might be any combination of
    - KSLAB_FLAG_IS_SWAPPABLE
    - KSLAB_FLAG_CACHE_ALIGN
*/
struct kslab_cache *kslab_cache_create(char *name,
				       size_t size,
				       int align,
				       int flags,
				       k_ui32_t grow_threshold);
int kslab_cache_destroy(struct kslab_cache *cache);

void *kslab_cache_alloc(struct kslab_cache *cache);
int kslab_cache_free(struct kslab_cache *cache, void *data);

void *kmalloc(size_t size);
int kfree(void *block);

#define kfree_size_known(a,b) kfree(a)

addr_t kvalloc(size_t nb_pages, bool_t is_swappable, bool_t needs_mapping);
int kvfree(addr_t addr, bool_t needs_unmapping);


/****************************************
 * For task module
 */
#include <task/task.h>

int init_kteam_as_kmem(struct team *kteam);




#endif /* __KMEM_H__ */
