/*
 * Copyright (C) 2000, David Decotigny
 * http://kos.enix.org                 
 * 
 * Macros for assertions and some debugging facilities
 * 
 * @(#) $Id: assert.h,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */


#ifndef __KOS_ASSERT_H__
#define __KOS_ASSERT_H__

#include <kos/macros.h>
#include <lib/stdio.h>

#define ASSERT(expr) ({ \
  int __res=(int)(expr); \
  if (! __res) \
    WARNING("Assertion " # expr " failed\n") ; \
})

#define ASSERT_FATAL(expr) ({ \
  int __res=(int)(expr); \
  if (! __res) { \
    FAILED_VERBOSE("Assertion " # expr " failed\n"); \
  } \
})


#ifdef __SILENT_ASSERT_RETURN__
# define ASSERT_RETURN     RETURN_IF_FAIL
# define ASSERT_RETURN_VAL RETURN_VAL_IF_FAIL
#else
# define ASSERT_RETURN(expr) ({ \
  int __res=(int)(expr); \
  if (! __res) { \
    WARNING("Assertion " # expr " failed - returning from function\n"); \
    return; \
  } \
})

# define ASSERT_RETURN_VAL(expr,val) ({ \
  int __res=(int)(expr); \
  if (! __res) { \
    WARNING("Assertion " # expr " failed - returning emergency value\n"); \
    return val; \
  } \
})
#endif


/*
 * This macro should be used when developping, for obvious fonctional
 * critical tests : if they don't fail for one time, they aren't to
 * fail at all. As soon as the test doesn't fail, it should be
 * completely safe to define __NO_CONCEPTION_DBG__. When
 * __NO_CONCEPTION_DBG__ is set, expr won't event be evaluated.
 */
#ifdef __NO_CONCEPTION_DBG__
#define CONCEPTION_ASSERT(expr) ({ ; })
#else
#define CONCEPTION_ASSERT(expr) ({ \
  int __res=(int)(expr); \
  if (! __res) { \
    FAILED_VERBOSE("[CONCEPTION] Assertion " # expr " failed\n"); \
  } \
})
#endif

#endif /* __KOS_LD_ASSERT_H__ */
