/*
 * Copyright (C) 2000, David Decotigny
 * http://kos.enix.org                 
 * 
 * Macros header
 * 
 * @(#) $Id: macros.h,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */


#ifndef __KOS_MACROS_H__
#define __KOS_MACROS_H__

#include <kos/types.h>

#ifndef NULL
#define NULL ((void*)0)
#endif

#ifndef RETURN_OK
#define RETURN_OK    0
#endif

#ifndef RETURN_FAIL
#define RETURN_FAIL -1
#endif

/* _a and _b are evaluated NO MORE THAN once ! */
#define MIN(_a,_b) \
  ({ typeof((_a)) __a = (_a); \
     typeof((_b)) __b = (_b); \
     ((__a<__b)?__a:__b); })

/* _a and _b are evaluated NO MORE THAN once ! */
#define MAX(_a,_b) \
  ({ typeof((_a)) __a = (_a); \
     typeof((_b)) __b = (_b); \
     ((__a>__b)?__a:__b); })

// Used in the modules header files in order to tell that the
// prototyped function is being exported
#define EXPORT

// When function arguments are unused
#define UNUSED(arg) (arg) = (arg)

/* Align on a boundary (MUST be a power of 2), so that return value <= val */
#define ALIGN_INF(val,boundary) \
  ((val) & (~((boundary)-1)))

/* Align on a boundary (MUST be a power of 2), so that return value >= val */
#define ALIGN_SUP(val,boundary) \
  ((((val)-1) & (~((boundary)-1))) + (boundary))

#include <debug/debug.h>

#define WARNING(fmt,args...) ({ \
  DEBUG_PRINT_VERBOSE("Warning: "); \
  DEBUG_PRINT(fmt, ## args) ; \
})

#define RETURN_IF_FAIL(expr) ({ \
  int __res=(int)(expr); \
  if (! __res) \
    return; \
})

#define RETURN_VAL_IF_FAIL(expr,val) ({ \
  int __res=(int)(expr); \
  if (! __res) \
    return val; \
})

#define RETURN_IF_FAIL_VERBOSE(expr) ({ \
  int __res=(int)(expr); \
  if (! __res) { \
    DEBUG_PRINT_VERBOSE(#expr " failed, returning from function"); \
    return; \
  } \
})

#define RETURN_VAL_IF_FAIL_VERBOSE(expr,val) ({ \
  int __res=(int)(expr); \
  if (! __res) { \
    DEBUG_PRINT_VERBOSE(#expr " failed, returning from function"); \
    return val; \
  } \
})

#define TBI(str) DEBUG_PRINT_VERBOSE("TBI: " str "\n")

#endif /* __KOS_MACROS_H__ */
