#ifndef __SPINLOCK_H__
#define __SPINLOCK_H__

#include <loader/mod.h>
#include <arch/lib/lib.h>
#include <kos/assert.h>
#include <config.h>

#define save_flags(x) asm volatile("pushfl ; popl %0":"=g"(x)::"memory")
#define restore_flags(x) asm volatile("push %0; popfl"::"g"(x):"memory")

#ifdef __DEADLOCK_CHECK__

//#define LOCKING_REPORT bochs_printk
#define LOCKING_REPORT(a...)

typedef struct {
  enum { LCK_UNDEF=0, LCK_READY=42 } state;
  int nb_readers, nb_writers;
} spinlock_t;
#define EXPORT_SPINLOCK(spinlock) EXPORT_VARIABLE(spinlock)

#define SPINLOCK(spinlock) \
  spinlock_t spinlock = { .nb_readers=0, .nb_writers=0, .state=LCK_READY }
#define spinlock_init(spinlock) \
        do { LOCKING_REPORT("init_sl(" # spinlock ")\n"); \
             (spinlock).state = LCK_READY; \
             (spinlock).nb_readers = 0; \
             (spinlock).nb_writers = 0; } while(0)

#define read_spin_lock(spinlock,flags) \
        do { save_flags(flags); cli(); \
             LOCKING_REPORT("read_lock_sl(" # spinlock " : "); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             ASSERT_FATAL((spinlock).nb_writers == 0); \
             (spinlock).nb_readers ++; \
             LOCKING_REPORT("read=%d,writ=%d)\n", (spinlock).nb_readers, (spinlock).nb_writers); \
           } while(0)
#define shared_spin_lock read_spin_lock

#define read_spin_unlock(spinlock,flags) \
        do { LOCKING_REPORT("read_unlock_sl(" # spinlock " : "); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             ASSERT_FATAL((spinlock).nb_writers == 0); \
             ASSERT_FATAL((spinlock).nb_readers > 0); \
             (spinlock).nb_readers --; \
             LOCKING_REPORT("read=%d,writ=%d)\n", (spinlock).nb_readers, (spinlock).nb_writers); \
             restore_flags(flags); } while(0)
#define shared_spin_unlock read_spin_unlock

#define read_spin_unlock_remote(spinlock,flags) \
        do { LOCKING_REPORT("read_lock_sl_remote(" # spinlock ")\n"); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             ASSERT_FATAL((spinlock).nb_writers == 0); \
             ASSERT_FATAL((spinlock).nb_readers > 0); \
             (spinlock).nb_readers --; \
           } while(0)

#define write_spin_lock(spinlock,flags) \
        do { save_flags(flags); cli(); \
             LOCKING_REPORT("write_lock_sl(" # spinlock " : "); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             ASSERT_FATAL((spinlock).nb_readers == 0); \
             ASSERT_FATAL((spinlock).nb_writers == 0); \
             (spinlock).nb_writers = 1; \
             LOCKING_REPORT("read=%d,writ=%d)\n", (spinlock).nb_readers, (spinlock).nb_writers); \
           } while(0)
#define exclusive_spin_lock write_spin_lock

#define write_spin_unlock(spinlock,flags) \
        do { LOCKING_REPORT("write_unlock_sl(" # spinlock " : "); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             ASSERT_FATAL((spinlock).nb_readers == 0); \
             ASSERT_FATAL((spinlock).nb_writers == 1); \
             (spinlock).nb_writers = 0; \
             restore_flags(flags); \
             LOCKING_REPORT("read=%d,writ=%d)\n", (spinlock).nb_readers, (spinlock).nb_writers); \
           } while(0)
#define exclusive_spin_unlock write_spin_unlock


#define write_spin_unlock_remote(spinlock,flags) \
        do { LOCKING_REPORT("write_unlock_sl_remote(" # spinlock ")\n"); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             ASSERT_FATAL((spinlock).nb_readers == 0); \
             ASSERT_FATAL((spinlock).nb_writers == 1); \
             (spinlock).nb_writers = 0; \
           } while(0)


#define write_spin_lock_local(spinlock,flags) \
        do { save_flags(flags); cli(); \
             LOCKING_REPORT("write_lock_sl_local(" # spinlock ")\n"); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
           } while(0)

#define write_spin_unlock_local(spinlock,flags) \
        do { LOCKING_REPORT("write_unlock_sl_local(" # spinlock ")\n"); \
             ASSERT_FATAL((spinlock).state == LCK_READY); \
             restore_flags(flags); \
           } while(0)

/* FIXME on SMP: wait for no other read_lock before setting to write_lock */
#define read_to_write_spin_lock(spinlock,flags) \
	do { LOCKING_REPORT("read_to_write_spin_lock(" # spinlock " : "); \
	     ASSERT_FATAL((spinlock).state == LCK_READY); \
 	     ASSERT_FATAL((spinlock).nb_readers == 1); \
             ASSERT_FATAL((spinlock).nb_writers == 0); \
             (spinlock).nb_readers--; \
             (spinlock).nb_writers++; \
             LOCKING_REPORT("read=%d,writ=%d)\n", (spinlock).nb_readers, (spinlock).nb_writers); \
	} while(0)

#define write_to_read_spin_lock(spinlock,flags) \
	do { LOCKING_REPORT("write_to_read_spin_lock(" # spinlock " : "); \
	     ASSERT_FATAL((spinlock).state == LCK_READY); \
 	     ASSERT_FATAL((spinlock).nb_readers == 0); \
             ASSERT_FATAL((spinlock).nb_writers == 1); \
             (spinlock).nb_readers++; \
             (spinlock).nb_writers--; \
             LOCKING_REPORT("read=%d,writ=%d)\n", (spinlock).nb_readers, (spinlock).nb_writers); \
	} while(0)

#else


typedef struct { } spinlock_t;

#define SPINLOCK(spinlock) \
  spinlock_t spinlock
#define spinlock_init(spinlock)
#define EXPORT_SPINLOCK(spinlock)

#define read_spin_lock(spinlock,flags) \
        do { save_flags(flags); cli(); } while(0)
#define shared_spin_lock read_spin_lock

#define read_spin_unlock(spinlock,flags) \
        do { restore_flags(flags); } while(0)
#define shared_spin_unlock read_spin_unlock

#define read_spin_unlock_remote(spinlock,flags) \
        do { } while(0)

#define write_spin_lock(spinlock,flags) \
        do { save_flags(flags); cli(); } while(0)
#define exclusive_spin_lock write_spin_lock

#define write_spin_unlock(spinlock,flags) \
        do { restore_flags(flags); } while(0)
#define exclusive_spin_unlock write_spin_unlock

#define write_spin_unlock_remote(spinlock,flags) \
        do { } while(0)

#define write_spin_lock_local(spinlock,flags) \
        do { save_flags(flags); cli(); } while(0)

#define write_spin_unlock_local(spinlock,flags) \
        do { restore_flags(flags); } while(0)

#define read_to_write_spin_lock(spinlock,flags)

#define write_to_read_spin_lock(spinlock,flags)

#endif

#define disable_local_interrupts(flags) \
        do { save_flags(flags); cli(); } while(0)

/* These two macros are ONLY used by tty in order to avoid infinite
   recursion in ktty_printk ! */
#define write_spin_unlock_no_debug(spinlock,flags) \
        do { restore_flags(flags); } while(0)
#define write_spin_lock_no_debug(spinlock,flags) \
        do { save_flags(flags); cli(); } while(0)


extern spinlock_t kernel_thread_list_lock;
extern spinlock_t kernel_team_list_lock;
extern spinlock_t kernel_kslab_lock;
extern spinlock_t sem_queue_lock;
extern spinlock_t kernel_kvalloc_lock;
extern spinlock_t kernel_team_pd_bitmap_lock;
/* Lock used to lock all kernel space PDs and PTs */
extern spinlock_t kernel_space_lock;

#endif /* __SPINLOCK_H__ */
