/*
 * Copyright (C) 2000
 * http://kos.enix.org
 *
 * A few indispensable standard stdio functions
 * Most of them are taken from the linux (2.4.0) sources
 *
 * @(#) $Id: conio.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <loader/mod.h>
#include <lib/stdio.h>
#include <lib/string.h>
#include <kos/assert.h>
#include <kos/spinlock.h>

#define COLUMNS			80
#define LINES			25
#define ATTRIBUTE		7

static int xpos;
static int ypos;

volatile unsigned char *video = (char*) 0x0;

static char printk_buf[1024];

inline static void scrollup_n (int n)
{
  int i;

  if (n <= 0)
    return;
  if (n >= LINES) {
    cls();
    return;
  }

  for (i = 0 ; i < LINES-n ; i++)
    memcpy((void *)(video + i*COLUMNS*2), (void *)(video + (i+n)*COLUMNS*2),
	   COLUMNS*2);
  for (; i < LINES ; i++)
    memset((void *)(video + i*COLUMNS*2), 0x0, COLUMNS*2);
}

inline static void scrollup_1 ()
{
  int i;
  for (i = 0 ; i < LINES-1 ; i++)
    memcpy((void*)(video + i*COLUMNS*2), (void*)(video + (i+1)*COLUMNS*2),
	   COLUMNS*2);
  memset((void*)(video + i*COLUMNS*2), 0x0, COLUMNS*2);
}


inline static void console_putchar_unsafe (int c)
{
  if (c == '\n' || c == '\r')
    {
    newline:
      xpos = 0;
      ypos++;
      if (ypos >= LINES) {
	scrollup_1();
	ypos--;
      }
      return;
    }

  *(video + (xpos + ypos * COLUMNS) * 2) = c & 0xFF;
  *(video + (xpos + ypos * COLUMNS) * 2 + 1) = ATTRIBUTE;

  xpos++;
  if (xpos >= COLUMNS)
    goto newline;
}


int console_printk(const char *fmt, ...)
{
  va_list args;
  k_ui32_t flags;
  int i, len;
  
  va_start(args, fmt);
  
  disable_local_interrupts(flags);
  len = vsnprintf(printk_buf, sizeof(printk_buf), fmt, args);
  for (i=0; i < len; i++) 
    console_putchar_unsafe(printk_buf[i]);
  restore_flags(flags);
  return i;
}

static int console_prints(const char *str)
{
  const char *c;
  k_ui32_t flags;

  disable_local_interrupts(flags);
  for (c = str; *c != '\0' ; c++)
    console_putchar_unsafe(*c);
  restore_flags(flags);

  return (int) (c-str);
}


void cls (void)
{
  int i;
  k_ui32_t flags;

  disable_local_interrupts(flags);
  for (i = 0; i < COLUMNS * LINES * 2; i++)
    *(video + i) = 0;

  xpos = 0;
  ypos = 0;
  restore_flags(flags);
}


void console_putchar (int c)
{
  k_ui32_t flags;

  disable_local_interrupts(flags);
  console_putchar_unsafe(c);
  restore_flags(flags);
}

EXPORT_FUNCTION(cls);
EXPORT_FUNCTION(console_putchar);
EXPORT_FUNCTION(console_printk);
EXPORT_FUNCTION(console_prints);
