#include <lib/libcmdline.h>
#include <lib/string.h>
#include <loader/mod.h>

#include <kos/macros.h>
#include <kmem/kmem.h>

#include <debug/debug.h>

#include <kos/types.h>

char *cmdline_get_module_parameters(kernel_parameter_t *kp, const char *modname, const char *pname)
{
  char *begin;
  int param_len = 0;
  char *params;

  char *cur;
  int pname_len = 0;

  int c;
  bool_t found = FALSE;
  bool_t strip_coma = FALSE;

  module_parameter_t *cparam_entry;

  CONCEPTION_ASSERT(modname && pname);

  /* notice if there are more than 1 entry for modname, it considers
     only the first one ! */
  for (c = 0, cparam_entry = kp->module_parameters ; 
       c < kp->nb_modules_parameters_entries ; 
       cparam_entry++, c++ )
    {
      if (!strcmp(cparam_entry->modname,modname))
	{
	  found = TRUE;
	  break;
	}
    }

  if (!found)
    return NULL;


  found = FALSE;
  cur = cparam_entry->params;
  pname_len = strlen(pname);

  for ( ; *cur != '\0' ; cur++ )
    {
      if (!strncmp(cur,pname,pname_len))
      {
	found = TRUE;
	break;
      }

      while (*(cur + 1) != '\0')
      {
	if (*cur == '"')
	{
	  strip_coma = !strip_coma;
	}
	else if (*cur == ',' && !strip_coma)
	{
	  break;
	}

	cur++;
      }
   }

  if (!found)
    return NULL;
  

  /* skip the string "pname=" */
  cur += pname_len + 1;
  begin = cur;
  strip_coma = FALSE;

  for ( ; *cur != '\0' ; cur++ )
      {
	/* check for syntax error */
	if (*cur == ',' && !strip_coma)
	  {
	    break;
	  }

	/* skip comas check if between double quotes */
	if (*cur == '"')
	  {
	    strip_coma = !strip_coma;
	  }
	  
	param_len++;
      }

  
  if (param_len == 0)
    return NULL;

  if (*begin == '"')
    {
      param_len -= 2;
      begin++;
    }
  
  params = kmalloc(param_len + 1);
  if (!params)
    return NULL;

  strncpy(params, begin, param_len);
  *(params + param_len) = '\0';

  return params;
}

EXPORT_FUNCTION(cmdline_get_module_parameters);

