/*
 * Copyright (C) 2000
 * http://kos.enix.org
 *
 * Standard string functions
 *
 * @(#) $Id: memcpy.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <loader/mod.h>
#include <lib/string.h>
#include <kos/types.h>
#include <kos/macros.h>
#include <kos/assert.h>

typedef int word; // used by memcpy

#define wsize        sizeof(k_ui32_t)
#define wmask        (wsize - 1)
#define RETURN        return (dst0)
#define VAL        c0
#define WIDEVAL        c
#define UINT_MAX 0xffffffff


EXPORT_FUNCTION(memcpy);


// Optimized version of memcpy, from FreeBSD sources.

void *memcpy( void *dst0, const void *src0, register unsigned int length)
{
        register char *dst = dst0;
        register const char *src = src0;
        register unsigned int t;

        if (length == 0 || dst == src)                /* nothing to do */
                goto done;

        /*
         * Macros: loop-t-times; and loop-t-times, t>0
         */
#define TLOOP(s) if (t) TLOOP1(s)
#define TLOOP1(s) do { s; } while (--t)

        if ((unsigned long)dst < (unsigned long)src) {
                /*
                 * Copy forward.
                 */
                t = (int)src;        /* only need low bits */
                if ((t | (int)dst) & wmask) {
                        /*
                         * Try to align operands.  This cannot be done
                         * unless the low bits match.
                         */
                        if ((t ^ (int)dst) & wmask || length < wsize)
                                t = length;
                        else
                                t = wsize - (t & wmask);
                        length -= t;
                        TLOOP1(*dst++ = *src++);
                }
                /*
                 * Copy whole words, then mop up any trailing bytes.
                 */
                t = length / wsize;
                TLOOP(*(word *)dst = *(word *)src; src += wsize; dst += wsize);
                t = length & wmask;
                TLOOP(*dst++ = *src++);
        } else {
                /*
                 * Copy backwards.  Otherwise essentially the same.
                 * Alignment works as before, except that it takes
                 * (t&wmask) bytes to align, not wsize-(t&wmask).
                 */
                src += length;
                dst += length;
                t = (int)src;
                if ((t | (int)dst) & wmask) {
                        if ((t ^ (int)dst) & wmask || length <= wsize)
                                t = length;
                        else
                                t &= wmask;
                        length -= t;
                        TLOOP1(*--dst = *--src);
                }
                t = length / wsize;
                TLOOP(src -= wsize; dst -= wsize; *(word *)dst = *(word *)src);
                t = length & wmask;
                TLOOP(*--dst = *--src);
        }
done:
        return (dst0);
}
