/*
 * Copyright (C) 2000
 * http://kos.enix.org
 *
 * Standard string functions : memset
 *
 * @(#) $Id: memset.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <loader/mod.h>
#include <lib/string.h>
#include <kos/types.h>
#include <kos/macros.h>
#include <kos/assert.h>

typedef int word; // used by memcpy

#define wsize        sizeof(k_ui32_t)
#define wmask        (wsize - 1)
#define RETURN        return (dst0)
#define VAL        c0
#define WIDEVAL        c
#define UINT_MAX 0xffffffff


EXPORT_FUNCTION(memset);


// Optimized version for memset, found in FreeBSD sources

void *memset(void *dst0, register int c0, register unsigned int length)
{
        register unsigned int t;
        register unsigned int c;
        register unsigned char *dst;

        dst = dst0;
        /*
         * If not enough words, just fill bytes.  A length >= 2 words
         * guarantees that at least one of them is `complete' after
         * any necessary alignment.  For instance:
         *
         *        |-----------|-----------|-----------|
         *        |00|01|02|03|04|05|06|07|08|09|0A|00|
         *                  ^---------------------^
         *                 dst                 dst+length-1
         *
         * but we use a minimum of 3 here since the overhead of the code
         * to do word writes is substantial.
         */
        if (length < 3 * wsize) {
                while (length != 0) {
                        *dst++ = VAL;
                        --length;
                }
                RETURN;
        }

        if ((c = (unsigned char)c0) != 0) {        /* Fill the word. */
                c = (c << 8) | c;        /* u_int is 16 bits. */
#if UINT_MAX > 0xffff
                c = (c << 16) | c;        /* u_int is 32 bits. */
#endif
#if UINT_MAX > 0xffffffff
                c = (c << 32) | c;        /* u_int is 64 bits. */
#endif
        }

        /* Align destination by filling in bytes. */
        if ((t = (long)dst & wmask) != 0) {
                t = wsize - t;
                length -= t;
                do {
                        *dst++ = VAL;
                } while (--t != 0);
        }

        /* Fill words.  Length was >= 2*words so we know t >= 1 here. */
        t = length / wsize;
        do {
                *(unsigned int *)dst = WIDEVAL;
                dst += wsize;
        } while (--t != 0);

        /* Mop up trailing bytes, if any. */
        t = length & wmask;
        if (t != 0)
                do {
                        *dst++ = VAL;
                } while (--t != 0);
        RETURN;
}
