/*
 * Copyright (C) 2000, Thomas PETAZZONI & David DECOTIGNY
 * http://kos.enix.org
 *
 * Binary search tree library. This header is used to select the type
 * of tree used for KOS : splay trees, AVL trees.
 *
 * @(#) $Id: libbst.h,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $ 
 */

#ifndef __LIBBST_H__
#define __LIBBST_H__

#define SPLAY_TREES
/* BE CAREFUL : AVL TREES AREN'T IMPLEMENTED YET */
/* #define AVL_TREES */


typedef struct tree_node tree_node_t;

struct tree_node
{
  unsigned int key;

  tree_node_t *up;
  tree_node_t *left;
  tree_node_t *right;
};

/* Returns true when to stop the walk-through */
typedef int (*visitor_t)(tree_node_t *node, void *client_data);

/* Returns -1 when node "<" client_data, 0 when equal, 1 when node ">" client_data */
typedef int (*compare_fct_t)(tree_node_t *node, void *client_data);

/* Splay trees */
#ifdef SPLAY_TREES

#include <libbst/_splay.h>

#define insert_node(a,b) _splay_insert_node(a,b)
#define delete_node(a,b) _splay_delete_node(a,b)
#define visit(a,b,c) _splay_visit(a,b,c)
#define bst_reset(tree,cleaner,client_data)      _splay_reset(tree,cleaner,client_data)
#define bst_lookup(tree,compare_fct,client_data) _splay_lookup(tree,compare_fct,client_data)

#endif /* SPLAY_TREES*/

/* AVL trees */
#ifdef AVL_TREES

#include <libbst/_avl.h>

#define insert_node(a,b) _avl_insert_node(a,b)
#define delete_node(a,b) _avl_delete_node(a,b)
#define visit(a,b,c) _avl_visit(a,b,c)
#define empty_tree(a,b,c) _avl_empty_free(a,b,c)

#endif /* AVL_TREES */

#endif /* __LIBBST_H__ */
