/* libhash.h - Copyright (c) 2002 Julien Munier <mejj@enix.org>
   libhash public include file

   the Kid Operating System (http://kos.enix.org)
   @(#) $Id: libhash.h,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $ */

#ifndef __LIBHASH_H__
#define __LIBHASH_H__

#include <kmem/kmem.h>

struct hash_item;

typedef int (*destruct_fn)(void *);
typedef int (*compare_fn)(void *, void *);
typedef int (*visit_fn)(struct hash_item *);

typedef struct hash {
  unsigned int prime;

  struct hash_item **table;
  destruct_fn destruct;
  compare_fn cmp;
} hash_t;

typedef struct hash_item {
  unsigned int key;
  struct hash_item *prev, *next;

  void *private;
} hash_item_t;


#define hash(h,i) ((i) % (h)->prime))

int hash_init(void);

/* @param destruct_fn may be NULL => equivalent to kfree */
struct hash *hash_create_table(unsigned int, compare_fn, destruct_fn);
int hash_delete_table(struct hash *);

int hash_set_cmp_fn(struct hash *, compare_fn );

unsigned int hash_std_str_to_int(struct hash *, const char *);

int hash_insert(struct hash *, unsigned int, void *);

int hash_remove(struct hash *, unsigned int, void *);

struct hash_item *hash_find(struct hash *, unsigned int, void *);
int hash_visit(struct hash *, visit_fn );


#endif /* __LIBHASH_H__ */


