#ifndef __LIBLIST_H__
#define __LIBLIST_H__

/*
 * Circular doubly-linked lists Implementation
 * KOS Project
 * (C) D. Decotigny 2001
 */

/* *_named are used when next and prev links are not exactly next
   and prev. For instance when we have next_in_team, prev_in_team,
   prev_global and next_global */

#define list_init_named(list,item,prev,next) ({ \
  (item)->next = (item)->prev = (item); \
  (list) = (item); \
})

#define list_get_head_named(list,prev,next) \
  (list)

#define list_get_tail_named(list,prev,next) \
  ((list)?((list)->prev):NULL)

/* Caution: before_this and item are expected to be valid ! */
#define list_insert_before_named(before_this,item,prev,next) ({ \
   (before_this)->prev->next = (item); \
   (item)->prev = (before_this)->prev; \
   (before_this)->prev = (item); \
   (item)->next = (before_this); \
})

/* Caution: after_this and item are expected to be valid ! */
#define list_insert_after_named(after_this,item,prev,next) ({ \
   (after_this)->next->prev = (item); \
   (item)->next = (after_this)->next; \
   (after_this)->next = (item); \
   (item)->prev = (after_this); \
})

#define list_add_head_named(list,item,prev,next) ({ \
  if (list) \
    list_insert_before_named(list,item,prev,next); \
  else \
    list_init_named(list,item,prev,next); \
  (list) = (item); \
})

#define list_add_tail_named(list,item,prev,next) ({ \
  if (list) \
    list_insert_before_named(list,item,prev,next); \
  else \
    list_init_named(list,item,prev,next); \
})

/* Caution: before_this and a_list are expected to be valid ! */
#define list_splice_before_named(before_this,a_list,prev,next) ({ \
  typeof(a_list) ___a_last  = (a_list)->prev; \
  (before_this)->prev->next = (a_list); \
  (a_list)->prev            = (before_this)->prev; \
  (before_this)->prev       = ___a_last; \
  ___a_last->next           = (before_this); \
})

/* Caution: after_this and a_list are expected to be valid ! */
#define list_splice_after_named(after_this,a_list,prev,next) ({ \
  typeof(a_list) ___a_last = (a_list)->prev; \
  (after_this)->next->prev = ___a_last; \
  ___a_last->next          = (after_this)->next; \
  (after_this)->next       = (a_list); \
  (a_list)->prev           = (after_this); \
})

#define list_splice_head_named(in_list,a_list,prev,next) ({ \
  if (! (in_list)) \
    (in_list) = (a_list); \
  else if (a_list) \
    list_splice_before_named(in_list, a_list, prev, next); \
  (in_list) = (a_list); \
})

#define list_splice_tail_named(in_list,a_list,prev,next) ({ \
  if (! (in_list)) \
    (in_list) = (a_list); \
  else if (a_list) \
    list_splice_before_named(in_list, a_list, prev, next); \
})

/* Caution: no check whether item really is in list ! */
#define list_delete_named(list,item,prev,next) ({ \
  if ( ((item)->next == (item)) && ((item)->prev == (item)) ) \
    (item)->next = (item)->prev = (list) = NULL; \
  else { \
    (item)->prev->next = (item)->next; \
    (item)->next->prev = (item)->prev; \
    if ((item) == (list)) (list) = (item)->next; \
    (item)->prev = (item)->next = NULL; \
  } \
})

#define list_pop_head_named(list,prev,next) ({ \
  typeof(list) __ret_elt = (list); \
  list_delete_named(list,__ret_elt,prev,next); \
  __ret_elt; })

#define list_foreach_forward_named(list,iterator,nb_elements,prev,next) \
        for (nb_elements=0, (iterator) = (list) ; \
             (iterator) && (!nb_elements || ((iterator) != (list))) ; \
             nb_elements++, (iterator) = (iterator)->next )

#define list_foreach_backward_named(list,iterator,nb_elements,prev,next) \
        for (nb_elements=0, (iterator) = list_get_tail_named(list,prev,next) ; \
             (iterator) && (!nb_elements || ((iterator) != (list))) ; \
             nb_elements++, (iterator) = (iterator)->prev )

#define list_foreach_named list_foreach_forward_named

#define list_foreach_early_break(list,iterator,nb_elements) \
  ((list) && ( \
    ((list) != (iterator)) || \
    ( ((list) == (iterator)) && (nb_elements == 0)) ))

#define list_collapse_named(list,iterator,prev,next) \
        for ( ; ({ ((iterator) = (list)) ; \
                   if (list) list_delete_named(list,iterator,prev,next) ; \
                   (iterator); }) ; )

#define list_init(list,item) \
  list_init_named(list,item,prev,next)

#define list_get_head(list) \
  list_get_head_named(list,prev,next) \

#define list_get_tail(list) \
  list_get_tail_named(list,prev,next) \

/* Caution: before_this and item are expected to be valid ! */
#define list_insert_after(after_this,item) \
  list_insert_after_named(after_this,item,prev,next)

/* Caution: after_this and item are expected to be valid ! */
#define list_insert_before(before_this,item) \
  list_insert_before_named(before_this,item,prev,next)

#define list_add_head(list,item) \
  list_add_head_named(list,item,prev,next)

#define list_add_tail(list,item) \
  list_add_tail_named(list,item,prev,next)

/* Caution: before_this and a_list are expected to be valid ! */
#define list_splice_before(before_this,a_list) \
  list_splice_before_named(before_this,a_list,prev,next)

/* Caution: after_this and a_list are expected to be valid ! */
#define list_splice_after(after_this,a_list) \
  list_splice_after_named(after_this,a_list,prev,next)

#define list_splice_head(to_list,a_list) \
  list_splice_head_named(to_list,a_list,prev,next)

#define list_splice_tail(to_list,a_list) \
  list_splice_tail_named(to_list,a_list,prev,next)

/* Caution: no check whether item really is in list ! */
#define list_delete(list,item) \
  list_delete_named(list,item,prev,next)

#define list_pop_head(list) \
  list_pop_head_named(list,prev,next)

#define list_foreach_forward(list,iterator,nb_elements) \
  list_foreach_forward_named(list,iterator,nb_elements,prev,next)

#define list_foreach_backward(list,iterator,nb_elements) \
  list_foreach_backward_named(list,iterator,nb_elements,prev,next)

#define list_foreach list_foreach_forward

#define list_collapse(list,iterator) \
  list_collapse_named(list,iterator,prev,next)

#endif /* __LIBLIST_H__ */
