/*
 * Copyright (C) 2000, PETAZZONI Thomas & David Decotigny
 * http://kos.enix.org
 *
 * Physical memory management.
 * 
 * @(#) $Id: _pmm.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <lib/stdio.h>
#include <kos/macros.h>
#include <debug/debug.h>

#include "_pmm.h"

// Main memory size
size_t main_memory_size = 0;
struct _gpfm_lists_s gpfm;

size_t _get_gpfm_ram_map_size()
{
  return gpfm.ram_map_size;
}

int gpfme_unlock(gpfme_t* gpfme, k_ui32_t* gpfm_lock_flags)
{
  UNUSED(gpfme);
  RETURN_VAL_IF_FAIL_VERBOSE(gpfm_lock_flags, -1);
  write_spin_unlock(gpfm.lock, *gpfm_lock_flags);
  return 0;
}

/* Return -1 on error, old swap status when Ok */
int change_gpfme_swap_status(gpfme_t *gpfme, int new_swap_status)
{
  int old_swap_status;

  RETURN_VAL_IF_FAIL_VERBOSE(gpfme, -1);

  if (gpfme->flags.page_type != PHYS_PAGE_USER)
    {
      WARNING("Not allowed to change swap status for non-user pages\n");
      return -1;
    }

  old_swap_status = gpfme->flags.swap_status;
  if (new_swap_status == old_swap_status)
    {
      return old_swap_status;
    }

  if (old_swap_status == PHYS_PAGE_SWAPPABLE)
    GPFM_LIST_DEL(swappable,gpfme);
  else
    GPFM_LIST_DEL(non_swappable,gpfme);
  
  if (new_swap_status == PHYS_PAGE_SWAPPABLE)
    GPFM_LIST_ADD(swappable, gpfme);
  else
    GPFM_LIST_ADD(non_swappable, gpfme);
  
  return old_swap_status;
}
