/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Include for physical memory management
 *
 * @(#) $Id: _pmm.h,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */


#ifndef ___PMM_H__
#define ___PMM_H__

#include <kos/types.h>
#include <loader/mod.h>
#include <kos/macros.h>
#include <liblist/liblist.h>
#include <kos/spinlock.h>
#include <pmm/pmm.h>

/*
 * Cannot further split the lock into
 * free/swappable/non_swappable/hw_mapping, because moving an element
 * from one list to the others requires getting the lock on both
 * lists. But the problem is that the order along which this is done
 * is not always the same ! For example, free->swappable and
 * swappable->free are both valid moves.
 *
 * Lock order: gpfme.lock then gpfm.lock. this ensures that vmap and
 * pmm have always consistent views: when pmm is moving the gpfme from
 * one list to the other, the gpfme being also locked ensures that
 * vmap cannot make progress in the meantime. Put another way, as log
 * as vmap holds the gpfme lock, pmm cannot move the gpfme in another
 * place.
 */

/* Global Page Frame lists */
struct _gpfm_lists_s {
  gpfme_t* ram_map;       // Array
  size_t   ram_map_size;

  gpfme_t* free;          // list
  gpfme_t* non_swappable; // list
  gpfme_t* swappable;     // list
  gpfme_t* hw_mapping;    // list

  spinlock_t lock;
};
extern struct _gpfm_lists_s gpfm;

/* RAM size */
extern size_t main_memory_size;

#define INIT_GPFME(gpfme) \
  ({ (gpfme)->slab = NULL ; \
     (gpfme)->ref_cnt = 0 ; \
     (gpfme)->mapping_list = NULL; })

#define GPFM_LIST_FOREACH(_listname,_visitor,_nb) \
  list_foreach_forward_named(gpfm._listname,_visitor,_nb,u._listname.prev,u._listname.next)

#define GPFM_LIST_ADD(listname,item) \
  list_add_head_named(gpfm.listname,item,u.listname.prev,u.listname.next)

#define GPFM_LIST_DEL(listname,item) /* DEL=Remove */ \
  list_delete_named(gpfm.listname,item,u.listname.prev,u.listname.next)

int init_gpfm(kernel_parameter_t *kp);

#endif /* __pmm_h__ */
