
#include <kos/macros.h>
#include <debug/debug.h>
#include <arch/mm/mm.h>
#include <kmem/kmem.h>

#include "_pmm.h"

int new_additional_allocatable_page(paddr_t paddr)
{
  int ret;
  k_ui32_t flags;
  gpfme_t* gpfme;

  /* Don't go any further if this assertion turns to be wrong */
  RETURN_VAL_IF_FAIL_VERBOSE(paddr >= main_memory_size, -1);

  /* Setup new gpfme */
  gpfme = kmalloc(sizeof(gpfme_t));
  RETURN_VAL_IF_FAIL_VERBOSE(gpfme, -1);

  INIT_GPFME(gpfme);
  gpfme->address           = paddr;
  gpfme->flags.page_type   = PHYS_PAGE_FREE;
  gpfme->flags.swap_status = PHYS_PAGE_NON_SWAPPABLE;

  /* Try to add the new gpfme in the free pages list */
  ret = 0;
  if (get_gpfme_at_phys_addr(paddr, & flags))
    ret = -1; /* Someone is already there */
  else
    GPFM_LIST_ADD(free, gpfme); /* Ok, add to the free pages list */
  write_spin_unlock(gpfm.lock, flags);  

  /* Someone else is already mapped at paddr */
  if (ret == -1)
    kfree(gpfme);

  return ret;
}


int remove_additional_allocatable_page(paddr_t paddr)
{
  int ret;
  k_ui32_t gpfm_lock_flags;
  gpfme_t* gpfme;

  /* If paddr is not really an "additional allocatable aread", don't
     go any further */
  RETURN_VAL_IF_FAIL_VERBOSE(paddr >= main_memory_size, -1);

  /* Get the gpfme at paddr, retrieve it from the free list */
  gpfme = get_gpfme_at_phys_addr(paddr, & gpfm_lock_flags);
  if (! gpfme)
    {
      write_spin_unlock(gpfm.lock, gpfm_lock_flags);
      return -1;
    }

  ret = 0;
  if (gpfme->flags.page_type == PHYS_PAGE_FREE)
    {
      GPFM_LIST_DEL(free, gpfme);
    }
  else
    ret = -1;
  write_spin_unlock(gpfm.lock, gpfm_lock_flags);

  /* No gpfme for this address, or gpfme still used */
  if (ret == -1)
    return -1;

  /* Release unused gpfme */
  kfree(gpfme);

  /* Free the gpfme for this address */
  return 0;
}
