
#include <kos/macros.h>
#include <debug/debug.h>
#include <arch/mm/mm.h>

#include "_pmm.h"

gpfme_t *get_gpfme_at_phys_addr(addr_t paddr, k_ui32_t* flags)
{
  int nb;
  gpfme_t  *gpfme, *result;

  write_spin_lock(gpfm.lock, *flags);

  /* If the requested address is in main memory (or mapped in a main
     memory area), gpfme are direct-mapped */
  if (paddr < main_memory_size)
    {
      result = gpfm.ram_map + (paddr >> PAGE_SIZE_SHIFT);
      return result;
    }


  /* ---------------------------------------------------------------
   * The requested address is not in the "main memory" area: need to
   * visit the lists.
   */

  /* Used below */
#define WATCH_AFTER(listname) \
  ({ \
  result = NULL; \
  GPFM_LIST_FOREACH(listname, gpfme, nb) \
    if (paddr == gpfme->address) \
      { \
	result = gpfme; \
	break; \
      } \
  if (result) \
     return result; \
  })
  /* [GARBAGE Note] "return result;" statement: we cannot lock the
     gpfme before the "unlock lists" (order is gpfme lock/lists
     lock). So we'll be able to lock the gpfme only AFTER the "unlock
     lists". Thus, when a gpfme for an hw_mapping or a
     additional_allocatable_page is returned, it MUST NOT be freed by
     another thread ! */

  /* Try hw_mapping ? */
  WATCH_AFTER(hw_mapping);

  /*
   * If we are here, that's because the system has another block of
   * memory that does not correspond to the usual RAM.
   * This would not happen in "conventional systems".
   */

  /* Otherwise => try the free list first */
  WATCH_AFTER(free);

  /* Otherwise => try the non swappable allocated list */
  WATCH_AFTER(non_swappable);

  /* Otherwise => try the swappable allocated list */
  WATCH_AFTER(swappable);

  return NULL;
}

gpfme_t *get_gpfme_at_virt_addr(vaddr_t vaddr, k_ui32_t* flags)
{
  addr_t paddr = arch_get_paddr_at_vaddr(vaddr);
  
  if(!paddr)
    {
      write_spin_lock(gpfm.lock, *flags);
      return NULL;
    }

  return get_gpfme_at_phys_addr(paddr, flags);
}
