#include <kos/macros.h>
#include <debug/debug.h>
#include "_pmm.h"

/* page_type MUST be either PHYS_PAGE_USER or PHYS_PAGE_KERNEL (NO
   check) */
paddr_t get_physical_page(int page_type, int swap_status)
{
  k_ui32_t flags;
  gpfme_t* gpfme;

  RETURN_VAL_IF_FAIL_VERBOSE((page_type == PHYS_PAGE_KERNEL)
			     || (page_type == PHYS_PAGE_USER),
			     0);

  write_spin_lock(gpfm.lock, flags);

  gpfme = list_get_head_named(gpfm.free, u.free.prev, u.free.next);
  GPFM_LIST_DEL(free, gpfme);
  INIT_GPFME(gpfme);
  gpfme->flags.page_type   = page_type;
  gpfme->flags.swap_status = swap_status;
  if (swap_status == PHYS_PAGE_SWAPPABLE)
    GPFM_LIST_ADD(swappable, gpfme);
  else
    GPFM_LIST_ADD(non_swappable, gpfme);

  write_spin_unlock(gpfm.lock, flags);

  return gpfme->address;
}
