
#include <kos/macros.h>
#include <debug/debug.h>
#include <arch/mm/mm.h>
#include <kmem/kmem.h>

#include "_pmm.h"


int declare_hw_mapping_page(paddr_t paddr)
{
  int ret;
  k_ui32_t flags;
  gpfme_t* gpfme;

  gpfme = get_gpfme_at_phys_addr(paddr, &flags);
  
  /* The hw_mapping will over overlap an allocatable area */
  if (gpfme)
    {
      /* Make sure the allocatable area is not already allocated =>
	 reclaimable */
      ret = 0;
      if (gpfme->flags.page_type == PHYS_PAGE_FREE)
	{
	  GPFM_LIST_DEL(free, gpfme);
	  INIT_GPFME(gpfme);
	  gpfme->flags.page_type = PHYS_PAGE_HW_MAPPING;
	  gpfme->flags.hw_mapping_reclaiming_status
	    = PHYS_PAGE_HW_MAPPING_RECLAIMABLE;
	  CONCEPTION_ASSERT(gpfme->address == paddr);
	  GPFM_LIST_ADD(hw_mapping, gpfme);
	}
      else
	ret = -1;

      write_spin_unlock(gpfm.lock, flags);
      return ret;
    }

  /* Release lock */
  write_spin_unlock(gpfm.lock, flags);

  /* No: the hw_mapping will be on its own (+gpfm not locked)
     => hw_mapping not reclaimable */
  
  gpfme = kmalloc(sizeof(gpfme_t));
  RETURN_VAL_IF_FAIL_VERBOSE(gpfme, -1); /* unlocked */
    
  /* Ok: build the new gpfme */
  INIT_GPFME(gpfme);
  gpfme->address = paddr;
  gpfme->flags.page_type = PHYS_PAGE_HW_MAPPING;
  gpfme->flags.hw_mapping_reclaiming_status
    = PHYS_PAGE_HW_MAPPING_NON_RECLAIMABLE;
  
  /* Try to insert it in the hw_mapping list */
  ret = 0;
  /* Make sure someone did not insert in the meantime (+lock gpfm) */
  if (get_gpfme_at_phys_addr(paddr, & flags))
    ret = -1;
  else
    GPFM_LIST_ADD(hw_mapping, gpfme);
  write_spin_unlock(gpfm.lock, flags);

  if (ret != 0)
    /* Place got busy while we kmalloced the new gpfme */
    kfree(gpfme);

  return ret;
}
