
#include <kos/macros.h>
#include <debug/debug.h>
#include <kmem/kmem.h>

#include "_pmm.h"

/* gpfme AND gpfm MUST be locked */
/* Return TRUE when gpfme must be kfree() */
static inline bool_t _release_gpfme_unsafe(gpfme_t* gpfme)
{
  /* Make sure nobody shares the page ! */
  RETURN_VAL_IF_FAIL_VERBOSE(gpfme->ref_cnt == 0, FALSE);

  /* If this is a "normal" page (user or kernel data), first remove it
     from its list */
  if ((gpfme->flags.page_type == PHYS_PAGE_KERNEL)
      || (gpfme->flags.page_type == PHYS_PAGE_USER))
    {
      if (gpfme->flags.swap_status == PHYS_PAGE_SWAPPABLE)
	{
	  GPFM_LIST_DEL(swappable,gpfme);
	}
      else
	{
	  GPFM_LIST_DEL(non_swappable,gpfme);      
	}
    }
  /* if this is a hardware mapping, remove it from the hw_mapping */
  else if (gpfme->flags.page_type == PHYS_PAGE_HW_MAPPING)
    {
      GPFM_LIST_DEL(hw_mapping,gpfme);
      /* Move it to the free list only if the hw_mapping is
         reclaimable. Otherwise, the gpfme element is released. */
      if (gpfme->flags.hw_mapping_reclaiming_status
	  == PHYS_PAGE_HW_MAPPING_NON_RECLAIMABLE)
	{
	  return TRUE; /* kfree */
	}
    }
  else if (gpfme->flags.page_type == PHYS_PAGE_FREE)
    return FALSE; // Somebody already freed the page
  else
    FAILED_VERBOSE("Invalid page type\n");

  /* Move the gpfme to the free pages' list */
  gpfme->flags.page_type   = PHYS_PAGE_FREE;
  gpfme->flags.swap_status = PHYS_PAGE_NON_SWAPPABLE;
  GPFM_LIST_ADD(free, gpfme);

  return FALSE;
}

int put_physical_page(paddr_t paddr)
{
  k_ui32_t flags;
  bool_t to_be_freed;
  gpfme_t* gpfme;
  
  gpfme = get_gpfme_at_phys_addr(paddr, & flags);
  if (! gpfme)
    {
      write_spin_unlock(gpfm.lock,flags);
      return -1;
    }

  to_be_freed = _release_gpfme_unsafe(gpfme);
  write_spin_unlock(gpfm.lock,flags);

  if (to_be_freed)
    kfree(gpfme);
  
  return 0;
}

