#include <kos/macros.h>
#include <debug/debug.h>

#include "_pmm.h"

int _gpfm_visit_list_unsafe(int page_type, pmm_list_visitor_t visitor, void* custom_param)
{
  gpfme_t* gpfme;
  int nb, res;
  k_ui32_t flags;

  ASSERT_RETURN_VAL(visitor, 0);

  res = 0;
  write_spin_lock(gpfm.lock, flags);
  switch (page_type)
    {
    case PHYS_PAGE_FREE:
      GPFM_LIST_FOREACH(free, gpfme, nb)
	{
	  CONCEPTION_ASSERT(gpfme->flags.page_type == page_type);
	  res = visitor(gpfme, custom_param);
	  if (res)
	    break;
	}
      break;

    case PHYS_PAGE_HW_MAPPING:
      GPFM_LIST_FOREACH(hw_mapping, gpfme, nb)
	{
	  CONCEPTION_ASSERT(gpfme->flags.page_type == page_type);
	  res = visitor(gpfme, custom_param);
	  if (res)
	    break;
	}
      break;

    case PHYS_PAGE_KERNEL:
    case PHYS_PAGE_USER:
      GPFM_LIST_FOREACH(swappable, gpfme, nb)
	{
	  if (gpfme->flags.page_type != page_type)
	    continue;
	  res = visitor(gpfme, custom_param);
	  if (res)
	    break;
	}
      if (res)
	break;
      GPFM_LIST_FOREACH(non_swappable, gpfme, nb)
	{
	  if (gpfme->flags.page_type != page_type)
	    continue;
	  res = visitor(gpfme, custom_param);
	  if (res)
	    break;
	}
      break;

    default:
      res = 0;
    }

  write_spin_unlock(gpfm.lock, flags);
  return res;
}
