#include <scheduler/_scheduler.h>
#include <idt/irq.h>
#include <task/task.h>
#include <idt/dst.h>
#include <kitc/kmsg.h>
#include <kos/macros.h>

volatile unsigned char *twiddle;
volatile jiffy_t jiffies = 0;

struct dst_job *consumer;
struct dst_job *producer;
kmsg_port_t *timer_port;

static void timer_dst_job_consummer(int dst_level, void *arg)
{
  void *msg;

  UNUSED(dst_level);
  UNUSED(arg);

  msg = kmsg_receive(timer_port);
  
  if(!msg)
    __dbg_printk(_B_CYAN "error while receiving\n" _B_NORM);
}

static void timer_dst_job_producer(int dst_level, void *arg)
{
  UNUSED(dst_level);
  UNUSED(arg);

  kmsg_send(timer_port, (void *) jiffies);
}

jiffy_t get_jiffies(void)
{
  return jiffies;
}


void timer_handler (int number, cpu_state_t *cpu_state)
{
  UNUSED(number);
  UNUSED(cpu_state);
  
  *(twiddle +1) = 3;
  (*twiddle)++;

  jiffies++;

  if(((jiffies+1000) % 1000) == 0)
    {
      mark_job(consumer, 3);
    }
  
  if((jiffies % 1000) == 0)
    {
      mark_job(producer, 4);
    }

  check_timers(jiffies);

}

int init_timer (void)
{
  register_irq_handler(IRQ_TIMER, timer_handler);

  consumer = new_job(timer_dst_job_consummer, NULL);
  
  if(!consumer)
    FAILED_VERBOSE("Couldn't create consumer\n");

  producer = new_job(timer_dst_job_producer, NULL);
  
  if(!producer)
    FAILED_VERBOSE("Couldn't create producer\n");

  timer_port = kmsg_create_port();
  
  if(!timer_port)
    FAILED_VERBOSE("Couldn't create port\n");

  return 0;
}


