/*
 * Copyright (C) 2000, Thomas PETAZZONI & David DECOTIGNY
 * http://kos.enix.org
 *
 * Functions needed to create and delete teams
 *
 * @(#) $Id: _task_team.c,v 1.2 2003/04/27 20:55:55 thomas Exp $
 */

#include <kos/macros.h>
#include <kos/spinlock.h>
#include <liblist/liblist.h>
#include <task/_task.h>

team_t *create_team(void)
{
  k_ui32_t flags;
  team_t *new;
  int i;

  new = kslab_cache_alloc(kmem_team_cache);
  if(new == NULL)
    return NULL;
  
  new->thread_list = NULL;

  for (i = 0; i < MAX_FD_PER_TEAM; i++)
    {
      new->ures_array[i] = NULL;
    }

  if(arch_team_mm_context_construct(& new->mm_context))
    {
      kslab_cache_free(kmem_team_cache, new);
      return NULL;
    }

  if(init_team_as (new))
    {
      arch_team_mm_context_destruct(&new->mm_context);
      kslab_cache_free(kmem_team_cache, new);
      return NULL;
    }

  write_spin_lock(kernel_team_list_lock, flags);

  list_add_tail(team_global_list, new);

  write_spin_unlock(kernel_team_list_lock, flags);

  return new;
}

int delete_team(team_t *team)
{
  RETURN_VAL_IF_FAIL_VERBOSE(team == get_current_team(), -1);
  RETURN_VAL_IF_FAIL_VERBOSE(team == get_kernel_team(), -1);

  TBI("Virer le mm_context du team + team de la liste");
  
  return 0;
}

