/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Functions needed to create and delete user threads.
 *
 * @(#) $Id: _task_uthread.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <task/_task.h>
#include <kos/assert.h>
#include <kmem/kmem.h>
#include <kos/system.h>
#include <kos/spinlock.h>
#include <lib/string.h>
#include <pmm/pmm.h>
#include <arch/mm/mm.h>
#include <kos/macros.h>

thread_t *create_user_thread(team_t *team, addr_t eip, void *data)
{
  thread_t *new_user_thread;
  k_ui32_t flags;

  if (team == NULL)
    team = get_current_team();

  new_user_thread = kmalloc(sizeof(thread_t));
  CONCEPTION_ASSERT(new_user_thread != NULL);

  strncpy(new_user_thread->name, "<user thread>", 32);
  
  new_user_thread->state = STOPPED;
  new_user_thread->type  = USER_THREAD;

  /* Assign thread's team */
  new_user_thread->mother_team = team;
  
  write_spin_lock(kernel_thread_list_lock, flags);

  if (allocate_cpl0_stack(new_user_thread))
    {
      WARNING("No more CPL0 stacks !\n");
      write_spin_unlock(kernel_thread_list_lock, flags);
      return NULL;
    }
  
  register_thread (new_user_thread, team);
  init_user_thread_context (new_user_thread, eip, data);
  new_user_thread->state = READY;
  
  write_spin_unlock(kernel_thread_list_lock, flags);
  
  return new_user_thread;
}

void delete_user_thread(thread_t *thread)
{
  kfree_size_known(thread, sizeof(thread_t));
}

EXPORT_FUNCTION(create_user_thread);
