/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Utility functions for task management
 *
 * @(#) $Id: _task_utils.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <task/_task.h>
#include <kos/system.h>
#include <kos/assert.h>
#include <kos/spinlock.h>
#include <arch/task/task.h>
#include <task/task.h>
#include <kmem/kmem.h>
#include <liblist/liblist.h>

struct thread *current_thread = NULL;
team_t *kernel_team = NULL;
addr_t thread_end_handler_stack = 0;
struct thread *thread_global_list = NULL;
struct team *team_global_list = NULL;

thread_t *get_current_thread(void)
{
  return current_thread;
}

void set_current_thread(thread_t *thread)
{

  if(current_thread != NULL &&
     current_thread->mother_team != thread->mother_team)
    {
      asm volatile("movl %0,%%cr3"::"r"(thread->mother_team->mm_context.pd));
    }
  
  current_thread = thread;
}

thread_t *get_global_thread_list_head(void)
{
  return thread_global_list;
}


int show_all_thread_info (void)
{ 
#define CASE_STR(esac) case esac: str = # esac; break
  char *str;
  thread_t *thread;
  k_ui32_t flags;
  unsigned int nb_elements;

  __dbg_printk(_B_RED "******** THREAD INFORMATIONS ********\n");

  read_spin_lock(kernel_thread_list_lock, flags);
  list_foreach_forward_named(thread_global_list, thread, nb_elements, prev_global, next_global)
    {
      __dbg_printk("==> Thread %s (addr 0x%x)\n", 
		     thread->name,
		     (unsigned)thread);
      switch(thread->type)
	{
	  CASE_STR(KERNEL_THREAD);
	  CASE_STR(USER_THREAD);
	default:
	  str = "unknown";
	}
      __dbg_printk("    | Type = %s\n", str);

      switch(thread->state)
	{
	  CASE_STR(READY);
	  CASE_STR(RUNNING);
	  CASE_STR(INTERRUPTED);
	  CASE_STR(STOPPED);
	  CASE_STR(DELETED);
	  CASE_STR(SLEEPING);
	  CASE_STR(WAITING);
	default:
	  str = "unknown";
	}
      __dbg_printk("    | State = %s\n", str);

      if(thread->type == USER_THREAD)
	__dbg_printk("    | Pile CPL3 = 0x%x\n", 
		       thread->cpu_context.cpl3);

      __dbg_printk("    | Pile CPL0 = 0x%x\n", 
		   (addr_t)thread->cpu_context.cpl0);
    }

  read_spin_unlock(kernel_thread_list_lock, flags);

  __dbg_printk("*************************************\n" _B_NORM);

  return 0;

}


/*
 * Kernel is a quite special team.
 * It has no address space.
 * All kernel threads are part of this team.
 */
team_t * get_kernel_team(void)
{
  return kernel_team;
}



