/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Debug functions for modules
 *
 * @(#) $Id: standalone.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <loader/mod.h>
#include <lib/stdio.h>

void itoa (char *buf, int base, int d)
{
  char *p = buf;
  char *p1, *p2;
  unsigned long ud = d;
  int divisor = 10;
  
  // If %d is specified and D is minus, put `-' in the head. 
  if (base == 'd' && d < 0)
    {
      *p++ = '-';
      buf++;
      ud = -d;
    }
  else if (base == 'x')
    divisor = 16;

  // Divide UD by DIVISOR until UD == 0. 
  do
    {
      int remainder = ud % divisor;
      
      *p++ = (remainder < 10) ? remainder + '0' : remainder + 'a' - 10;
    }
  while (ud /= divisor);

  // Terminate BUF. 
  *p = 0;
  
  // Reverse BUF. 
  p1 = buf;
  p2 = p - 1;
  while (p1 < p2)
    {
      char tmp = *p1;
      *p1 = *p2;
      *p2 = tmp;
      p1++;
      p2--;
    }
}

void __bochs_putchar( int c ) 
{
  asm("outb %b0, $0xe9"::"a"(c):"memory");
}


void __dbg_printk(char *format, ...) 
{
  char **arg = (char **) &format;
  int c;
  char buf[20];
  
  arg++;
  

  while ((c = *format++) != 0)
    {
      if (c != '%')
	__bochs_putchar (c);
      else
	{
	  char *p;
	  
	  c = *format++;
	  switch (c)
	    {
	    case 'd':
	    case 'u':
	    case 'x':
	      itoa (buf, c, *((int *) arg++));
	      p = buf;
	      goto string;
	      break;

	    case 's':
	      p = *arg++;
	      if (! p)
		p = "(null)";

	    string:
	      while (*p)
		__bochs_putchar (*p++);
	      break;

	    default:
	      __bochs_putchar (*((int *) arg++));
	      break;
	    }
	}
    }
}

void __bochs_putlong(unsigned long num)
{
  __dbg_printk("%d (0x%x)\n", num, num);
}


void __dbg_dumpmem(char *src, unsigned long size)
{
  int i;

  __dbg_printk("Dumpmem at 0x%x :\n", src);
  
  for ( i = 0 ; i < size ; i++ )
    {
      char c = *(src++);
      
      if (c)
	__dbg_printk("%c (0x%x) | ", c, c);
      else
	__dbg_printk("\\0 | ");
    }

  __dbg_printk("End of Dumpmem\n");
}


#define PRH page_fault_prehan

void PRH (int numirq, addr_t eip, int error_code)
{
  /*
  addr_t cr2;

  asm ("movl %%cr2, %0"
       :"=r"(cr2) 
       : );

  page_fault_handler (eip, error_code, cr2);
  */

}

int start_me (kernel_parameter_t *kp)
{
  __dbg_printk("Hello\n");
  return 0;
}


__init_text int init_module(kernel_parameter_t *kp)
{
  __dbg_printk("Adresse page_fault_prehandler=0x%x (0x%x).\n",
		 PRH, *((unsigned*)PRH));

  return 0;
}


int cleanup_module(void)
{
  return 0;
}

EXPORT_FUNCTION(__dbg_dumpmem);
EXPORT_FUNCTION(__bochs_putchar);
EXPORT_FUNCTION(__dbg_printk);
EXPORT_FUNCTION(__bochs_putlong);
DECLARE_INIT_SYMBOL(init_module, INIT_LEVEL0);
DECLARE_CLEANUP_SYMBOL(cleanup_module, CLEANUP_LEVEL0);

EXPORT_FUNCTION(page_fault_prehan);
DECLARE_KERNEL_ENTRY(start_me);
