#include <tty/_tty.h>
#include <lib/string.h>
#include <lib/stdio.h>
#include <kos/spinlock.h>
#include <kos/asm.h>

static struct shadow_resource *ktty = NULL;


extern unsigned char *video;
extern struct interface tty_interface;


/* FIXME ! For a very strange reason, these variables cannot be
   initialized directly here, otherwise, we get a "init_ktty_lvl0
   causes a section type conflict" at compile time ! */

__init_data static int ypos;
__init_data static int xpos;
__init_data static int attribute;
static spinlock_t ktty_buf_lock;
static char ktty_buf[1024];

static void _ktty_scrollup_1 ()
{
  int i;

  for (i = 0; i < TTY_LINES - 1; i++)
    memcpy((void *) (video + i * TTY_COLS * 2),
	   (void *) (video + (i + 1) * TTY_COLS * 2),
	   TTY_COLS * 2);
  
  memset((void *) (video + i * TTY_COLS * 2), 0x0, TTY_COLS * 2);
}

static void _ktty_print_newline()
{
  xpos = 0;
  ypos++;
  if (ypos >= TTY_LINES)
    {
      _ktty_scrollup_1();
      ypos--;
    }
}

static void _ktty_putchar (int c)
{

  if (c == '\n' || c == '\r')
    {
      _ktty_print_newline();
      return;
    }

  *(video + (xpos + ypos * TTY_COLS) * 2)     = c & 0xFF;
  *(video + (xpos + ypos * TTY_COLS) * 2 + 1) = attribute;
  
  xpos++;
  
  if(xpos >= TTY_COLS)
    _ktty_print_newline();
}

int ktty_printk(const char *fmt, ...)
{
  va_list args;
  k_ui32_t flags;
  int i, len;
  
  write_spin_lock_no_debug(ktty_buf_lock, flags);
  va_start(args, fmt);
  len = vsnprintf(ktty_buf, sizeof(ktty_buf), fmt, args);

  /* If ktty isn't initialized to work with Babel, use the
     ktty_putchar function */
  if(ktty == NULL)
    {
      for (i = 0; i < len; i++)
	_ktty_putchar(ktty_buf[i]);
    }
  /* Initialization of ktty is complete --> use Babel ! */
  else
    {
      struct kernel_methods_table *kmt = 
	(struct kernel_methods_table *) 
	ktty->translator->interface->kernel_methods;

      kmt->write(ktty, ktty_buf, 0, len);
    }
  
  write_spin_unlock_no_debug(ktty_buf_lock, flags);

  return len;
}


/* __init_text */ int _tty_init_ktty_lvl0(void)
{
  /* clear screen */
  memset((void *) video, 0x0, TTY_LINES * TTY_COLS * 2);
  xpos = 0;
  ypos = 0;
  attribute = TTY_COLOR_WHITE;

  spinlock_init(ktty_buf_lock);

  /* put cursor at the top left of the screen */
  outb(0xE, 0x3D4);
  outb(0,   (0x3D4 + 1));
  outb(0xF, 0x3D4);
  outb(0,   (0x3D4 + 1));

  return 0;
}



__init_text struct shadow_resource * _tty_init_ktty_lvl1(void)
{
  /* Register console for kernel messages */
  if(bbl_add_translator(& tty_interface, "ktty", (void *) KERNEL_CONSOLE_ID) 
     == NULL)
    return NULL;
  
  /* Open this console */
  ktty = bbl_open_sr(NULL, "/dev/tty/ktty");

  if(ktty == NULL)
    return NULL;

#define TTY_VAR(var) (((struct tty_translator *) ktty->translator)->var)

  TTY_VAR(px) = xpos;
  TTY_VAR(py) = ypos;

  _tty_change_console(ktty);

  return ktty;
}
