#include <tty/_tty.h>
#include <kos/asm.h>
#include <arch/mm/mm.h>
#include <lib/string.h>
#include <klavier/klavier.h>

extern struct shadow_resource *ttys[MAX_CONSOLE_NB];
struct shadow_resource *current_tty;

#define TTY_VAR(var) (((struct tty_translator *) sr->translator)->var)
#define CURRENT_TTY_VAR(var) (((struct tty_translator *) current_tty->translator)->var)

int _tty_change_console(struct shadow_resource *sr)
{
  unsigned int addr;
  unsigned int pos;

  addr = (unsigned) TTY_VAR(phys_video_mem) - BASE_SCREEN_MEM;
  pos  = addr + (TTY_VAR(px) + TTY_VAR(py) * TTY_COLS) * 2;

  __dbg_printk("chgt de console pour %d (0x%x)\n", 
	       TTY_VAR(console_id),
	       TTY_VAR(phys_video_mem));

  current_tty = sr;

  /* change console */
  outb(0x0C, 0x3D4);
  outb( (addr >> 9), (0x3D4 + 1));
  outb(0x0D, 0x3D4);
  outb( (addr >> 1), (0x3D4 +1));
  
  /* update cursor */
  outb(0xE, 0x3D4);
  outb( (pos  >> 8), (0x3D4 + 1));
  outb(0xF, 0x3D4);
  outb( (pos & 0xFF), (0x3D4 + 1));

  return 0;
}

int _tty_in_handler(int ascii)
{
  int next;
  
  if(ascii >= ALTF1 && ascii <= ALTF8)
    {
      __dbg_printk("changement de console : %d (%d)\n",
		   ascii-ALTF1, ascii);
      _tty_change_console(ttys[ascii-ALTF1]);
      return 0;
    }

  __dbg_printk("Char recu : %c (%d) \n", ascii, ascii);

  next = (CURRENT_TTY_VAR(kbd_end) + 1) & (KBD_BUF_SIZE - 1);
  if(next == CURRENT_TTY_VAR(kbd_end))
    return -1;
  
  CURRENT_TTY_VAR(kbd_buf)[CURRENT_TTY_VAR(kbd_end)] = ascii;
  CURRENT_TTY_VAR(kbd_end) = next;
  kwaitqueue_wake_up(& CURRENT_TTY_VAR(waitqueue));

  return 0;
}


int tty_kernel_read(struct shadow_resource *sr, char *buf, 
		    offset_t offset, size_t len)
{
  int i = 0;
  UNUSED(sr);
  UNUSED(offset);

  while (len > 0)
    {
      if(TTY_VAR(kbd_begin) == TTY_VAR(kbd_end))
	{
	  __dbg_printk("ON SE BLOQUE\n");
	  kwaitqueue_add(& TTY_VAR(waitqueue));
	  __dbg_printk("ON A ETE DEBLOQUE\n");
	}
  
      buf[i] = TTY_VAR(kbd_buf)[TTY_VAR(kbd_begin)];

/*        DEBUG_PRINT("Bonjour de tty_kernel_read %d (%c)\n", len, buf[i]); */
      
      TTY_VAR(kbd_begin) = (TTY_VAR(kbd_begin) + 1) & (KBD_BUF_SIZE - 1);
      i++;
      len--;
    }

  __dbg_printk("ON RETOURNE\n");
  return 0;
}



static void _tty_newline(struct shadow_resource *sr)
{
  TTY_VAR(px) = 0;
  TTY_VAR(py)++;
  
  if(TTY_VAR(py) >= TTY_LINES)
    {
      /* Scrollup one line */
      memcpy((void *) (TTY_VAR(virt_video_mem)),
	     (void *) (TTY_VAR(virt_video_mem) + 2 * TTY_COLS),
	     (TTY_LINES - 1) * TTY_COLS * 2);
      
      memset((void *) (TTY_VAR(virt_video_mem) + ((TTY_LINES - 1) * TTY_COLS * 2)),
	     0x0,
	     TTY_COLS * 2);

      TTY_VAR(py)--;
    }
  
  return;
}


int tty_kernel_write(struct shadow_resource *sr, char *buf, 
		     offset_t offset, size_t len)
{
  unsigned int i;

/*    DEBUG_PRINT("Bonjour de tty_kernel_write (0x%x - %d - %d)\n", */
/*  	      buf, offset, len); */

  CONCEPTION_ASSERT(sr && buf);

  buf += offset;
  
  for(i = 0; i < len; i++)
    {
      if(buf[i] == '\n' || buf[i] == '\r')
	{
	  _tty_newline(sr);
	  continue;
	}
      
      
      *(TTY_VAR(virt_video_mem) + (TTY_VAR(px) + TTY_VAR(py) * TTY_COLS) * 2)     = buf[i] & 0xFF;
      *(TTY_VAR(virt_video_mem) + (TTY_VAR(px) + TTY_VAR(py) * TTY_COLS) * 2 + 1) = TTY_VAR(attribute);
      
      TTY_VAR(px)++;

      if(TTY_VAR(px) >= TTY_COLS)
	_tty_newline(sr);
    }

  return 0;
}

int tty_read(struct resource *res, char *buf, 
	     offset_t offset, size_t len)
{
  tty_kernel_read(res->shadow_resource, buf, offset, len);

  return 0;
}

int tty_write(struct resource *res, char *buf, 
	      offset_t offset, size_t len)
{
  ksem_wait( & ((struct tty_translator *) res->shadow_resource->translator)->sem );
  tty_kernel_write(res->shadow_resource, buf, offset, len);
  ksem_post( & ((struct tty_translator *) res->shadow_resource->translator)->sem );
  
  return 0;
}
