#ifndef ___TTY_H__
#define ___TTY_H__

#include <kos/types.h>
#include <babel/babel.h>
#include <tty/tty.h>
#include <kitc/ksem.h>
#include <kitc/kwaitqueue.h>

/* Specific TTY stuff */
#define MAX_CONSOLE_NB    8
#define KERNEL_CONSOLE_ID 0

#define TTY_COLOR_BLACK   0
#define TTY_COLOR_BLUE    1
#define TTY_COLOR_GREEN   2
#define TTY_COLOR_CYAN    3
#define TTY_COLOR_RED     4
#define TTY_COLOR_MAGENTA 5
#define TTY_COLOR_YELLOW  6
#define TTY_COLOR_WHITE   7

#define TTY_LINES 25
#define TTY_COLS  80

#define BASE_SCREEN_MEM   0xB8000

#define KBD_BUF_SIZE 32

typedef struct tty_methods_table 
{
  int (*read)(struct resource *,
	      char *buf, offset_t offset, size_t len);
  int (*write)(struct resource *,
	       char *buf, offset_t offset, size_t len);
} tty_methods_table_t;

typedef struct tty_translator 
{
  struct translator parent;

  unsigned int console_id;
  /* Pointer to physical memory associated */
  volatile unsigned char *phys_video_mem;
  /* Pointer to virtual memory associated */
  volatile unsigned char *virt_video_mem;
  /* Current position on the screen */
  int    px, py;
  int    attribute;
  ksem_t sem;
  kwaitqueue_t waitqueue;
  char   kbd_buf[KBD_BUF_SIZE];
  int    kbd_begin;
  int    kbd_end;
} tty_translator_t;

int _tty_init_ktty_lvl0(void);
struct shadow_resource *_tty_init_ktty_lvl1(void);
int _tty_in_handler(int ascii);
int _tty_change_console(struct shadow_resource *to_sr);


int tty_kernel_read(struct shadow_resource *, char *buf, 
		    offset_t offset, size_t len);
int tty_kernel_write(struct shadow_resource *, char *buf, 
		     offset_t offset, size_t len);
int tty_read(struct resource *, char *buf, 
	     offset_t offset, size_t len);
int tty_write(struct resource *, char *buf, 
	      offset_t offset, size_t len);

#endif

