#include <babel/babel.h>
#include <kos/types.h>
#include <tty/_tty.h>
#include <kmem/kmem.h>
#include <lib/string.h>

static int tty_constructor(struct translator *t, void *rc);
static int tty_destructor(struct translator *trans, void *rc);

struct kernel_methods_table tty_kmt =
{
  .read        = tty_kernel_read,
  .write       = tty_kernel_write,
  .init_sr     = NULL,
  .init_res    = NULL,
  .cleanup_sr  = NULL,
  .cleanup_res = NULL
};

struct tty_methods_table tty_mt = 
{
  .read  = tty_read,
  .write = tty_write
};

interface_t tty_interface =
{
  .name=        "tty",
  .family=      "tty",
  .description= "TTY driver",
  .authors=     "thomas petazzoni",
  .depends=     NULL,

  .version =   { 0 , 1 , 0 },
  .prev=       NULL,
  .next=       NULL,
  .trans_list= NULL,

  .max_trans_instance= MAX_CONSOLE_NB,
  .trans_size=         sizeof(struct tty_translator),
  .res_size=           0,
  .sr_size=            0,
  .trans_cnt=          0,
  .trans_greatest_id=  0,
  
  .constructor= tty_constructor,
  .destructor=  tty_destructor,
  
  .kernel_methods= (struct kernel_methods_table *)&tty_kmt,

  .methods_table= (struct interface_mt *)&tty_mt,
  .shared_data=   NULL 
};

extern unsigned char *video;

/* rc is the console number, between 0 and MAX_CONSOLE_NB */
static int tty_constructor(struct translator *t, void *rc)
{
  struct tty_translator *tty_trans = (struct tty_translator *) t;
  int console_id = (int) rc;

  CONCEPTION_ASSERT( (0 <= console_id) && (console_id <= MAX_CONSOLE_NB) );

  tty_trans->console_id = console_id;

  /* Special kernel console */
  if(console_id == KERNEL_CONSOLE_ID)
    {
      tty_trans->phys_video_mem = (volatile unsigned char *)
	BASE_SCREEN_MEM;
      tty_trans->virt_video_mem = (volatile unsigned char *) video;
    }
  else
    {
      tty_trans->phys_video_mem = (volatile unsigned char *) 
	BASE_SCREEN_MEM + (console_id * PAGE_SIZE);
      tty_trans->virt_video_mem = (volatile unsigned char *) 
	kvalloc(1, FALSE, FALSE);
      
      if(tty_trans->virt_video_mem == NULL)
	{
	  __dbg_printk("failure while alloc\n");
	  return -1;
	}
      
      map_virtual_page(NULL, 
		       (unsigned) tty_trans->virt_video_mem,
		       (unsigned) tty_trans->phys_video_mem,
		       VM_ACCESS_READ | VM_ACCESS_WRITE);
      
      /* Just generate a test message on each user console to see if
         it's working */
      tty_trans->virt_video_mem[148] = 'U';
      tty_trans->virt_video_mem[149] = TTY_COLOR_MAGENTA;
      tty_trans->virt_video_mem[150] = 'S';
      tty_trans->virt_video_mem[151] = TTY_COLOR_MAGENTA;
      tty_trans->virt_video_mem[152] = 'E';
      tty_trans->virt_video_mem[153] = TTY_COLOR_MAGENTA;
      tty_trans->virt_video_mem[154] = 'R';
      tty_trans->virt_video_mem[155] = TTY_COLOR_MAGENTA;
      tty_trans->virt_video_mem[156] = ' ';
      tty_trans->virt_video_mem[157] = TTY_COLOR_MAGENTA;
      tty_trans->virt_video_mem[158] = console_id + '0';
      tty_trans->virt_video_mem[159] = TTY_COLOR_MAGENTA;
    }

  if(ksem_create(& tty_trans->sem, 0, 1) != 0)
    return -1;

  if(kwaitqueue_init(& tty_trans->waitqueue) != 0)
    return -1;
  
  tty_trans->px = 0;
  tty_trans->py = 0;
  tty_trans->attribute = TTY_COLOR_WHITE;
  tty_trans->kbd_begin = 0;
  tty_trans->kbd_end   = 0;
  memset(tty_trans->kbd_buf, 0x0, KBD_BUF_SIZE);
  
  return 0;
}

static int tty_destructor(struct translator *t, void *rc)
{
  struct tty_translator *tty_trans = (struct tty_translator *) t;
  UNUSED(rc);
  
  kvfree((unsigned) tty_trans->virt_video_mem, TRUE);

  return 0;
}
