#include <loader/mod.h>
#include <lib/stdio.h>
#include <kos/macros.h>
#include <klavier/klavier.h>
#include <tty/_tty.h>

extern struct interface tty_interface;
unsigned char *video = 0x0;
struct shadow_resource *ttys[MAX_CONSOLE_NB];
extern struct shadow_resource *current_tty;

__init_text static int init_module_level0(kernel_parameter_t *kp)
{
  video = (unsigned char *) kp->video_memory;

  _tty_init_ktty_lvl0();

  return 0;
}


__init_text static int init_module_level1(kernel_parameter_t *kp)
{
  int i;

  UNUSED(kp);
  
  __dbg_printk("Running TTY initialization\n");
  printk("(Initializing TTY...");

  /* Registering tty interface */
  if(bbl_add_interface(&tty_interface) != 0)
    {
      printk("Failed)\n");
      return -1;
    }

  /* Initializing ktty translator and shadow resource */
  ttys[KERNEL_CONSOLE_ID] = _tty_init_ktty_lvl1();
  if(ttys[KERNEL_CONSOLE_ID] == NULL)
    {
      printk("Failed)\n");
      return -1;
    }
  
  current_tty = ttys[KERNEL_CONSOLE_ID];
  
  /* Initializing user tty translators */
  for (i = (KERNEL_CONSOLE_ID + 1); i < MAX_CONSOLE_NB; i++)
    {
      if(bbl_add_translator(& tty_interface, NULL, (void *) i) == NULL)
	{
	  printk("Failed)\n");
	  return -1;
	}
    }

  /* Initializing user tty shadow resources */
  for (i = (KERNEL_CONSOLE_ID + 1); i < MAX_CONSOLE_NB; i++)
    {
      char  buffer[32];
      snprintf(buffer, sizeof(buffer), "/dev/tty/tty%d", i-1);
      ttys[i]= bbl_open_sr(NULL, buffer);
      if(ttys[i] == NULL)
	{
	  printk("Failed)\n");
	  return -1;
	}
    }

  if(register_tty_in_handler(_tty_in_handler) != 0)
    {
      printk("Failed)\n");
      return -1;
    }
  
  printk("Ok)\n");

  return 0;
}

DECLARE_INIT_SYMBOL(init_module_level0, INIT_LEVEL0);
DECLARE_INIT_SYMBOL(init_module_level1, INIT_LEVEL1);
EXPORT_FUNCTION(ktty_printk);

