#ifndef ___VMM_H__
#define ___VMM_H__

#include <loader/mod.h>
#include <vmm/vmm.h>
#include <kos/macros.h>

/* ****************************************************
 * Team address space
 */

int as_add_vr(team_address_space_t *as,
	      virtual_region_t *vr);

__init_text int __init_as_engine(void);


/* ****************************************************
 * Virtual region
 */

/* allocates+duplicates a virtual region. the new virtual region is
   ready to be inserted in the new tree. */
virtual_region_t *vr_dup(virtual_region_t *vr);

/*
 * Page fault on vaddr
 * Returns 0 when could map vaddr
 * Otherwise -1 (=> must send SEGV to faulting thread)
 */
int vr_page_fault(virtual_region_t *vr,
		  vaddr_t vaddr,
		  pf_flags_t *pf);

int access_range_dump (access_range_t *access_range);
int vr_dump(virtual_region_t *vr);
int vr_dump_tree (virtual_region_t *vr_root);

__init_text int __init_vr_engine(void);

/* ****************************************************
 * Babelistification
 */
int __init_babel_kmem(void);
int __init_anon_master(void);

/* 
 * Seeks alone_on_pt list in order to find, if existing, a virtual
 * region which suits our needs (same offset in sr, and same size).
 */
virtual_region_t *kres_get_suitable_vr_if_exists (struct kres* kres,
						  offset_t offset, size_t size,
						  access_right_t rwx,
						  sharing_type_t sharing);

/* If already_know_if_alone_in_pt
 *   add vr to the list corresponding to vr->is_alone_on_pt
 * else
 *   determine on which list vr must be added
 */
int kres_add_mapping(struct kres *kres,
		     virtual_region_t *vr,
		     bool_t already_know_if_alone_in_pt);

int kres_remove_mapping(struct kres *kres,
			virtual_region_t *vr);


bool_t kres_map_resize_notification(struct kres *kres,
				    virtual_region_t *vr,
				    size_t new_size);

/*
 * Page fault on vaddr
 * Returns 0 when could map vaddr
 * Otherwise -1 (=> must send SEGV to faulting thread)
 */
int kres_page_fault(struct kres *kres,
		    virtual_region_t *vr,
		    vaddr_t vaddr,
		    access_range_t *ar,
		    pf_flags_t *pf);

__init_text int __init_kmem_resource (void);

#endif /* __vmm_h__ */




