#include <vmm/vmm.h>
#include <arch/mm/mm.h>
#include <liblist/liblist.h>
#include <pmm/pmm.h>
#include <kos/system.h>
#include <kos/assert.h>
#include <kos/errno.h>
#include <vmm/_vmm.h>
#include <karm/interface.h>
#include <karm/interface/mapping.h>
#include <karm/devfs.h>
#include <karm/open.h>

/* These functions are defined further */
static bool_t _kmem_kres_map(struct virtual_region* vr, vaddr_t vaddr,
			     access_range_t *ar, pf_flags_t *pf);
static size_t _kmem_kres_sync(struct virtual_region* vr, vaddr_t vaddr_start, 
			      size_t sz);

/* The mapping interface implementation */
struct mapping mapping_ops = 
  {
  _kmem_kres_map,
  _kmem_kres_sync
  };

struct view kmem_view = 
  {
    .iid = INTERFACE_MAPPING_ID,
    .nb_ops = 2,
    .ops = & mapping_ops
  };

static bool_t _kmem_kres_map(struct virtual_region* vr, vaddr_t vaddr,
			     access_range_t *ar, pf_flags_t *pf)
{
  UNUSED(pf);

  DEBUG_PRINT("kmem_kres_map(0x%x, 0x%x, %d)\n",
	      (unsigned)vr, vaddr, (unsigned)ar);
  
  FAILED_VERBOSE("Demand MUST BE impossible in kernel memory allocator");

  return TRUE;
}


static size_t _kmem_kres_sync(struct virtual_region* vr, vaddr_t vaddr_start, 
			      size_t sz)
{
  DEBUG_PRINT("kmem_kres_sync(0x%x, 0x%x, %d)\n",
	      (unsigned)vr, vaddr_start, sz);
  return -1;
}

/* This is the ures that covers the kernel space, and that forbids
   demand allocation */
struct ures *global_kmem_ures;

int init_team_as(struct team *team)
{
  struct team_address_space *as;
  struct virtual_region     *vr;

  ASSERT_RETURN_VAL(team, -1);

  as = & team->address_space;

  ASSERT_RETURN_VAL(! as_init(as, team), -1);

  /* Map kmem resource for kernel text+data */
  vr = as_map_kres(as, global_kmem_ures->kres,
		   0,
		   KERNEL_MEMORY_END-KERNEL_MEMORY_START,
		   KERNEL_MEMORY_START, TRUE,
		   MAP_SHARED,
		   VM_ACCESS_READ | VM_ACCESS_WRITE);
  ASSERT_RETURN_VAL(vr, -1);

  return 0;
}

__init_text int __init_kmem_resource (void)
{
  dev_t device;
  int result;

  devfs_register("mem", "kmem", 1, & kmem_view, & device);
  
  result = open("/dev/mem/kmem", INTERFACE_MAPPING_ID, & global_kmem_ures);

  if(result < 0) 
    {
      devfs_unregister(device);
      return result;
    }
  
  return 0;
}
