#include <arch/mm/mm.h>
#include <pmm/pmm.h>
#include <kos/assert.h>
#include "_vmm.h"

int map_virtual_page(struct team* dest_team,
		     addr_t virt, addr_t phys,
		     access_right_t access_rights)
{
  return arch_map_virtual_page((dest_team) ? (&dest_team->mm_context) : NULL,
			       virt, phys, access_rights);
}

int protect_virtual_page(struct team* dest_team,
			 vaddr_t vaddr,
			 access_right_t access_rights)
{
  return arch_protect_virtual_page((dest_team) ? (&dest_team->mm_context) : NULL,
				   vaddr, access_rights);
}


int unmap_virtual_page(struct team* dest_team,
		       vaddr_t vaddr)
{
  team_mm_context_t* mm_ctxt = (dest_team) ? (&dest_team->mm_context) : NULL;

  switch(arch_get_vpage_status(mm_ctxt, vaddr))
    {
    case PHYS_PAGE_PRESENT:
      /* unmap this page */
      arch_unmap_virtual_page(mm_ctxt, vaddr);
      break;

    case PHYS_PAGE_SWAPPED:
      TBI("To be done: unmap a swapped page");
      /* - When ref_cnt reaches 0, tell the swapper that this page
	   has become unused
	 - Unmap this vpage */
      arch_unmap_virtual_page(mm_ctxt, vaddr);
      break;

    case PHYS_PAGE_UNMAPPED:
      arch_unmap_virtual_page(mm_ctxt, vaddr);
      break;

    default:
      FAILED_VERBOSE("Invalid vpage status!");
    }

  return 0;
}

/*
 * For the current thread, unmap the virtual range from the current
 * address space.  No check is provided against accross-region
 * unmapping.  Unused PT's are not freed. This operation will be
 * provided by a garbage collector.
 */
int unmap_virtual_range(struct team* dest_team, range_t *range)
{
  int ret;
  vaddr_t page;

  ret = 0;
  for ( page = range->start ; page < range->end ; page += PAGE_SIZE)
    if (unmap_virtual_page(dest_team, page))
      ret = 1;

  return ret;
}
