#ifndef __KOS_ATOMIC_X86_H__
#define __KOS_ATOMIC_X86_H__

/* Largely inspired fron Linux i386/atomic.h */

/*
 * Atomic operations that C can't guarantee us.  Useful for
 * resource counting etc..
 */

#ifdef CONFIG_SMP
#define LOCK "lock ; "
#else
#define LOCK ""
#endif

/*
 * Make sure gcc doesn't try to be clever and move things around
 * on us. We need to use _exactly_ the address the user gave us,
 * not some alias that contains the same information.
 */
typedef struct { volatile int counter; } atomic_t;

#define ATOMIC_INIT(v,val)  ({ (v) = (atomic_t) { (val) }; })

/**
 * atomic_inc - increment atomic variable
 * @v: pointer of type atomic_t
 * 
 * Atomically increments @v by 1.  Note that the guaranteed
 * useful range of an atomic_t is only 24 bits.
 */ 
#define atomic_inc(/* atomic_t */ v) \
({                                   \
        __asm__ __volatile__(        \
                LOCK "incl %0"       \
                :"=m" ((v).counter)  \
                :"m" ((v).counter)); \
})

/**
 * atomic_dec - decrement atomic variable
 * @v: pointer of type atomic_t
 * 
 * Atomically decrements @v by 1.  Note that the guaranteed
 * useful range of an atomic_t is only 24 bits.
 */ 
#define atomic_dec(/* atomic_t */ v) \
({                                   \
        __asm__ __volatile__(        \
                LOCK "decl %0"       \
                :"=m" ((v).counter)  \
                :"m" ((v).counter)); \
})


#endif /* __KOS_ATOMIC_X86_H__ */
