/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Specific part of memory management for x86 architecture : init GDT.
 *
 * @(#) $Id: _gdt.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <kos/types.h>
#include <loader/mod.h>
#include <debug/debug.h>
#include <kos/assert.h>
#include <lib/stdio.h>
#include <lib/string.h>
#include "_mm.h"

struct {
  k_ui16_t size;
  addr_t   addr;
} __attribute__((packed)) gdtr;

typedef struct {
  unsigned long long segment_limit_low  : 16;
  unsigned long long base_addr_low      : 24;
  unsigned long long type               :  4;
  unsigned long long zero1              :  1;
  unsigned long long dpl                :  2;
  unsigned long long present            :  1;
  unsigned long long segment_limit_high :  4;
  unsigned long long available          :  1;
  unsigned long long zero2              :  2;
  unsigned long long granularity        :  1;
  unsigned long long base_addr_high     :  8;
} tss_seg_sel_t __attribute__((packed));

unsigned long long gdt[SEG_NB]  __attribute__ ((aligned (8)));


int arch_init_gdt_tss_entry(int seg_sel, tss_t *tss)
{
  tss_seg_sel_t tss_seg_sel;

  CONCEPTION_ASSERT((seg_sel == SEG_TSS) || (seg_sel == SEG_DOUBLE_FAULT_TSS));

  tss_seg_sel.segment_limit_low  = (sizeof(tss_t) - 1) & ~(1<<16);
  tss_seg_sel.base_addr_low      = (unsigned)tss & ~(1<<24);
  tss_seg_sel.type               = 1<<3 | 1;
  tss_seg_sel.zero1              = 0;
  tss_seg_sel.dpl                = 0;
  tss_seg_sel.present            = 1;
  tss_seg_sel.segment_limit_high = ((sizeof(tss_t) - 1) >> 16) & ~(1<<4);
  tss_seg_sel.available          = 0;
  tss_seg_sel.zero2              = 0;
  tss_seg_sel.granularity        = 0;
  tss_seg_sel.base_addr_high     = ((unsigned)tss >> 24) & ~(1<<8);

  memcpy(gdt+seg_sel, &tss_seg_sel, sizeof(tss_seg_sel_t));

  return 0;
}

void arch_dump_gdt(void)
{
  int i;

  printk("\nGDT Dump @0x%x\n", (unsigned)gdt);
  for (i = 0; i < SEG_NB; i++)
    {
      printk("Segment %d, LOW : 0x%x, HIGH : 0x%x\n",
	     i,
	     (unsigned)(gdt[i] & 0xFFFFFFFF),
	     (unsigned)((((unsigned long long)gdt[i]) >> 32) & ~((unsigned long long)1<<32)));
    }
}

__init_text int _init_gdt(void)
{
  gdtr.size = sizeof(gdt);
  gdtr.addr = (addr_t) &gdt;

  gdt[SEG_NULL_ID] = 0;

  gdt[SEG_CODE_KERNEL_ID]   = 0x00CF9B000000FFFF;
  gdt[SEG_DATA_KERNEL_ID]   = 0x00CF93000000FFFF;
  gdt[SEG_CODE_USER_ID]     = 0x00CFFB000000FFFF;
  gdt[SEG_DATA_USER_ID]     = 0x00CFF3000000FFFF;
  gdt[SEG_TSS]              = 0x0;
  gdt[SEG_DOUBLE_FAULT_TSS] = 0x0;
  		  
  asm volatile("lgdt %0":: "m" (gdtr));
  
  return 0;
}
