#ifndef ___KOS_MM_X86_H__
#define ___KOS_MM_X86_H__

#include <arch/mm/mm.h>
#include <arch/task/task.h>
#include <arch/internals.h>
#include <pmm/pmm.h>
#include <loader/mod.h>

/* Index of the PD for the very primitive primary first team */
#define PRIMARY_TEAM_PD_ID 0

#define PAGE_PRESENT         (1<<0)
#define PAGE_NOT_PRESENT     (0)
#define IS_PRESENT(p)        ((p) & PAGE_PRESENT)
#define PAGE_RW              (1<<1)
#define PAGE_RO              (0)
#define PAGE_USER            (1<<2)
#define PAGE_SUPERVISOR      (0)
/* The two following defines are not imposed by Intel PTE format, but
   we only chose to use bit 9 to mark swapped pages. */
#define PAGE_SWAPPED         (1<<9) 
#define PAGE_UNSWAPPED       (0)
#define PAGE_WT              0x08   /* 00001000b : Write through */
#define PAGE_CD              0x10   /* 00010000b : Cache disabled */

#define IS_SWAPPED(p)        ((p) & PAGE_SWAPPED)
#define IS_UNMAPPED(p)       ((!IS_PRESENT(p)) && (!IS_SWAPPED(p)))

#define IS_KERNEL_PAGE(virt) ((virt) < KERNEL_MEMORY_END)

#define va_to_pde(va)    ((va) >> 22)
#define va_to_pte(va)    (((va) >> 12) & 0x3ff)

struct rmap_session_s {
  struct mapping_s* page_rmap;
  struct mapping_s* pt_rmap;
};

struct _rmap_ops_s {
  int (*pre_add_rmapping)(struct rmap_session_s *rmap_session);
  int (*do_add_rmapping)(struct rmap_session_s *rmap_session,
			 k_ui32_t index_team, gpfme_t *gpfme, 
			 vaddr_t virt, bool_t is_pt);
  int (*post_add_rmapping)(struct rmap_session_s *rmap_session);
  
  int (*pre_del_rmapping)(struct rmap_session_s *rmap_session);
  int (*do_del_rmapping)(struct rmap_session_s *rmap_session,
			 k_ui32_t index_team, gpfme_t *gpfme, 
			 vaddr_t virt);
  int (*post_del_rmapping)(struct rmap_session_s *rmap_session);
};
extern struct _rmap_ops_s _rmap_ops;

/* Defined in _gdt.c */
int _init_gdt(void);

/* Defined in _team_mm_context.c */
int _init_pd_table_mapper(void);

/* in _pgflt.c */
cpu_state_t *_page_fault (int numirq,
			  cpu_state_t *cpu_state);

/* in _vmap.c */
page_entry_t _get_pte_unsafe(vaddr_t virt);

__init_text int _remap_init(void);

__init_text int _rmap_init(kernel_parameter_t*);

extern spinlock_t arch_remap_tmp_page;

#endif
