
#define DONT_CALL_UNSAFE_HALT_HANDLER

#include <kos/assert.h>
#include <kos/types.h>
#include <lib/stdio.h>
#include <arch/task/task.h>
#include "_mm.h"
#include <kos/asm.h>
#include <task/task.h>
#include <kos/spinlock.h>
#include <vmm/vmm.h>


cpu_state_t *_page_fault (int numirq, cpu_state_t *cpu_state)
{
  struct team_address_space *as;
  struct pf_flags pf_flags;
  addr_t cr2;

  static int pf_nested_level = 0;
  pf_nested_level++;
  if (pf_nested_level != 1)
    {
      bochs_printk(">>> ###FATAL### Nested #PF <<<\n");
      asm volatile("cli");
      for( ; ; ) asm volatile("hlt");
    }

  UNUSED(numirq);
  CONCEPTION_ASSERT(get_current_thread() && get_current_team());

  as = & get_current_team()->address_space;
  CONCEPTION_ASSERT(as);

  asm ("movl %%cr2, %0"
       :"=r"(cr2) 
       : );

  /*
   * Build pf_flags_t
   */
  pf_flags.page_present
    = (cpu_state->error_code & 0x1)?PF_PRESENT:PF_NOT_PRESENT;
  pf_flags.access
    = (cpu_state->error_code & 0x2)?VM_ACCESS_WRITE:VM_ACCESS_READ;
  pf_flags.who
    = (cpu_state->error_code & 0x4)?PF_ACCESS_USER:PF_ACCESS_SUPERVISOR;

  //  dump_cpu_state(cpu_state);

  if (as_page_fault(as, cr2, & pf_flags)) {
    char buf[200];

    __dbg_backtrace(get_current_thread(), cpu_state);

    addr2sym(cpu_state->eip, buf, sizeof(buf));
    DEBUG_PRINT("# Uncaught page fault at ip 0x%x (%s)\n",
		cpu_state->eip, buf);
    DEBUG_PRINT("#  on addr 0x%x, with error code 0x%x\n",
		cr2, cpu_state->error_code);
    DEBUG_PRINT("#  => [%s] / [%s] access / [%s] mode\n",
		(pf_flags.page_present == PF_PRESENT)?"Present":"Not present",
		(pf_flags.access == VM_ACCESS_WRITE)?"Write":"Read",
		(pf_flags.who == PF_ACCESS_USER)?"User":"Supervisor");

    FAILED_VERBOSE("*** TBI: Send SEGV to faulting thread ***\n");
  }

  pf_nested_level --;
  return cpu_state;
}


