/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Specific part of memory management for x86 architecture.
 *
 * @(#) $Id: mm.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <loader/mod.h>
#include <kos/spinlock.h>
#include <idt/exception.h>

#include "_mm.h"

SPINLOCK(kernel_space_lock);
SPINLOCK(kernel_team_pd_bitmap_lock);

__init_text static int init_module_level1(kernel_parameter_t *kp)
{
  UNUSED(kp);

  printk("(Init x86/mm: ");

  printk("[GDT");
  if (_init_gdt())
    return 1;
  printk("] ");

  printk("[#PF handler");
  if(register_exception_handler(PAGE_FAULT, _page_fault) == NULL)
    return 1;
  printk("] ");

  printk("[pd table mapper");
  if (_init_pd_table_mapper())
    return 1;
  printk("] ");

  printk("[remap init");
  if(_remap_init() != 0)
    return -1;
  printk("] ");

  printk("[rmap");
  if(_rmap_init(kp))
    return -1;
  printk("] ");

  printk("Ok)\n");

  return 0;
}

__init_text static int post_init_module_level0(kernel_parameter_t *kp)
{
  UNUSED(kp);
  
  return 0;
}

DECLARE_INIT_SYMBOL(init_module_level1, INIT_LEVEL1);
DECLARE_INIT_SYMBOL(post_init_module_level0, POST_INIT_LEVEL0);

EXPORT_FUNCTION(arch_map_virtual_page);
EXPORT_FUNCTION(arch_init_gdt_tss_entry);
EXPORT_FUNCTION(arch_dump_gdt);
EXPORT_FUNCTION(arch_get_paddr_at_vaddr);
EXPORT_FUNCTION(arch_unmap_virtual_page);
EXPORT_FUNCTION(arch_get_vpage_status);
EXPORT_FUNCTION(arch_team_mm_context_construct);
EXPORT_FUNCTION(arch_team_mm_context_destruct);
EXPORT_FUNCTION(arch_init_kteam_mm_context);
EXPORT_FUNCTION(arch_remap_virtual_page);
EXPORT_FUNCTION(arch_protect_virtual_page);
