#ifndef __KOS_MM_x86_H__
#define __KOS_MM_x86_H__

#include <kos/types.h>
#include <kos/spinlock.h>

#define PAGE_SIZE_SHIFT 12
#define PAGE_SIZE       (1 << PAGE_SIZE_SHIFT)
#define PAGE_ADDR_MASK  (~(PAGE_SIZE-1))
#define PAGE_FLAGS_MASK (PAGE_SIZE-1)

#define KERNEL_MEMORY_NUMBER_OF_PT 512
#define KERNEL_MEMORY_START PAGE_SIZE
#define KERNEL_MEMORY_END   (1UL*KERNEL_MEMORY_NUMBER_OF_PT*1024*PAGE_SIZE)

#define USER_MEMORY_START   KERNEL_MEMORY_END
#define USER_MEMORY_END     0xFFFFFFFF

/* Align on a 4k boundary, so that return value >= size */
#define PAGE_ALIGN_SUP(size) \
  ALIGN_SUP((size), PAGE_SIZE)

/* Align on a 4k boundary, so that return value <= addr */
#define PAGE_ALIGN_INF(addr) \
  ALIGN_INF((addr), PAGE_SIZE)

typedef enum { 
  VM_ACCESS_READ  = 0x1, 
  VM_ACCESS_WRITE = 0x2, 
  VM_ACCESS_EXEC  = 0x4
} access_right_t;

typedef k_ui32_t page_entry_t; // for pd/pt declarations

typedef struct {
  paddr_t    pd;
  k_ui16_t   pd_index_in_pd_table;
  spinlock_t user_space_lock;
} team_mm_context_t;

typedef struct mapping_s
{
  k_ui32_t index_team :10;
  k_ui32_t index_pde  :10;
  k_ui32_t index_pte  :10;
  k_ui32_t is_pt      :1;
  k_ui32_t unused     :1;
  struct mapping_s *next;
} mapping_t;

/* vpage AND ppage MUST be page-aligned. When dest_mm_ctxt is NULL,
   then always map in the current team. Function must always be
   operational, even when rmap is not yet initalised. */
int arch_map_virtual_page(team_mm_context_t* dest_mm_ctxt,
			  vaddr_t vpage, paddr_t ppage,
			  access_right_t access_rights);

/* Change the flags for the (mapped) vpage */
int arch_protect_virtual_page(team_mm_context_t* dest_mm_ctxt,
			      vaddr_t vpage,
			      access_right_t access_rights);

/* vpage AND new_ppage MUST be page-aligned. When dest_mm_ctxt is
   NULL, then always remap in the current team. vpage is unmapped from
   previous physical address, and remapped at new_ppage. The vpage may
   be the page of the current stack, or of the current IP. */
int arch_remap_virtual_page(team_mm_context_t* dest_mm_ctxt,
			    vaddr_t vpage, paddr_t new_ppage);

/* vpage MUST be page-aligned. When dest_mm_ctxt is NULL, then always
   unmap in the current team. */
int arch_unmap_virtual_page(team_mm_context_t* dest_mm_ctxt,
			    vaddr_t vpage);

typedef enum { PHYS_PAGE_PRESENT,
	       PHYS_PAGE_SWAPPED,
	       PHYS_PAGE_UNMAPPED } vpage_status_t;
vpage_status_t arch_get_vpage_status(team_mm_context_t* dest_mm_ctxt,
				     vaddr_t vaddr);

paddr_t arch_get_paddr_at_vaddr(addr_t virt_addr);

/* Team address space management: for VMM use */
struct team;
int arch_team_mm_context_construct(team_mm_context_t* mm_ctxt);
int arch_team_mm_context_destruct(team_mm_context_t* mm_ctxt);
int arch_init_kteam_mm_context(team_mm_context_t* mm_ctxt);

struct tss;
int arch_init_gdt_tss_entry(int seg_sel, struct tss *tss);
void arch_dump_gdt(void);

#endif /* __KOS_MM_x86_H__ */
