/*
 * Copyright (C) 2000, Thomas PETAZZONI & David DECOTIGNY
 * http://kos.enix.org
 *
 * Specific part of memory management for x86 architecture.
 * DOUBLE FAULT Management.
 *
 * @(#) $Id: _dbflt.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#define DONT_CALL_UNSAFE_HALT_HANDLER

#include <kos/types.h>
#include <kos/assert.h>
#include <kos/macros.h>
#include <kos/system.h>
#include <debug/debug.h>
#include "_task.h"

cpu_state_t *double_fault_handler(int numint, cpu_state_t *cpu_state)
{
  tss_t *system_tss;
  thread_t *current_thread;

  UNUSED(numint);
  UNUSED(cpu_state);
  
  __dbg_printk(_B_GREEN "[Stack management] FAULT OCCURED !\n" _B_NORM);

  current_thread = get_current_thread();
  system_tss = get_system_tss();

  dump_tss(current_thread->name, system_tss);

  /*
    |     |  <= Access made here : Bad !
    -------
    |     |  <= Physically mapped area (2 pages)
    |     |  <= 
    |-----|
    |     |  <= Not physically mapped (1 page). Access here => Stack Overflow
    |-----|
    |     |  <= Access made here : Bad !
  */

  if((system_tss->esp < (current_thread->cpl0_stack_base_addr + 
			CPL0_KERNEL_STACK_VIRTUAL_ADDR -
			 CPL0_KERNEL_STACK_PHYSICAL_SIZE))
     && 
     (system_tss->esp > (current_thread->cpl0_stack_base_addr)))
    {
      FAILED_VERBOSE("[Stack management] Thread \"%s\" used too much stack\n",
		     current_thread->name);
    }
  else if((system_tss->esp > (current_thread->cpl0_stack_base_addr +
			     CPL0_KERNEL_STACK_VIRTUAL_ADDR -
			     CPL0_KERNEL_STACK_PHYSICAL_SIZE)
	   &&
	   (system_tss->esp < (current_thread->cpl0_stack_base_addr +
			       CPL0_KERNEL_STACK_VIRTUAL_ADDR))))
    {
      FAILED_VERBOSE("[Stack management] Access was correct, but page wasn't correctly mapped (thread\"%s\") \n",
		     current_thread->name);
    }
  else
    {
      FAILED_VERBOSE("[Stack management] Thread \"%s\" made an very wrong access to the stack\n", current_thread->name);
      
    }

  return NULL;
}

