/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * Functions needed to create and delete user threads.
 *
 * @(#) $Id: _thread_cpu_context.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#include <kos/assert.h>
#include <kos/system.h>
#include <lib/string.h>
#include <pmm/pmm.h>
#include <kos/macros.h>

#include "_task.h"

void init_kernel_thread_context (thread_t *thread,
				 kernel_thread_fct_t *kt, 
				 void *data)
{
  cpu_state_t *new_cpu_state;

  /* we reserve 4 bytes to store the kernel thread parameter */
  thread->cpu_context.cpl0 = (cpu_state_t*)
    (thread->cpl0_stack_base_addr + 
     CPL0_KERNEL_STACK_VIRTUAL_SIZE - (sizeof(cpu_state_t) + 8));

  new_cpu_state = thread->cpu_context.cpl0;

  memset(new_cpu_state, 0x0, sizeof(cpu_state_t));
  new_cpu_state->eip    = (addr_t)kt;
  new_cpu_state->cs     = SEG_CODE_KERNEL_ID << 3;
  new_cpu_state->eflags = 514;
  new_cpu_state->ss     = SEG_DATA_KERNEL_ID << 3;
  new_cpu_state->ds     = SEG_DATA_KERNEL_ID << 3;
  new_cpu_state->es     = SEG_DATA_KERNEL_ID << 3;
  new_cpu_state->fs     = SEG_DATA_KERNEL_ID << 3;
  new_cpu_state->gs     = SEG_DATA_KERNEL_ID << 3;

  new_cpu_state->u.cpl0.return_addr   = (unsigned) thread_end_handler;
  new_cpu_state->u.cpl0.params        = (unsigned) data;
}

void init_user_thread_context (thread_t *thread, 
			       addr_t eip,
			       void *data)
{
  cpu_state_t *new_cpu_state;
  addr_t ppage;

  UNUSED(data);
  
  thread->cpu_context.cpl0 = (cpu_state_t*)
    ( thread->cpl0_stack_base_addr +
      CPL0_USER_STACK_VIRTUAL_SIZE - (sizeof(cpu_state_t) + 8));
  new_cpu_state = thread->cpu_context.cpl0;

  __dbg_printk(_B_RED "Addr of CPL0 Stack is 0x%x (base=0x%x)\n" _B_NORM, 
	       thread->cpu_context.cpl0,
	       thread->cpl0_stack_base_addr);
  
  memset(new_cpu_state, 0x0, sizeof(cpu_state_t));
  new_cpu_state->eip = eip;
  new_cpu_state->cs  = (SEG_CODE_USER_ID << 3) | 3;
  new_cpu_state->eflags = 514 | (1<<12) | (1<<13);

  /* Even if we have a CPL3 thread, this 'ss' must point to the data
     kernel segment. The real ss which is going to be used is defined
     below */
  new_cpu_state->ss     = SEG_DATA_KERNEL_ID << 3;
  new_cpu_state->ds     = (SEG_DATA_USER_ID << 3) | 3;
  new_cpu_state->es     = (SEG_DATA_USER_ID << 3) | 3;
  new_cpu_state->fs     = (SEG_DATA_USER_ID << 3) | 3;
  new_cpu_state->gs     = (SEG_DATA_USER_ID << 3) | 3;

  
  /* Address of the CPL3 stack */
  new_cpu_state->u.cpl3.esp3 = 
    (unsigned) (CPL3_HEAP_ADDR + CPL3_HEAP_SIZE - PAGE_SIZE - 4);

  /* Segment selector for the CPL3 stack (used when going to lower
     privilege) */
  new_cpu_state->u.cpl3.ss3 =
    (SEG_DATA_USER_ID << 3) | 3;
  /*
  new_cpu_state->u.cpl3.return_addr =
    (unsigned) user_thread_end_handler;

  new_cpu_state->u.cpl3.return_cs = 
    (SEG_CODE_KERNEL_ID << 3);
  */

  /*
  __dbg_printk(_B_RED "ESP3 = 0x%x (0x%x)\n" _B_NORM,
	       (unsigned)new_cpu_state->u.cpl3.esp3,
	       &new_cpu_state->u.cpl3.return_addr);
  */
  ppage = get_physical_page(PHYS_PAGE_USER, PHYS_PAGE_SWAPPABLE);
  CONCEPTION_ASSERT(ppage != 0);

  __dbg_printk(_B_RED "Mapping 0x%x=>0x%x (%d)\n" _B_NORM,
	       PAGE_ALIGN_INF(new_cpu_state->u.cpl3.esp3),
	       ppage, thread->mother_team->mm_context.pd_index_in_pd_table);
  
  arch_map_virtual_page(& thread->mother_team->mm_context,
			PAGE_ALIGN_INF(new_cpu_state->u.cpl3.esp3),
			ppage,
			VM_ACCESS_READ | VM_ACCESS_WRITE);

}
