/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * TSS (task state management) handling
 *
 * @(#) $Id: _tss.c,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */


#include <kmem/kmem.h>
#include <idt/exception.h>
#include <kos/assert.h>
#include <arch/mm/mm.h>
#include <kos/types.h>
#include "_task.h"
#include <lib/string.h>
#include <loader/mod.h>

tss_t *system_tss;
tss_t *double_fault_tss;


tss_t *get_system_tss(void)
{
  return system_tss;
}

void update_system_tss(addr_t stack_top)
{
  system_tss->esp0 = stack_top;
  ASSERT_FATAL(system_tss->ss0  == (SEG_DATA_KERNEL_ID << 3));
}

__init_text void __init_double_fault_tss(void)
{

  addr_t dbflt_stack;

  double_fault_tss = kmalloc(sizeof(tss_t));
  ASSERT_FATAL(double_fault_tss != NULL);

  //  printk("Double Fault TSS Addr 0x%x\n", double_fault_tss);

  memset(double_fault_tss, 0, sizeof(double_fault_tss));

  dbflt_stack = kvalloc(1, FALSE, TRUE);
  dbflt_stack += PAGE_SIZE-1;

  __dbg_printk(_B_RED "Stack allocated for double fault 0x%x\n" _B_NORM,
		 dbflt_stack);

  double_fault_tss->esp0 = dbflt_stack;
  double_fault_tss->ss0  = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->esp1 = dbflt_stack;
  double_fault_tss->ss1  = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->esp2 = dbflt_stack;
  double_fault_tss->ss2  = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->cr3  = CURRENT_PD_PADDR;
  double_fault_tss->eip  = (addr_t) exception_8_alias;
  /* keep the IF flag cleared + set the nested flag bit */
  double_fault_tss->eflags = 2 | (1<<14);
  /* we really need this in order to have a correct ESP ! */
  double_fault_tss->esp  = dbflt_stack;
  double_fault_tss->ebp  = 0;
  double_fault_tss->gs   = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->fs   = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->ds   = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->ss   = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->es   = SEG_DATA_KERNEL_ID<<3;
  double_fault_tss->cs   = SEG_CODE_KERNEL_ID<<3;
  
  arch_init_gdt_tss_entry(SEG_DOUBLE_FAULT_TSS, double_fault_tss);

  register_exception_handler(DOUBLE_FAULT, double_fault_handler);

}

__init_text void __init_system_tss(void)
{
  k_ui16_t seg_sel = SEG_TSS<<3;
  
  system_tss = kmalloc(sizeof(tss_t));
  ASSERT_FATAL(system_tss != NULL);

  //  printk("System TSS Addr 0x%x\n", system_tss);

  memset(system_tss, 0, sizeof(system_tss));

  /* we do not initialize the stack address, it will be done later by
     main, with update_system_tss_stack_addr function */

  system_tss->ss0    = SEG_DATA_KERNEL_ID<<3;
  system_tss->ss1    = SEG_DATA_KERNEL_ID<<3;
  system_tss->ss2    = SEG_DATA_KERNEL_ID<<3;
  system_tss->cr3    = CURRENT_PD_PADDR;
  system_tss->eip    = 0;
  system_tss->eflags = 514;
  system_tss->gs     = SEG_DATA_KERNEL_ID<<3;
  system_tss->fs     = SEG_DATA_KERNEL_ID<<3;
  system_tss->ds     = SEG_DATA_KERNEL_ID<<3;
  system_tss->ss     = SEG_DATA_KERNEL_ID<<3;
  system_tss->es     = SEG_DATA_KERNEL_ID<<3;
  system_tss->cs     = SEG_CODE_KERNEL_ID<<3;

  arch_init_gdt_tss_entry(SEG_TSS, system_tss);

  asm volatile("ltr %0"::"r"(seg_sel));
}
