/*
 * Copyright (C) 2000, Thomas PETAZZONI
 * http://kos.enix.org
 *
 * This file contains types definitions for tasking management on x86 architecture.
 *
 * @(#) $Id: task.h,v 1.1.1.1 2003/04/27 12:49:49 thomas Exp $
 */

#ifndef __TASK_X86_H__
#define __TASK_X86_H__

#include <kos/types.h>

/* This struct must be a multiple of 4 bytes long (in order for the
   pushl in the irq handler [idtasm.S] to be valid) */
typedef struct cpu_state_s {
  k_ui16_t gs;
  k_ui16_t fs;
  k_ui16_t es;
  k_ui16_t ds;
  k_ui16_t ss;
  k_ui16_t dont_use;
  k_ui32_t eax;
  k_ui32_t ebx;
  k_ui32_t ecx;
  k_ui32_t edx;
  k_ui32_t esi;
  k_ui32_t edi;
  k_ui32_t ebp;
  k_ui32_t error_code;
  addr_t   eip;
  k_ui32_t cs;
  k_ui32_t eflags;
  union 
  {
    struct 
    {
      k_ui32_t return_addr;
      k_ui32_t params;
    } cpl0;
    
    struct 
    {
      k_ui32_t esp3;
      k_ui32_t ss3;
      /*
      k_ui32_t return_addr;
      k_ui16_t return_cs;
      k_ui16_t unused;
      */
    } cpl3;
  } u;
} cpu_state_t __attribute__((packed));


typedef struct {
  cpu_state_t *cpl0;
  addr_t       cpl3;
} thread_cpu_context_t;

typedef struct tss {
  /* 0 */
  k_ui16_t back_link;
  k_ui16_t reserved1;
  /* 4 */
  addr_t esp0;
  /* 8 */
  k_ui16_t ss0;
  k_ui16_t reserved2;
  /* 12 */
  addr_t esp1;
  /* 16 */
  k_ui16_t ss1;
  k_ui16_t reserved3;
  /* 20 */
  addr_t esp2;
  /* 24 */
  k_ui16_t ss2;
  k_ui16_t reserved4;
  /* 28 */
  addr_t cr3;
  addr_t eip;
  k_ui32_t eflags;
  k_ui32_t eax;
  k_ui32_t ecx;
  k_ui32_t edx;
  k_ui32_t ebx;
  k_ui32_t esp;
  k_ui32_t ebp;
  k_ui32_t esi;
  k_ui32_t edi;
  /* 72 */
  k_ui16_t es;
  k_ui16_t reserved5;
  /* 76 */
  k_ui16_t cs;
  k_ui16_t reserved6;
  /* 80 */
  k_ui16_t ss;
  k_ui16_t reserved7;
  /* 84 */
  k_ui16_t ds;
  k_ui16_t reserved8;
  /* 88 */
  k_ui16_t fs;
  k_ui16_t reserved9;
  /* 92 */
  k_ui16_t gs;
  k_ui16_t reserved10;
  /* 96 */
  k_ui16_t ldtr;
  k_ui16_t reserved11;
  /* 100 */
  k_ui16_t debug_flag;
  k_ui16_t iomap_base_addr;
  /* 104 */
} __attribute__((packed)) tss_t;

#include <task/task.h>

void init_kernel_thread_context (thread_t *thread,
				 kernel_thread_fct_t *kt, 
				 void *data);
void init_user_thread_context (thread_t *thread, 
			       addr_t eip,
			       void *data);

void cpl0_switch_no_return (cpu_state_t *cpu_state);

void cpl0_switch_with_return (cpu_state_t **from_context,
			      cpu_state_t *to_context);

void update_system_tss(addr_t stack_top);

void dump_cpu_state(cpu_state_t *cpu_state);

#endif /* __TASK_X86_H__ */
